#   Farm-prog_common.tcl
#    this is the parts of evFarm-prog.tcl and spyFarm-prog.tcl which are identical

set lsmake_pid   -1
set selection     0
set filter      "*"
set prog_exec    ""
set prog_edit    ""
set file_select  ""
set dir_select   ""
set user_select  ""
set md_directory ""
set cf_file      ""
set rn_file      ""



#####################  FRAMES  #############################

proc make-frame {} {
   global env filter
   	
   :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name [self icon-title] :resize F

      :panel P1 :layout T
	 :layout  H
 	 :button  EDIT      :w 10  :command click-edit
	 :button  COMPILE   :w 10  :command click-compile
	 :button  INSTALL   :w 10  :command click-install
         :hskip 20
         :menubutton OPTIONS :menu [list [list File New Delete Rename] [list Directory Make Delete Rename]] \
              :command click-options
         :hskip 20
 	 :button  REDISPLAY :w 10  :command display
         :button  UP 	    :w 5   :command click-up
         :button  DOWN      :w 5   :command click-down
      
     :panel P2 :layout T :padx 10 :pady 10
        :layout H; :row-gap 5
        :text FILTER    :label "Filter : "   :w 20  :valuex 12c 
        :hskip 100
        :button NEOGEN  :label "NEO++ Code Generator" :command "inform neogen start"
        :next-row T
        :text DIRECTORY :label "Directory :" :w 80  :valuex 12c

      :panel P3 :layout T :padx 10 :pady 0
         :layout V
         :list DIRECTORYLIST  :rows 12 :cols 80 :exclusive T :double-click T
   	 bind [:path DIRECTORYLIST].list <ButtonRelease-2> "click-edit"
   	 bind [:path DIRECTORYLIST].list <ButtonRelease-3> "click-up"
            	 
      :panel P4 :layout T :padx 10 :pady 10
         :layout H; :col-gap 10  	 
         :text PROGEDIT :label "Program Editable : "   :w 24 :deaf T
         :text PROGEXEC :label "Program Executable : " :w 24 :deaf T       

      :panel-dialog LOAD 100 "click-load" 
      
   :show T
}

proc display-program {} {
   global directory prog_exec prog_edit Farm_workdir

   if {![file isdirectory $directory]} then {set directory $Farm_workdir}
    
   set tmp [file join $directory $prog_exec]
   if {[file isdirectory $tmp] || ![file executable $tmp]} then {
      :modify INSTALL :inactive T
      set prog_exec ""
   } else {
      :modify INSTALL :inactive F
   }    
   
   set tmp  [file join $directory $prog_edit] 
   if {[file isdirectory $tmp] || ![isfileC $prog_edit]} then {
      :modify COMPILE :inactive T
      set prog_edit ""
   } else {
      :modify COMPILE :inactive F
   }
   
   :modify DIRECTORY :value $directory
   :modify PROGEXEC  :value $prog_exec
   :modify PROGEDIT  :value $prog_edit
} 

proc mkf-mkdir {} {
   global env directory

   :set footer_position center
   :frame [self frame-name].mkdir :label "Make Directory" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name [self icon-title] :resize F 

      :panel MD_P :layout T :padx 20 :pady 20
         :layout V
         :text MD_PATH :label "Parent Directory Name : "  :w 45 :valuex 24c :value "$directory" :read-only T
         :text MD_DIR  :label "Directory Name : "         :w 30 :valuex 24c
       
      :panel-dialog MKDIR 70 "click-mkdir" 
    
   :show T
}

proc mkf-create {} {
   global env directory

   :set footer_position center
   :frame [self frame-name].create :label "Create File C/C++" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name [self icon-title] :resize F

      :panel CF_P :layout T :padx 10 :pady 10
         :layout V
         :text CF_PATH :label "Directory Name : " :w 45 :valuex 18c :value "$directory" :read-only T
         :text CF_FILE :label "File Name : "      :w 30 :valuex 18c
      
      :panel-dialog CREATE 50 "click-create" 

   :show T
}

proc mkf-rename {flag} {
   global env file_select dir_select

   set file_dir ""
   switch $flag {
      "f" {if {$file_select == ""} \
              then {set-footer "No file selected !!!"; return} \
              else {set file_dir $file_select}}
      "d" {if {$dir_select == ""} \
              then {set-footer "No directory selected !!!"; return} \
      	      else {set file_dir $dir_select}}
   }

   if {$file_dir == ""} then {set-footer "No file or directory selected !!!"; return}
 
   :set footer_position center
   :frame [self frame-name].rename :label "Rename File or Directory" \
      :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name [self icon-title] :resize F

      :panel CF_P :layout T :padx 40 :pady 20
         :layout V
         :text FILE    :label "Old Name : " :w 30 :valuex 11c :value "$file_dir"   :read-only T
         :text RN_FILE :label "New Name : " :w 30 :valuex 11c
      
      :panel-dialog RENAME 50 "click-rename $file_dir" 

   :show T
}
   

proc :panel-dialog {b size cmd} {
  
   set label [string toupper [string index $b 0]][string tolower [string range $b 1 end]]

   :panel $b :layout B :padx 10 :expand no :fill none
     set w [winfo parent [:path $b]]
       :layout H; :col-gap [expr 2 * $size]

       :hskip $size
       :button [set b]CANCEL :label  Cancel :w 6 :h 3         :command  "close-frame $w"
       :button [set b]SELECT :label  $label :w 6 :h 3 :hlt 2  :command $cmd
}
   

   
#####################  PROC  #############################

proc clear-globalvar {} {
   global prog_exec prog_edit file_select dir_select

   set prog_exec ""
   set prog_edit ""
   set file_select ""
   set dir_select ""
}   
  
proc display {} {
   global directory prog_exec prog_edit

   read-directory  $directory
   paint-directory $directory 
   display-program
}  


proc paint-directory {directory} {
   global directorylist filter

   if {$filter == ""} then {set filter "*"}

   set directories {}
   foreach target $directorylist {    
      if {[string match $filter [last $target]]} then {lappend directories $target}
   }
      
   :modify FILTER        :value "$filter"
   :modify DIRECTORY     :value "$directory"
   :modify DIRECTORYLIST :empty T :strings $directories :see 0

   set-footer "[llength $directorylist] files/directories"
}


proc delete-file {} {
   global directory file_select user_select
   
   if {$file_select == ""} then {set-footer "No file selected !!!"; return}

   if {$user_select != [whoami]} then {
      midas-warning "User [whoami]: permission denied."
      return
   }

   set q [midas-confirmation "The file \"$file_select\" will be deleted.\n\n\Do you wish to continue?"]
   if {$q} then {
      if {[catch {:file delete [file join $directory $file_select]} out]} then {
         set-footer $out
      } else {
         display
      }
   } 
}
     
proc delete-directory {} {
   global dir_select directory user_select
   
   if {$dir_select == ""} then {set-footer "No directory selected !!!"; return}

   if {$user_select != [whoami]} then {
      midas-warning "User [whoami]: permission denied."
      return
   }
      
   set q [midas-confirmation "The directory \"$dir_select\" and all files in it will be deleted.\
	   \n\n\Do you wish to continue?"]
   if {$q} then {
      if {[catch {:file delete -r [file join $directory $dir_select]} out]} then {
         set-footer $out
      } else {
         display
      }
   } 
}

proc create-file {file} {   

   if {[file exists $file]} then {return "The name [file tail $file] already exist !!!"}  
 
   if {![isfileC $file]} then {return "Cannot create $file: It is not a file C/C++ !!!"}

   if {[catch {open $file {RDWR CREAT}} id]} then {return "Cannot create $file: $id"}

   close $id
   return ""
}

proc rename-file {oldfile newfile } {
   global directory file_select
   
   if {$oldfile == ""} then {return "No file selected !!!"}

   if {[file exists $newfile]} then {return "Cannot rename \"$oldfile\": \"$newfile\" already exist !!!"}

   if {[isfileC $oldfile]} then {
      if {![isfileC $newfile]} then {
         return "Cannot rename \"$oldfile\": \"$newfile\" is not a file C/C++ !!!"
      }
   } else {
      if {[isfileC $newfile]} then {
         return "Cannot rename \"$oldfile\": \"$newfile\" is a file C/C++ !!!"
      }
   }

   if {[catch {:file rename $oldfile $newfile} out]} then {return $out} else {return ""}
}
 

#####################  CLICK  #############################

proc click arg {
   global debug md_directory cf_file rn_file filter selection

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {return [click-quit $f]}
   
   clear-footer
   switch $w {

      FILTER          {set filter $v; display}
      DIRECTORY       {click-directory $v}
      DIRECTORYLIST   {set selection $i; click-list $v}
      
      MD_DIR	      {set md_directory $v}      
      CF_FILE	      {set cf_file $v}
      RN_FILE	      {set rn_file $v}
     
      default         {puts stdout  $arg}
   }
}

proc double-click arg {
   global debug 

   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   switch $w {
      DIRECTORYLIST  {click-down}
      default        {puts stdout $arg}
   }
}

proc click-options {args} {

#  insert-debug "click-options with \{$args\}"

    switch [third $args] {
       0   {mkf-create}
       1   {delete-file}
       2   {mkf-rename f}
       3   {mkf-mkdir}
       4   {delete-directory}
       5   {mkf-rename d}
    }
}


proc click-directory {d} {
   global directory  prog_exec  env dir_select
   
   if {"$d" == "~"} then {set d $env(HOME)}

   if {[file isdirectory $d]} then {
      set directory $d
      set prog_exec ""
      set prog_edit ""
      set dir_select ""
      display
   } else {
      set-footer "$d is not a directory !!!"
      :modify DIRECTORY :value $directory
   }
}


proc click-list {target} {
   global directory prog_exec prog_edit file_select dir_select user_select
   
   set filemode    [first $target]
   set user_select [third $target]
   set dir_file    [last $target]

   if {[string index $filemode 0] == "l"} then {
#    caution - it is a symlink
      set dir_file [last_but_two $target]
   }

   if {![file isdirectory [file join $directory $dir_file]]} then {
       :modify DIRECTORY :value "$directory"
       if {![file executable [file join $directory $dir_file]]} then {
          if {[isfileC $dir_file]} then {
             set prog_edit $dir_file
             set prog_exec [file rootname $dir_file]
          } else {
             set-footer "$dir_file is not an executable file or editable C/C++ file !!!"
             set prog_edit ""
             set prog_exec ""
          }
       } else {
          set prog_exec $dir_file
          set prog_edit ""
       }
       set file_select $dir_file
       set dir_select ""
   } else {
      :modify DIRECTORY :value "[file join $directory $dir_file]"
      set dir_select $dir_file
      set file_select ""
      set prog_exec ""
      set prog_edit ""
   }
   display-program
}

proc click-up {} {
   global directory prog_exec 

   set directory [directory-up $directory]
   display
   clear-globalvar
}   

proc click-down {} {
   global directory selection prog_exec prog_edit

   set tmp [directory-down $directory $selection]
   if {$tmp != $directory} then {
      set directory $tmp
      set prog_exec ""  
      display
   }
   clear-globalvar
}

proc click-edit {} {
   global prog_edit directory file_select ERROR editor
   
   if {$file_select == ""} then {set-footer "No file selected"; return}

   set file [file join $directory $file_select] 

   if [file isdirectory $file] {
      set-footer "$file is a directory !!!"
   } else {
      if {[edit $file] == $ERROR} then {vi $file}
   }
}
   
proc click-load {} {
   global mytaskfirstname directory prog_exec

   set file  [file join $directory $prog_exec]

   if {[file isdirectory $file] || ![file executable $file]} then {
      set-footer "File selected is not an executable program !!!" 
      inform $mytaskfirstname deactive-start_unload
   } else {
      close-frame
      inform $mytaskfirstname update-load $directory $prog_exec
   }
}

proc click-mkdir {} {
   global  directory md_directory _frame 
   
   if {$md_directory == ""} then {set-footer $_frame "No directory entered !!!"; return}

   set m [:file mkdir [file join $directory $md_directory]]

   if {$m == ""} then {
      close-frame $_frame
      display
   } else {
      set-footer $_frame $m
   }

   set md_directory ""
}


proc click-create {} {
   global _frame cf_file directory
   
   if {$cf_file == ""} then {set-footer $_frame "No file entered !!!"; return}

   set m [create-file [file join $directory $cf_file]]

   if {$m == ""} then {
      close-frame $_frame
      display
   } else {
      set-footer $_frame $m
   }

   set cf_file ""
}

proc click-rename {file} {
   global _frame rn_file  prog_exec prog_edit directory 
 
   if {$rn_file == ""} {set-footer $_frame "No name entered !!!"; return}
    
   set m [rename-file [file join $directory $file] [file join $directory $rn_file]]

   if {$m != ""} then {set-footer $_frame $m; return}
   
   if {[isfileC $rn_file]} then {
      set oldfile [file rootname $file]
      set newfile [file rootname $rn_file]
      if [file isfile $oldfile] {set m [rename-file [file join $directory $oldfile] [file join $directory $newfile]]}
   }

   set rn_file ""
   set prog_exec ""
   set prog_edit ""

   if {$m == ""} then {
      close-frame $_frame
      display
   } else {
      set-footer $_frame $m
   }
}
   

proc click-compile {} {
   global env prog_exec prog_edit directory lsmake_pid
   global Farm_host
   
   if {![isfileC $prog_edit]} then {
      set-footer "No executable or editable file selected !!!"
      return
   }
   
   set lsmake  $env(LSFHOME)/bin/lsmake
   
   if {[progexist $lsmake]} then {
      midas-warning "Another program is making - wait for it to finish."
      return
   }

   if {[pidexist $lsmake_pid]} then {exec /bin/kill $lsmake_pid}
     
   if {$prog_exec == ""} then {
      set env(EVFARM_PROG) [get-execname $prog_edit]
   } else {
      set env(EVFARM_PROG) $prog_exec
   }
      
   if {![file isdirectory $directory]} then {set-footer "$directory is not a directory !!!"; return}

   set-busy-watch "Compiling program \"$prog_edit\" : Waiting..." 

   set work_dir [pwd]
   cd $directory 
   set lsmake_pid [lsmake $Farm_host]
   cd $work_dir  

   clear-busy-watch
}

proc click-install {} {
   global prog_exec directory ERROR
   global Farm_installdir

   if {![file executable $directory/$prog_exec]} then {
       set-footer "Program selected is not executable !!!"
       return
   }

   set hosts {}
   foreach item [read-Farm_conf] {
      if {$item != {}} then {lappend hosts [first $item]}
   }

   if {[bhosts_ok $hosts] == $ERROR} then {return $ERROR}

   set-busy-watch "Installing neo program \"$prog_exec\" in hosts $hosts : Waiting..."

   foreach host $hosts {
      set m [lsrcp $directory/$prog_exec $host $Farm_installdir/$prog_exec]
      if {$m == $ERROR} then {return}
   }
   clear-busy-watch
}

   
#####################  START  #############################

proc start args {
   global env started debug directory prog_exec

   if {$debug} {puts "args = $args"}

   set directory  [first  $args]
   set prog_exec  [second $args]
      
   init-environment
   init-parameters
   if {$started != 0} then {open-frame} else {make-frame; set started 1}
   display
}




