# Farm_common.tcl
#    this is the parts of evFarm.tcl and spyFarm.tcl which are identical


#####################  FRAMES  #############################

proc make-frame {} {
    global env started 
    global M1text

    set cols 110

    set title_list [format "   %-15s %-12s %-13s %-14s %-25s %-16s %-14s %s" \
       Name Status From_Host Exec_Host Starting_Time User Job_Status Job_Id ]

    :frame [self frame-name] :label "[self frame-title]" :show-footer T \
       :icon-image "@$env(ICONHOME)/default" \
       :icon-name "[self icon-title]" :resize F

       :panel ACTIONS1  :layout T  :padx 5 :pady 10
   	 :layout H; :col-gap 20
	 :button QUIT   :w 15
	 :button STAT   :label "Statistics..." :w 15 :command "xlsmon"
	 :hskip 80
	 :message TIME  :label "Last updated: [ttime]"
	 :button UPDATE :w 15

      :panel FARM :layout T :padx 5 :pady 5
         :layout V
           :message M1 :label "$M1text" 
           :galley FARM_LIST :rows 2 :cols $cols :deaf T 

      :panel ACTIONS2 :layout T :padx 5 :pady 5
	 :layout H; :col-gap 10
           :message M1 :label "NEO++ Programs" 
	   :button LOAD    :label "Load" :w 8
	   :button COMPILE :label "Compile" :w 8
           :button NEOGEN  :label "NEO++ Code Generator" 
	   :button START   :w 8 :inactive T
           :button TERM    :label "Terminate" :inactive T
	   :button UNLOAD  :w 8 :inactive T
   	   
      :panel PROCESSORS :layout T :padx 5 :pady 5
         :layout H; :col-gap 5
           :text PROG :label "Program   : " :w 60 :shifters T
           :hskip 10
  	   :text USER :label "User : " :w 10 :value "[whoami]"   :deaf T
           :text HOST :label "Host : " :w 10 :value "[hostname]" :deaf T

      :panel STATUS :layout T :padx 5 :pady 5
         :layout V; :row-gap 5
           :message M2 :label "Status of Jobs"
	   :message M3 :label $title_list
           :list STATUSJOBS :rows 12 :cols $cols :exclusive T :double-click T
   	   bind [:path STATUSJOBS].list <ButtonRelease-2> "click-job_info"
           :text JOB  :label "Job Selected : " :w 32 :deaf T

      :panel MESS :layout T :padx 5 :pady 5
         :layout H; :col-gap 20; :row-gap 5
           :message M4 :label "Messages" 
	   :button  CLEAR   :label "Clear Messages"
           :next-row T
	   :galley MESSAGES :rows 6 :cols $cols :deaf T 

    :show T
}



#####################  PROC  #############################


proc display-Farm {} {
   global  Farm_data 
   
   set Farm_data [read-Farm_conf]

   set ll ""
   foreach item $Farm_data {
      if {$item != {}} {append ll "[first $item] ([second $item])    "}
   }

   :modify FARM_LIST :empty T :strings  $ll
}

proc display-status_jobs {} {
    global OK ERROR Farm_queue

    if {![bping]} then {return $ERROR}
    
    set-footer "Read Status of Jobs..."
    set info [read-status_jobs]
    if {$info == $ERROR} then {
        set status [list "No unfinished job found in queue <online>"]
    } else {
        set status $info
    }
    clear-footer
    :modify STATUSJOBS :empty T :strings $status
    :modify TIME       :label "Updated at [ttime]"
    return $info
}

proc read-status_jobs {} {
    global OK ERROR Farm_queue
    set out [bjobs $Farm_queue]
    if {$out == $ERROR} then {
        set status $ERROR
    } else {
        set status {}
        foreach item $out {
           set l ""
           foreach f [split $item \t] {append l $f}
           lappend status $l
        }
    }
    return $status
}

proc display-program {} {
   global workdir prog_name job_name

   if {$job_name == ""} then {deactive-start_unload}
   if {$prog_name == "" || $workdir == ""} then {
      set prog ""
   } else {
      set prog $workdir/$prog_name
   }
      
   :modify PROG :value $prog
   :modify JOB  :value $job_name
}

proc load-Processors {} {
   global workdir job_name prog_name ERROR OK Farm_data Farm_queue

   if {$workdir == "" || $job_name == "" || ![bping]} then {return $ERROR}

   set-busy-watch "Loading neo program \"$prog_name\" into all Farm Processors : Waiting..."	      
   set names [bjobs_names $Farm_queue]
   foreach name $names {
	if {$name == $job_name} then {
	   midas-warning "Unable to load the program \"$prog_name\" : this program already loaded."
           clear-busy-watch
	   return $ERROR
	}
   }
 
   set data {}
   set hosts {}
   foreach item $Farm_data {
      if {$item != {}} then {
	 lappend data  [list [first  $item] [second $item]]
	 lappend hosts [first $item]
      }
   }
   if {[bhosts_ok $hosts] == $ERROR} then {clear-busy-watch-error; return $ERROR}

   set out {}
   foreach item $data {
     set host [first $item]
     append out " [bsub-Farm $host $prog_name 1]"
     if {$out == $ERROR} then {clear-busy-watch-error; return $ERROR}
     if {[second $item] == 2} then {
	 append out " [bsub-Farm $host $job_name 2]"
     }
   }
      
   write-Farm_neo $workdir $prog_name "load"
   print-messages $out
   display-status_jobs

   clear-busy-watch
   
   inform sys click-redisplay

   return "Program $prog_name loaded into the Processor Farm"
}

proc active-start_unload {} {
   :modify START   :inactive F
   :modify TERM    :inactive F
   :modify UNLOAD  :inactive F
}

proc deactive-start_unload {} {
   :modify START   :inactive T
   :modify TERM    :inactive T
   :modify UNLOAD  :inactive T
}
   
#---------- Proc called extern modules -------------------------

proc update-load {dir prog} {
   global workdir prog_name job_name ERROR

   clear-footer
   set workdir      $dir
   set prog_name    $prog
   set job_name     [get-jobname $prog_name]
   display-program
      
   if {[load-Processors] == $ERROR} then {
      deactive-start_unload
   } else {
      display-status_jobs
      active-start_unload
   } 
}


#####################  CLICK  #############################

proc click {arg} {
   global debug line_index

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit $f; return}

   clear-footer
   set line_index $i
   switch  $w {
      QUIT 	  {close-frame $f}
      UPDATE 	  {click-update}

      PROG   	  {click-program $v}
      STATUSJOBS  {click-list $v}
      LOAD 	  {click-load}
      COMPILE     {click-compile}
      NEOGEN      {click-neogen}
      START 	  {click-start}
      TERM        {click-term}
      UNLOAD 	  {click-unload}

      CLEAR 	  {:modify MESSAGES :empty T}

      default     {puts stdout $arg}
   }
}

proc double-click arg {
   global debug 

   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   switch $w {
      STATUSJOBS  {click-bpeek $v}
      default     {puts stdout $arg}
   }
}
   
proc click-program {prog} {
   global workdir prog_name job_name line_index ERROR

   set job_name ""
   :modify STATUSJOBS :deselect $line_index
   :modify JOB        :value $job_name
  
   if {![file isdirectory $prog] && [file executable $prog]} then {
       set workdir   [file dirname $prog]       
       set prog_name [file tail    $prog]
       set job_name  [get-jobname  $prog_name]
       display-program
       if {[set m [load-Processors]] == $ERROR} then {
          deactive-start_unload
       } else {
          print-messages $m
          display-status_jobs
          active-start_unload
       }   
   } else {
       set-footer "The program \"$prog\" is not executable !!!"
   }
}
  
proc click-load {} {
   global  mytaskfirstname job_name line_index workdir prog_name

   set job_name ""
   :modify STATUSJOBS :deselect $line_index
   :modify JOB        :value $job_name   
   deactive-start_unload
   inform $mytaskfirstname-prog start $workdir $prog_name
}

proc click-compile {} {
   global  mytaskfirstname workdir prog_name

   inform $mytaskfirstname-prog start $workdir $prog_name
}

proc click-neogen {} {inform neogen start}

proc click-start  {} {start-Processors}
proc click-term   {} {stop-Processors}
proc click-unload {} {unload-Processors; inform sys click-redisplay}

proc click-list {l} {
   global job_name job_id

   if {[llength $l] == 0} then {set job_id 0; return}
   if {[scan [last $l] {%[0-9]} integer]} then {
      set job_id $integer
   } else {
      set job_id 0
   }
   if {$job_id} then {
      set job_name [first $l]
      display-program
      active-start_unload
   }
}

proc click-bpeek {l} {
   global job_id ERROR
   set elements 1000
   
   if {$job_id <= 0 || ![bping]} then {return}
   
   set-busy-watch "Reading the stdout and stderr outputs of job \"$job_id\" : Waiting..."
   set info [bpeek $job_id $elements]
   clear-busy-watch
   if {$info == $ERROR} then {return} 

#   midas-information "[join $info \n]"
      midas-information $info
}

proc click-job_info {} {
   global job_id ERROR

   if {$job_id <= 0 || ![bping]} then {return}
      
   set-busy-watch "Read information about job \"$job_id\" : Waiting..."
   set info [bjobs_info $job_id]
   clear-busy-watch

   midas-information  "[join $info \n]"
}

proc click-update {} {
   read-Farm_neo
   display-program
   display-Farm
   display-status_jobs
}


#####################  LOAD  #############################

proc start {} {
   global started OK ERROR Farm_queue
   
   init-environment
   init-parameters
   if {$started != 0} then {
       open-frame
   } else {
       make-frame
       set started 1
   }
   read-Farm_neo
   display-program
   display-Farm
   display-status_jobs
   if {[bping]} then {
      if {[bjobs_run $Farm_queue] == $OK} then {active-start_unload}
   }
}


