# beSetup-ef.tcl---Back End Setup Event Processor Farm frame
#    Note - this file is almost identical to ebSetup-sf.tcl
#    any changes made here are probably also needed there
 
loadsource common.lnl
loadsource common.orbix
loadsource common.$mytaskfirstname

set icontitle 	   "efSetup"
set label "Event Processors"

set Farm_conf ""
set Farm_line_select {}
set Farm_list {}

set open_frame_mod 0
set install_needed 0

set host ""
set host_hold ""
set proc_nbr 1

set hosts_allocate_list {}
set hosts_available_list {}


#####################  FRAMES  #############################

proc make-frame {} {
   global env edit
   global line_index Farm_conf

   if {$edit} {
      set option_list F; set inactive F
   } else {
      set option_list T; set inactive T
      set line_index -1
   }

   :frame [self frame-name]  :label [self frame-title] :show-footer T \
        :icon-image "@$env(ICONHOME)/euroball" \
        :icon-name [self icon-title] :resize F

        :panel BEGIN :layout T :padx 10 :pady 5
         :layout H
           :button QUIT :w 15

        :panel TITLE :layout T :padx 10 :pady 5
   	   :layout H
           :message TTL_1 :label "Event Processors Configuration File :" 
           :message TTL_2 :label "$Farm_conf"
 
        :panel DATA :layout T :padx 70 :pady 0 
   	   :layout V
	     :message MESS :label "Host Name \tEnabled Processors"
           :list LIST :rows 5 :cols 48 :exclusive T :deaf $option_list

        :panel ACTIONS :layout T :padx 70 :pady 5
   	   :layout H
	     :button ADD    :label "Add..."    :w 10 :inactive $inactive
	     :button MODIFY :label "Modify..." :w 10 :inactive $inactive
	     :button DELETE :label "Delete..." :w 10 :inactive $inactive

   :show T
}


proc make-frame-modify {} {
   global host hosts_available_list proc_nbr
   global modify add Farm_line_select
   global label env open_frame_mod

   set title ""
   set lab ""

   if {$modify} then {
      set title "$label : Modify"
      set lab "APPLY"
      click-list $Farm_line_select
      select-line
   }
   if {$add} then {
      set title "$label : Add"
      set lab "ADD"
      set host ""
      set proc_nbr 1
      deselect-line
   }

   :set footer_position center
   :frame [self frame-name].mod :label $title :show-footer T \
        :icon-image "@$env(ICONHOME)/euroball" \
	  :icon-name [self icon-title] :resize F

      :panel MOD_PARAM :layout T :padx 50 :pady 20 
     	 :layout H; :row-gap 5
   	 :menubutton HOSTMENU  :label Host :menu $hosts_available_list  :w 10 
  	 :text HOST :label "" :w 10  :value $host :deaf T 
         :next-row T
   	 :number PROCNBR :label "Processors enabled  "  :w 2  :min 1 :max 2 :value $proc_nbr
 
       :panel-dialog 40 CANCEL "click-mod-cancel" $lab "click-mod-add" 
       
   :show T

   set open_frame_mod 1
}


#####################  PROC  #############################

#-------------------------  parameters ---------------------------------

proc init-parameters {} {
    global Farm_conf env edit frametitle bin

    if {$edit} then {
       set frametitle "Event Processors Configuration Tool (Edit)"
    } else {
       set frametitle "Event Processors Configuration Tool (Display)"
    }

    set Farm_conf [file join $env(EB_ETC) evFarm.conf]
    init-hosts
    init-procnbr
}


proc init-hosts {} {
    global hosts_available_list hosts_list

    set hosts_available_list $hosts_list
}

proc init-procnbr {} {global proc_nbr; set proc_nbr 1}

proc display {} {
    global host proc_nbr

    :modify HOST     :value " $host"
    :modify PROCNBR  :value $proc_nbr
}



#-------------------------  Event Farm ---------------------------------

proc write-Farm_conf {fn } {
    global  Farm_list

    set header { {# Definition of Event Processors Farm}
	         {#}
	         {# Host Name 	  Number of Processors Enabled}
	         {#} }

    set data [concat $header $Farm_list]
    write-file $fn $data
    distribution $fn
}

proc display-Farm_conf {data} {
   global Farm_list Farm_line_select line_index edit

   set Farm_list {}
   foreach r $data {lappend Farm_list [get-line $r]}

   if {[llength $Farm_list] != 0 } {
      set line_index 0
      select-line
      if {!$edit} {deselect-line}
   } 
}

proc get-line {ps} {
   global  hosts_allocate_list hosts_available_list

   set host        [first $ps]
   set proc_nbr [second $ps]

   lappend hosts_allocate_list  $host

   set hosts_available_list [ldelete $hosts_available_list $host]

   return [format-line $host $proc_nbr]
}

proc format-line {h n} {

   format " %-15s %d" $h $n
}

proc select-line {} {
   global line_index Farm_list Farm_line_select

   :modify LIST :empty T :append T :strings $Farm_list \
				   :select $line_index

   set Farm_line_select [lindex $Farm_list $line_index]
}

proc deselect-line {} {
   global line_index Farm_list

   :modify LIST :empty T :append T :strings $Farm_list \
				   :deselect $line_index
}

proc modify-conf {} {
   global Farm_line_select line_index Farm_list
   global host host_hold proc_nbr
   global hosts_allocate_list hosts_available_list

   set r [format-line $host $proc_nbr]

   if {$line_index != -1} {
      if {[llength $Farm_list] >= 0} {
         set ll [lreplace $Farm_list $line_index $line_index $r]
         set Farm_list $ll
         select-line

         set i [lsearch -exact $hosts_allocate_list $host_hold]
         if {$i >= 0} then {
	       set hosts_allocate_list [lreplace $hosts_allocate_list $i $i $host]
	   }
         set i [lsearch -exact $hosts_available_list $host]
         if {$i >= 0} then {
	       set hosts_available_list [lreplace $hosts_available_list $i $i $host_hold]
	   }
      }
   }

}

proc add-conf {} {
   global  host line_index Farm_list proc_nbr
   global  hosts_allocate_list hosts_available_list

   set r [format-line $host $proc_nbr]
   lappend Farm_list [get-line $r]
   set line_index [expr [llength $Farm_list] - 1]
   select-line

   set hosts_available_list [ldelete $hosts_available_list $host]
   lappend hosts_allocate_list  $host
}


#####################  CLICK  #############################

proc click {arg} {
   global debug 

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   if {$c==":quit"} then {click-quit $f; return}

   switch  $w {
      LIST        	{click-list $v}
      MODIFY 		{click-modify}
      ADD 	      	{click-add}
      DELETE 		{click-delete}
      HOSTMENU 		-
      HOST 		      {click-host    $v}
      PROCNBR		{click-procnbr $v}

      QUIT 		      {click-quit $f}

      default 		{puts stdout $arg}
      }
   }

proc click-quit {f} {
    global open_frame_mod mytaskfirstname

    switch -glob $f {
      *.MOD {set open_frame_mod 0; close-frame $f}
      default {
          if {$open_frame_mod} then {
             set open_frame_mod 0
             close-frame [self frame-name].mod
          }
          close-frame $f
      }
    }
    clear-footer
    set open_frame_mod 0
    inform $mytaskfirstname quit-evFarm
}


#------------------------- Event Farm --------------------------------

proc click-list {e} {
   global host 
   global Farm_line_select line_index Farm_list
   global open_frame_mod add label proc_nbr

   if {$add} {deselect-line; return}

   set Farm_line_select $e
   set line_index [lsearch -exact $Farm_list $e]

   set line [expr $line_index + 1]
   set-footer "$label: line $line selected..."

   set host        [first  $e]
   set proc_nbr [second $e]

   if {$open_frame_mod} {display}
}


proc click-modify {} {
   global  add modify label Farm_list

   if {$Farm_list == {}} {return}
   set modify 1; set add 0
   set-footer "$label: modify..."
   make-frame-modify
}


proc click-add {} {
   global  add modify label

   set-footer "$label: add line..."
   set add 1; set modify 0
   make-frame-modify
}

proc click-delete {} {
   global  Farm_list line_index Farm_line_select
   global  hosts_allocate_list hosts_available_list
   global  host label frame_name install_needed
   global  Farm_conf add mytaskfirstname

   if {$Farm_list == {}} {return}
   set add 0

   click-list $Farm_line_select
   set line [expr $line_index + 1]

   set i [lsearch -exact $Farm_list $Farm_line_select]
   if {$i >= 0} {
      set ll [lreplace $Farm_list $i $i]
      set Farm_list $ll
      set line_index 0
      select-line

      set hosts_allocate_list [ldelete $hosts_allocate_list $host]
      lappend hosts_available_list  $host

      inform $mytaskfirstname update-evFarm_conf $Farm_list
      set install_needed 1
      set-footer "$label: install (needed).."
   }
}

proc click-host {h} {
   global host host_hold install_needed

   clear-footer [self frame-name].mod
   set host_hold $host
   set host $h
   :modify HOST :value " $host"
   set install_needed 1
}

proc click-procnbr {n} {
   global proc_nbr install_needed

   clear-footer [self frame-name].mod
   set proc_nbr $n
   :modify PROCNBR :value $n
   set install_needed 1
}

proc click-mod-add {} {
   global  host install_needed Farm_list mytaskfirstname
   global  modify add label open_frame_mod

   if {$install_needed} then {
      set-footer "$label: install (needed)..."
      set z [llength $host]
      if {$z == 0} then {
         set-footer [self frame-name].mod "Choice parameters in the menu list !!!"
         set-footer "$label: Error !!!" 
         display-error "Entry invalid parameter : null value"
         clear-footer
         set open_frame_mod 0
         return
      }
      if {$modify} {modify-conf }
      if {$add} {add-conf; select-line}
   } else {
      select-line
   }

   set add 0; set modify 0; set open_frame_mod 0
   inform $mytaskfirstname update-evFarm_conf $Farm_list
   close-frame [self frame-name].mod
}
 
proc click-mod-cancel {} {
   global  open_frame_mod install_needed
   global  add modify

   set install_needed 0

   select-line
   set open_frame_mod 0
   set add 0
   set modify 0
   close-frame [self frame-name].mod
}


#----------------- proc installation -------------------------------

proc install {} {
   global  Farm_conf label install_needed mytaskfirstname

   if {$install_needed == 0} {return 1}
 
   set i [midas-confirmation \
       "$label - Installation\n\n\
       This operation will \n\
       MODIFY the configuration files\n\
       Do you wish to continue?"]

   if {$i} then {
       write-Farm_conf $Farm_conf
   } else {
       inform $mytaskfirstname cancel-evFarm_conf
   }
  
   set install_needed 0
   clear-footer
   return 0
}



#####################  START  #############################
proc start args {
   global started edit install_needed
   
   set edit   	[first  $args]
   set cancel 	[second $args]
   set data   	[third  $args]
    
   if {$cancel} {
      set install_needed 0
      clear-footer
   }
#   init-environment  -  done in common.beSetup
   init-parameters
   if {$started != 0} then {
       open-frame
   } else {
       make-frame
       set started 1
   }
   centre-frame
   display-Farm_conf $data
}
