# beSetup-lg.tcl---Back End Setup Logger frame
 
loadsource common.lnl
loadsource common.orbix
loadsource common.$mytaskfirstname

set icontitle  	   "lgSetup"
set label "Logger"

set log_line_select {}
set log_list      {}

set open_frame_mod   0
set install_needed 0

set log_conf ""; set log_cf      ""
set host     ""; set host_hold   ""
set server   ""; set server_hold ""
set bin      ""

set hosts_allocate_list     {}; set hosts_available_list  {};
set servers_persistent_list {}; set servers_allocate_list {};
set servers_available_list  {};


#####################  FRAMES  #############################

proc make-frame {} {
   global env edit log_conf

   if {$edit} then {set option_list F; set inactive F}\
              else {set option_list T; set inactive T}

   :frame [self frame-name]  :label [self frame-title] \
	:show-footer T \
      :icon-image "@$env(ICONHOME)/euroball" \
	:icon-name [self icon-title] :resize F

        :panel BEGIN :layout T
          :layout H
           :button QUIT :w 15

        :panel TITLE :layout T :padx 10 :pady 10 
          :layout H
           :message TTL_1 :label "Logger Configuration File :"
           :message TTL_2 :label "$log_conf"
 
        :panel DATA :layout T :padx 45 :pady 0 
   	    :layout V
	     :message MESS :label "Host Name \tServer Name"
           :list LIST :rows 2 :cols 40 :exclusive T :deaf $option_list

        :panel ACTIONS :layout T :padx 45 :pady 5
         :layout H
	   :button MODIFY :label "Modify..." :w 10 :inactive $inactive

   :show T
}


proc make-frame-modify {} {
   global host server open_frame_mod
   global hosts_list servers_available_list
   global label env log_line_select

   click-list $log_line_select
   select-line

   :frame [self frame-name].mod :label "$label : Modify" \
	:show-footer T \
      :icon-image "@$env(ICONHOME)/euroball" \
	:icon-name [self icon-title] :resize F

        :panel MOD_PARAM :layout T :padx 50 :pady 20 
     	   :layout V  
   	   :choice-stack HOST :label Host :strings $hosts_list  :value [lsearch $hosts_list $host]
	   :row-gap 8
  	   :menubutton SERVERMENU :label Server :menu $servers_available_list :w 10
 	   :layout H
 	   :text SERVER :label "" :w 15  :value "$server"

        :panel-dialog 40 CANCEL "click-mod-cancel" APPLY "click-mod-add" 

   :show T

   set open_frame_mod 1
}



#####################  PROC  #############################

#-------------------------  parameters -----------------------------------

proc init-parameters {} {
    global env log_conf log_cf edit frametitle bin

    if {$edit} then {
       set frametitle "Logger Configuration Tool (Edit)"
    } else {
       set frametitle "Logger Configuration Tool (Display)"
    }

    set bin      "$env(EB_BIN)/log_server"
    set log_conf "$env(EB_ETC)/log.conf"
    set log_cf   "$env(EB_CONFIG)/log.cf"

    init-hosts
    init-servers
}

proc init-hosts {} {
    global hosts_available_list hosts_list

    set hosts_available_list $hosts_list
}

proc init-servers {} {
    global servers_available_list servers_persistent_list

    set servers_persistent_list {logger}
    set servers_available_list  $servers_persistent_list
}

proc display-lg {} {
    global host server hosts_list

    :modify HOST   :value [lsearch $hosts_list $host]
    :modify SERVER :value " $server"
}



#-------------------------  Logger ------------------------------------

proc write-log_conf {fn } {
    global  log_list

    set header { {# Configuration file for logger}
	      {#}
	      {#  host          servername}
	      {#} }

    set data [concat $header $log_list]
    write-file $fn $data
    distribution $fn
}

proc write-log_cf {fn } {
    global  log_list

    foreach r $log_list {
	set host   [first  $r]
	set server [second $r]
    }
    set data [list "$host\t :$server"]
    write-file $fn $data
    distribution $fn
}

proc display-log_conf {data} {
   global  log_list edit
   set log_list {}

   foreach r $data {lappend log_list [get-line $r]}
   if {[llength $log_list] >= 0} {select-line} 
   if {!$edit} {deselect-line}
}

proc get-line {ps} {
   global  hosts_allocate_list hosts_available_list
   global  host_hold server_hold

   set host   [first $ps]
   set server [second $ps]

   set host_hold   $host
   set server_hold $server

   lappend hosts_allocate_list  $host
   set hosts_available_list [ldelete $hosts_available_list $host]

   return [format-line $host $server]
}

proc format-line {h s} {

   format "   %-12s %-23s" $h $s
}


proc select-line {} {
   global log_list log_line_select

   :modify LIST :empty T :append T :strings $log_list :select 0

   set log_line_select [lindex $log_list 0]
}

proc deselect-line {} {
   global  log_list

   :modify LIST :empty T :append T :strings $log_list :deselect 0
}

proc modify-conf {} {
   global log_list
   global host server

   set r [format-line $host $server]
   set log_list [list $r]
   select-line
}


#####################  CLICK  #############################

proc click {arg} {
   global debug hosts_list

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   if {$c==":quit"} then {click-quit $f; return}

   switch  $w {
      LIST 		{click-list $v}
      MODIFY 	{click-modify}
      HOST 		{click-host [lindex $hosts_list $v]}
      SERVERMENU 	-
      SERVER 	{click-server $v}

      QUIT 		{click-quit $f}

      default 	{puts stdout $arg}
   }
}

proc click-quit {f} {
    global open_frame_mod mytaskfirstname

    switch -glob $f {
      *.MOD {set open_frame_mod 0; close-frame $f}
      default {
          if {$open_frame_mod} then {
             set open_frame_mod 0
             close-frame [self frame-name].mod
          }
          close-frame $f
      }
    }

    clear-footer
    inform $mytaskfirstname quit-log
    set open_frame_mod 0
}


#-------------------------  Logger ------------------------------------

proc click-list {e} {
   global host server
   global log_list open_frame_mod label

   set-footer "$label: line selected..."

   set host     [first $e]
   set server   [second $e]

   if {$open_frame_mod} {display-lg}
}

proc click-modify {} {
   global label

   set-footer "$label: modify..."
   make-frame-modify
}

proc click-host {h} {
   global host install_needed

   clear-footer [self frame-name].mod
   set host $h
   set install_needed 1
}

proc click-server {s} {
   global server install_needed

   clear-footer [self frame-name].mod
   set server $s
   :modify SERVER :value " $server"
   set install_needed 1
}

proc click-mod-add {} {
   global  host server install_needed log_list mytaskfirstname
   global  open_frame_mod label

   if {$install_needed} then {
      set-footer "$label: install (needed)..."
      set z [expr [llength $host] && [llength $server]]
      if {$z == 0} then {
         set-footer [self frame-name].mod "Choice parameters in the menu list !!!"
         set-footer "$label: Error !!!"     
         display-error "Entry invalid parameter : null value"
         clear-footer
         set open_frame_mod 0
         return
      }
      modify-conf
   } else {
      select-line
   }

   set open_frame_mod 0
   inform $mytaskfirstname update-log_conf $log_list
   close-frame [self frame-name].mod
}
 
proc click-mod-cancel {} {
   global  open_frame_mod install_needed

   set install_needed 0
   set open_frame_mod 0

   close-frame [self frame-name].mod
}


#----------------- proc installation -------------------------------

proc putit {} {
    global  log_list log_cf bin
    global  host_hold server_hold
    set mess_list {}

    set mes [killit $host_hold $server_hold]
    if {$mes == -1} {return -1} else {print-messages $mes}

    foreach r $log_list {
	set host   [first  $r]
	set server [second $r]
    }

       lappend mess_list "Putting the logger server $server on machine $host"
       set z [catch {exec putit -h $host $server $bin} m]
       if {$z} {print-messages "$m"; return -1}
       lappend mess_list $m

       set z [catch {exec chmodit -h $host $server "i+all"} m]
       if {$z} {print-messages "$m"; return -1}
       lappend mess_list $m

       set z [catch {exec chmodit -h $host $server "l+all"} m]
       if {$z} {print-messages "$m"; return -1}
       lappend mess_list $m

    set host_hold   $host
    set server_hold $server

    write-log_cf $log_cf
    print-messages $mess_list
    return 0
}


proc install {} {
   global  log_conf label install_needed mytaskfirstname

   if {$install_needed == 0} {return 1}
   
   set i [midas-confirmation \
       "$label - Installation\n\n\
       This operation will \n\
       KILL any existing server,\n\
       and PUT (start) a new server \n\
       Do you wish to continue?"]

   if {$i} then {
       if {[putit] == -1} then {
	   inform $mytaskfirstname cancel-log_conf
	   return -1
	 }
       write-log_conf $log_conf
   } else {
       inform $mytaskfirstname cancel-log_conf
   }
   set install_needed 0
   clear-footer
   return 0
}
   
#####################  START  #############################
proc start args {
   global started edit install_needed 

   set edit   	[first  $args]
   set cancel 	[second $args]
   set data   	[third  $args]

   if {$cancel} {
      set install_needed 0
      clear-footer
   }
#   init-environment  -  done in common.beSetup
   init-parameters
   if {$started != 0} then {
      open-frame
   } else {
       make-frame
       set started 1
   }
   centre-frame
   display-log_conf $data
}





