# beSetup-spy.tcl---Back End Setup Spy frame
 
loadsource common.lnl
loadsource common.orbix
loadsource common.$mytaskfirstname

set icontitle 	    "spySetup"
set label "Spy Manager"

set spyM_line_select {}
set spyM_list     {}
set spyM_conf 	     ""
set spyM_cf       ""
set open_frame_mod    0
set install_needed 0

set host     ""
set host_hold   ""
set server   ""
set server_hold ""
set max_exp   0
set capab 	  0
set bin      ""

set ports_allocate_list     {}
set ports_available_list  {}
set servers_persistent_list {}
set servers_allocate_list {}
set servers_available_list  {}


#####################  FRAMES  #############################

proc make-frame {} {
   global env edit spyM_conf

   if {$edit} {
      set option_list F; set inactive F
   } else {
      set option_list T; set inactive T
      set line_index -1
   }

   :frame [self frame-name] :label [self frame-title] :show-footer T \
      :icon-image "@$env(ICONHOME)/euroball" \
	:icon-name [self icon-title] :resize F

        :panel BEGIN :layout T
          :layout H
          :button QUIT :w 15

        :panel TITLE :layout T :padx 10 :pady 10 
   	   :layout H
           :message TTL_1 :label "Manager Configuration File :"
           :message TTL_2 :label "$spyM_conf" 
 
        :panel DATA :layout T :padx 10 :pady 0 
   	   :layout V
	     :message MESS :label \
   "Host Name \tServer Name \tPort \tNumber \tMax Experiments \tCapability \tLogger File"
           :list LIST :rows 1 :cols 90 :exclusive T :deaf $option_list
 
        :panel ACTIONS :layout T :padx 10 :pady 5
   	   :layout H
	     :button MODIFY :label "Modify... " :w 10 :inactive $inactive

   :show T
}

proc make-frame-modify {} {
   global  host port server max_exp capab 
   global  hosts_list servers_available_list
   global  ports_available_list
   global  label env spyM_line_select
   global  open_frame_mod 

   click-list $spyM_line_select
   select-line

   :frame [self frame-name].mod :label "$label : Modify" \
	:show-footer T \
	:icon-image "@$env(ICONHOME)/euroball" \
	:icon-name [self icon-title] :resize F
	
        :panel MOD_PARAM :layout T :padx 50 :pady 20 
     	   :layout H; :row-gap 8
   	   :choice-stack HOST :label Host :strings $hosts_list :value [lsearch $hosts_list $host]
           :next-row T
  	   :menubutton SERVERMENU :label Server  :menu $servers_available_list
 	   :text SERVER :label "" :w 20 :value "$server"
     	   :next-row T
   	   :menubutton PORTMENU   :label "    Port    "  :menu $ports_available_list
  	   :text PORT :label ""   :w 6  :deaf T :value $port
     	   :next-row T
  	   :choice CAP :label Capability :strings [list "Off" "On"] :w 5 :value $capab
           :next-row T
  	   :number EXP :label "Max Experiments" :w 2  :min 0 :max 10 :value $max_exp

        :panel-dialog 40 CANCEL "click-mod-cancel"  APPLY  "click-mod-apply" 
        	      
   :show T

   set open_frame_mod 1
}


#####################  PROC  #############################

#-------------------------  parameters ---------------------------------

proc init-parameters {} {
    global log_file  spyM_conf spyM_cf
    global env edit bin frametitle

    if {$edit} then {
       set frametitle "Spy Configuration Tool (Edit)"
    } else {
       set frametitle "Spy Configuration Tool (Display)"
    }

    set bin	  "$env(EB_BIN)/EB_SpyManager"

    set log_file  "log.cf"
    set spyM_conf "$env(EB_ETC)/spyM.conf"
    set spyM_cf   "$env(EB_CONFIG)/spyM.cf"

    init-ports
    init-servers
    init-experiments
}

proc init-ports {} {
    global ports_available_list ports_list_spy

    set ports_available_list $ports_list_spy
}

proc init-servers {} {
    global servers_available_list servers_persistent_list

    set servers_persistent_list {SpyManager}
    set servers_available_list  $servers_persistent_list
}

proc init-experiments {} {set max_exp 10}

proc display {} {
    global host server port max_exp capab 
    global hosts_list ports_available_list

    :modify SERVER :value " $server"
    :modify HOST   :value [lsearch $hosts_list $host]
    :modify PORT   :value $port
    :modify EXP    :value $max_exp
    :modify CAP    :value $capab
}

proc write-spyM_conf {fn} {
    global spyM_list

    set header { {# Configuration file for Spy Manager}
	      {#}
{# host	     servername		       TS-config       MAX_EXP 	 use_capability  log-config}
	      {#} }

    set data [concat $header $spyM_list]
    write-file $fn $data
    distribution $fn
}

proc write-spyM_cf {fn} {
    global spyM_list

    foreach r $spyM_list {
	set host   [first  $r]
	set server [second $r]
    }
    set data [list "$host\t :$server"]
    write-file $fn $data
    distribution $fn
}

proc display-spyM_conf {data} {
   global spyM_list edit
   set spyM_list {}

   foreach r $data {lappend spyM_list [get-line $r]}
   if {[llength $spyM_list] >= 0 } {select-line} 
   if {!$edit} {deselect-line}
}

proc get-line {e} {
   global host_hold server_hold

   set host      [first $e]
   set server    [second $e]
   set port      [third $e]
   set exp       [fourth $e]
   set cap       [fifth $e]
   set log_f     [sixth $e]

   set host_hold   $host
   set server_hold $server

   return [format-line $host $server $port $exp $cap $log_f]
}

proc format-line {h s cf e c lf} {

   format " %-12s %-15s %-14s %-13d %-6d %s" $h $s $cf $e $c $lf
}

proc select-line {} {
   global  spyM_list spyM_line_select

   :modify LIST :empty T :append T :strings $spyM_list :select 0
   set spyM_line_select [lindex $spyM_list 0]
}
   
proc deselect-line {} {
   global  spyM_list

   :modify LIST :empty T :append T :strings $spyM_list :deselect 0
}

proc modify-spyM {} {
   global spyM_list
   global host server max_exp capab port log_file 
   
   set r [format-line $host $server $port $max_exp $capab $log_file]
   set spyM_list [list $r]
   select-line
}


#####################  CLICK  #############################

proc click {arg} {
   global debug hosts_list ports_available_list

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]

   if {$c==":quit"} then {click-quit $f; return}

   switch  $w {
      LIST 	      {click-list $v}
      MODIFY      {click-modify}
      HOST        {click-host [lindex $hosts_list $v]}
      SERVERMENU  -
      SERVER      {click-server $v}
      PORTMENU    -
      PORT 	      {click-port $v}
      EXP 	      {click-exp $v}
      CAP 	      {click-cap $v}
      MOD_CANCEL  {click-mod-cancel}
      MOD_APPLY   {click-mod-apply}

      QUIT 	      {click-quit $f}

      default 	  {puts stdout $arg}
   }
}

proc click-quit {f} {
    global  open_frame_mod mytaskfirstname

    switch -glob $f {
      *.MOD {set open_frame_mod 0; close-frame $f}
      default {
          if {$open_frame_mod} then {
             set open_frame_mod 0
             close-frame [self frame-name].mod
          }     
          close-frame $f
      }
    }
    clear-footer
    inform $mytaskfirstname quit-spy
    set open_frame_mod 0
}

#---------------- Manager ------------------------------------

proc click-list {e} {
   global host server port max_exp capab
   global label open_frame_mod

   set-footer "$label: line selected..."

   set host     [first  $e]
   set server   [second $e]
   set port     [third  $e]
   set max_exp  [fourth $e]
   set capab    [fifth  $e]
#  set log_file [sixth  $e]

   if {$open_frame_mod} {display}
}

proc click-modify {} {
   global label

   set-footer "$label: modify..."
   make-frame-modify
}

proc click-host {h} {
   global host install_needed 

   clear-footer [self frame-name].mod
   set host $h
   set install_needed 1
}

proc click-server {s} {
   global server install_needed

   clear-footer [self frame-name].mod
   set server $s
   :modify SERVER :value " $server"
   set install_needed 1
}

proc click-port {p} {
   global port install_needed

   clear-footer [self frame-name].mod
   set port $p
   :modify PORT :value $port
   set install_needed 1
}

proc click-exp {e} {
   global max_exp install_needed

   clear-footer [self frame-name].mod
   set max_exp $e
   :modify EXP :value $e
   set install_needed 1
}

proc click-cap {c} {
   global capap install_needed

   clear-footer [self frame-name].mod
   set capab $c
   :modify CAP :value $c 
   set install_needed 1
}

proc click-mod-apply {} {
   global  host server label install_needed mytaskfirstname
   global  open_frame_mod spyM_list

   if {$install_needed} then {
      set-footer "$label: install (needed)..."
      set z [expr [llength $host] && [llength $server]]
      if {$z == 0} then {
         set-footer [self frame-name].mod "Choice parameters in the menu list !!!"
         set-footer "$label: Error !!!"
         display-error "Entry invalid parameter : null value"
         clear-footer
         set open_frame_mod 0
         return
      }
      modify-spyM
   } else {
      select-line
   }
   set open_frame_mod 0
   inform $mytaskfirstname update-spyM_conf $spyM_list
   close-frame [self frame-name].mod
}

 
proc click-mod-cancel {} {
   global  open_frame_mod install_needed

   set install_needed 0
   set open_frame_mod 0

   close-frame [self frame-name].mod
   }

#----------------- proc installation -------------------------------

proc putit {} {
    global  bin  log_file spyM_list spyM_cf
    global  host_hold server_hold
    set mess_list {}
    set host ""; set server ""; set port 0; set exp 0; set cap 0
    set log_f ""


    set mes [killit $host_hold $server_hold]
    if {$mes == -1} {return -1} else {print-messages $mes}

    foreach r $spyM_list {
	set host      [first  $r]
	set server    [second $r]
	set port      [third  $r]
	set exp       [fourth $r]
	set cap       [fifth  $r]
	set log_f     [sixth  $r]
    }

    lappend mess_list "Putting (starting) $server $port $exp $cap $log_f on machine $host"
    set z [catch {exec putit -h $host $server \
		       "$bin $server $port $exp $cap $log_f"} m]
    if {$z} {print-messages "$m"; return -1}
    lappend mess_list $m

    set z [catch {exec chmodit -h $host $server "i+all"} m]
    if {$z} {print-messages "$m"; return -1}
    lappend mess_list $m

    set z [catch {exec chmodit -h $host $server "l+all"} m]
    if {$z} {print-messages "$m"; return -1}
    lappend mess_list $m

    set host_hold   $host
    set server_hold $server

    write-spyM_cf $spyM_cf
    print-messages $mess_list
    return 0
}

proc install {} {
   global  spyM_conf label install_needed mytaskfirstname

   if {$install_needed == 0} {return 1}
 
   set i [midas-confirmation \
       "$label - Installation\n\n\
       This operation will \n\
       KILL any existing server,\n\
       and PUT (start) a new server \n\
       Do you wish to continue?"]

   if {$i} then {
       if {[putit] == -1} then {
	   inform $mytaskfirstname cancel-spyM_conf
	   return -1
       }
       write-spyM_conf $spyM_conf
   } else {
       inform $mytaskfirstname cancel-spyM_conf
   }
  
   set install_needed 0
   clear-footer
   return 0
}


#####################  START  #############################

proc start args {
   global started edit install_needed

#   insert-debug "start: args = $args"

   set edit   	[first $args]
   set cancel 	[second $args]
   set dataM    [third $args]

   if {$cancel} {
      set install_needed 0
      clear-footer
   }
#   init-environment  -  done in common.beSetup
   init-parameters
   if {$started != 0} then {
      open-frame
   } else {
      make-frame
      set started 1
   }
   centre-frame
   display-spyM_conf $dataM
}

