# beSetup-stat.tcl---Back End Setup Statistics frame
 
loadsource common.lnl
loadsource common.orbix
loadsource common.$mytaskfirstname

set icontitle "statSetup"
set label    "Statistics"

set open_frame_mod 0
set hosts_list  {}
set menu_hosts  {}
set type_select {}
set servers_select {}


#####################  FRAMES  #############################

proc make-frame {} {
   global env hosts_list edit
   set offset 0; set rows 10

   if {$edit} {set inactive F} else {set inactive T}

   :frame [self frame-name]  :label [self frame-title] :show-footer T \
      :icon-image "@$env(ICONHOME)/euroball" \
      :icon-name [self icon-title] :resize F

      :panel BEGIN :layout T :padx 5 :pady 5
        :layout H
	  :button QUIT    :w 15
          :hskip 100
          :button REMOVE  :w 15  :inactive $inactive
          :button KILL    :w 15 
 	  :button DISPLAY :w 15
          :hskip 100
   	  :menubutton UPDATE :label "Update Host" :strings $hosts_list

      :panel DATA :layout V :padx 5 :pady 0
        set i 0
        set n 1
        foreach h $hosts_list {
     	     :layout H
	     :message $h-LS :label "Implemented Servers" :x [expr $offset - 3] :y 0
	     :message $h-NAME :bold T :label "$h" :x [expr 110 + $offset]
	     :message $h-PS   :bold F :label "Active Servers" :x [expr 155 + $offset] 
           :layout V
           :list $h-LS_LIST :rows $rows  :cols 18 :exclusive F :x $offset :double-click T
           :layout H
           :list $h-PS_LIST :rows $rows :cols 18 :exclusive F :double-click T 

	     set offset [expr 290 + $offset]; incr i
	     if {$i == [expr 3 * $n]} {
	       :panel DATA-$i :layout V :padx 5 :pady 0
     	       incr n; set offset 0
	    }
	 }

   :show T
}

proc make-frame-serv {type server} {
   global env open_frame_serv edit

   if {$edit} then {set option_cons F} else {set option_cons T}

   set serv [string tolower $server]
   if {$type == "PS_LIST"} then {
       set label "Server details for active server \: $server"
       set name "[self frame-name].p$serv"
       set cons "CONSOLE_P$serv"
   }  else {
       set label "Server details for server \: $server"
       set name "[self frame-name].l$serv"
       set cons "CONSOLE_L$serv"
   }
	
   :frame $name :show-footer F :label $label \
       :icon-image "@$env(ICONHOME)/euroball" \
       :icon-name [self icon-title] :resize F

       :panel SERV :layout T :fill both
         :galley $cons :rows 11 :cols 80 :deaf $option_cons

       :show T
   set open_frame_serv 1
}


#####################  PROC  #############################

proc init-parameters {} {
    global edit frametitle

    if {$edit} then {
       set frametitle "Back End Setup Statistics (Edit)"
    } else {
       set frametitle "Back End Setup Statistics (Display)"
    }
}

proc display {} {
   global hosts_list ERROR menu_hosts hosts_list
   
   set-busy-watch "Statistics : Waiting..."
   foreach h $hosts_list {
	if {[set list_servers [lsit $h]] == $ERROR} {
	   set active_servers $ERROR
	} else {
	   set active_servers [psit $h]
	}
	set ps_list {}
	foreach ps $active_servers {lappend ps_list [format "%s" $ps]}
	set ls_list {}
	foreach ls $list_servers {lappend ls_list [format "%s" $ls]}

	:modify $h-PS_LIST :empty T :append T :strings $ps_list
      :modify $h-LS_LIST :empty T :append T :strings $ls_list
   }
   set menu_hosts {}
   set t [ttime]
   foreach h $hosts_list {lappend menu_hosts "$h\t ($t)"}
   lappend menu_hosts "Update All Hosts"
   
   :modify UPDATE :strings $menu_hosts
   clear-busy-watch
}

#####################  CLICK  #############################

proc click {arg} {
   global debug type_select servers_select

   if {$debug} then {puts stderr "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {close-frame $f; return}
   if {$c == ":list-select"} {
       set type_select $w
       set servers_select [split $v \n]
   }

   switch  $w {
      KILL        {click-kill    $type_select $servers_select}
      REMOVE      {click-remove  $type_select $servers_select}
      DISPLAY 	{click-display $type_select $servers_select}
      UPDATE 	{click-update  $v $i}
      QUIT 	{close-frame   $f}
   }
}

proc double-click arg {
   global debug 
   if {$debug} then {puts stderr "double-click with argument \{$arg\}"}

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
      
   if {$c == ":list-select"} then {click-display $w $v} \
   			     else {puts stdout $arg}
}

proc click-kill {w servers} {
   global ERROR menu_hosts

   set tmp  [split $w -]
   set host [string tolower [first $tmp]]
   set type [second $tmp]
   set servs [join $servers ",  "]

   if {$type == "LS_LIST"} {
       midas-warning "The KILL command can not be used from Implemented Servers."
       return
   }

   if {$w == {} && $servers == {}} then {
       set-footer "Cannot kill: server not selected !!!"
       return
   }

   set q [midas-confirmation \
	 "The server(s) \"$servs\" will be killed.\n\n\
	  Do you wish to continue?"]

   if {$q} then {
      set-busy-watch "Killing server(s) \"$servs\":  Please wait..."
      foreach server $servers {
        if {[Killit $host $server] == $ERROR} {
	    clear-busy-watch-error
          return
	  }
      }
      set i 0
      foreach h $menu_hosts {
         if {[first $h] == $host} {break}
         incr i
      }
      click-update $host $i
      clear-busy-watch
   }
}


proc click-remove {w servers} {
   global frame_name ERROR menu_hosts

   set tmp  [split $w -]
   set host [string tolower [first $tmp]]
   set type [second $tmp]
   set servs [join $servers ",  "]

   if {$type == "PS_LIST"} {
       midas-warning \
         "The REMOVE command can not be used from Active Servers."
       return
       }
   if {$w == {} && $servers == {}} then {
      set-footer "Cannot remove: server not selected !!!"
      return
      }

   set q [midas-confirmation \
	 "The server(s) \"$servs\" will be removed.\n\n\
	  Do you wish to continue?"]

   if {$q} then {
      set-busy-watch "Removing server(s) \"$servs\":  Please wait..."
      foreach server $servers {
         if {[set info [Rmit $host $server]] == $ERROR} {
	     clear-busy-watch-error; return
	   }
      }
      set i 0
      foreach h $menu_hosts {
         if {[first $h] == $host} {break}
         incr i
      }
      click-update $host $i
      clear-busy-watch
   }
}

proc click-display {w servers} {
   global mkfr_nbr frame_name ERROR

   set tmp   [split  $w -]
   set host  [string tolower [first $tmp]]
   set type  [second $tmp]
   set servs [join   $servers ",  "]

   if {$w == {} && $servers == {}} then {
      set-footer "Cannot open display: server not selected !!!"
      return
   }

   set-busy-watch "Display Server(s) \"$servs\": Please wait..."
   foreach server $servers {
      set serv [string tolower $server]
      if {$type == "PS_LIST"} then {
         set cmd catit_process
         set cons CONSOLE_P$server
      } else {
	   set cmd catit
         set cons CONSOLE_L$server
      }
      if {[set info [$cmd $host $server]] == $ERROR} {
	  clear-busy-watch-error
        return
	}
      make-frame-serv $type $server
      :modify $cons  :append T :strings "[join $info \n]"       
   }
   clear-busy-watch
}

proc click-update {str index} {
   global ERROR menu_hosts hosts_list

   set hosts {}
   set selected [first $str]
   if {$selected == "Update"} then {set hosts $hosts_list} \
   			      else {lappend hosts $selected}
          
   set hs [join $hosts ",  "]
   set i 0
   set-busy-watch "Update Statistics Host \"$hs\": Please wait..."
   foreach h $hosts {
      if {[set list_servers [lsit $h]] == $ERROR} \
         then {set active_servers $ERROR} \
         else {set active_servers [psit $h]}
      set ps_list {}
      foreach ps $active_servers {lappend ps_list [format "%s" $ps]}
      set ls_list {}
      foreach ls $list_servers {lappend ls_list [format "%s" $ls]}
      :modify $h-PS_LIST :empty T :append T :strings $ps_list
      :modify $h-LS_LIST :empty T :append T :strings $ls_list
      set mes "$h\t ([ttime])"      
      if {$selected == "Update"} {
         set menu_hosts [lreplace $menu_hosts $i $i $mes]; incr i
      } else {
         set menu_hosts [lreplace $menu_hosts $index $index $mes]
      }  
   }
   :modify UPDATE :strings $menu_hosts

   clear-busy-watch
}

#####################  START  #############################
proc start args {
   global started edit hosts_list
   
   set edit       [first  $args]
   set hosts_list [second $args]
   
#   init-environment  -  done in common.beSetup
   init-parameters
   if {$started != 0} then {
      open-frame
   } else {
       make-frame
       set started 1
   }
   centre-frame
   display
}

proc install {} {return}
