# camaccontrol.tcl---CAMAC Configuration & Control Frame
#     This is the Euroball version but is a good start for any use of camac with VXI since it is 
#     assumed that the FVI would be used

loadsource camaccontrol.common


proc repaint {} {
   global env debug moduletype modulename camaccrate camacstation vsn subaddr numchans group item ipg iincr icomment
   global modulechoice cratechoice stationchoice numchoice 
   if {$debug} then {puts stderr "repaint"}
   set mt [lsearch $modulechoice $moduletype]
   if {$mt == -1} then {puts stderr "cannot find $moduletype"}
   set cc [lsearch $cratechoice $camaccrate]
   set cn [lsearch $stationchoice $camacstation]
     :modify MODULE   :value $mt
     :modify CRATE    :value $cc
     :modify STATION  :value $cn
     :modify VSN      :value $vsn
     :modify SUBADDR  :value $subaddr
     :modify CHANNELS :value $numchans
     :modify GROUP    :value $group
     :modify ITEM     :value $item
     :modify IPG      :value $ipg
     :modify IINCR    :value $iincr
     :modify COMMENT  :value $icomment
}


proc click arg {
   global debug env preferences
   global vsn subaddr
   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   if {$c==":Enter"} then {return}
   switch $w  {

	MODULE	{click-module $v}
	CRATE	{click-crate  $v}
	STATION	{click-station $v}
	VSN     {set vsn $v}
        SUBADDR {set subaddr $v}
        CHANNELS {click-channels $v}
	GROUP   {click-group $v}
	ITEM	{click-item $v}
	IPG	{click-ipg $v}
        IINCR   {click-iincr $v}
        COMMENT {click-comment $v}

        FVICONFIG   {fvi_config_frame}
        M1B1LOW   {set preferences(FVI.Module1.Branch1minCrate) $i}
        M1B1HIGH  {set preferences(FVI.Module1.Branch1maxCrate) $i}
        M1B2LOW   {set preferences(FVI.Module1.Branch2minCrate) $i}
        M1B2HIGH  {set preferences(FVI.Module1.Branch2maxCrate) $i}
        M2B1LOW   {set preferences(FVI.Module2.Branch1minCrate) $i}
        M2B1HIGH  {set preferences(FVI.Module2.Branch1maxCrate) $i}
        M2B2LOW   {set preferences(FVI.Module2.Branch2minCrate) $i}
        M2B2HIGH  {set preferences(FVI.Module2.Branch2maxCrate) $i}
        FVIDEFAULT  {fvi_default_config; fvi_config_frame}

	ADD	{click-add}
	REPLACE	{click-replace}
	DELETE	{click-delete}
        CLEAR   {click-clear}
        DEFAULT {click-default}

        LOAD	{load-configuration+LUT}
        MAKELUT {makeLUT}

	CONFIG-DATALIST {click-datalist $c [lindex $arg 4]}
	
        RESET   {DAQ-reset}
        GO      {DAQ-go}
        HALT    {DAQ-halt}

        SAVERESTORE    {inform camacsaverestore SSRstart}
        SR             {update_preferences; inform camacsaverestore CSRstart}
        PRINT   {
            set-footer "See Session Log"
            click-printconfig
        }

        REDISPLAY  {click-redisplay}

        default   {puts stdout $arg}
    }
}


set LUTmsg "\n Do you wish to create a new LUT for the FVI module(s)?\n\
       You will need to do this if you have changed the configuration of FERA data sources\n\n"


proc fvi_default_config {} {
   global preferences

   set preferences(FVI.Module1.Branch1minCrate) 1
   set preferences(FVI.Module1.Branch1maxCrate) 10
   set preferences(FVI.Module1.Branch2minCrate) 11
   set preferences(FVI.Module1.Branch2maxCrate) 20

   set preferences(FVI.Module2.Branch1minCrate) 21
   set preferences(FVI.Module2.Branch1maxCrate) 30
   set preferences(FVI.Module2.Branch2minCrate) 31
   set preferences(FVI.Module2.Branch2maxCrate) 40
}

fvi_default_config

proc makeLUT {} {
  global env debug preferences datalist LUTdir 
  global modulechoice  databits 
  set-footer "Making the FVI LUT file"
#    initialise LUT arrays with a default value
    for {set vsn 0} {$vsn < 256} {incr vsn} {
      for {set subaddr 0} {$subaddr < 16} {incr subaddr} {
        set LUT1(1:$vsn:$subaddr) 0
        set LUT2(1:$vsn:$subaddr) 0
        set LUT1(2:$vsn:$subaddr) 0
        set LUT2(2:$vsn:$subaddr) 0
      }
    }
#    calculate LUT entry for each subaddr in each module
  foreach data $datalist {
    set moduletype [first $data]
    set dbits [lindex $databits [lsearch $modulechoice $moduletype]]
    set camaccrate [third $data]
    set vsn [fifth $data]
    set subaddr [sixth $data]
    set numchans [seventh $data]
    set group [eigth $data]
    set item [nineth $data]
    set ipg [tenth $data]
    set iincr [eleventh $data]

    set items 0
    for {set i 0} {$i < $numchans} {incr i} {
        incr items
        if {$camaccrate >= $preferences(FVI.Module1.Branch1minCrate) \
            &&  $camaccrate <= $preferences(FVI.Module1.Branch1maxCrate) } then {set LUT1(1:$vsn:$subaddr) "$item $group"}
        if {$camaccrate >= $preferences(FVI.Module1.Branch2minCrate) \
            &&  $camaccrate <= $preferences(FVI.Module1.Branch2maxCrate) } then {set LUT2(1:$vsn:$subaddr) "$item $group"}
        if {$camaccrate >= $preferences(FVI.Module2.Branch1minCrate) \
            &&  $camaccrate <= $preferences(FVI.Module2.Branch1maxCrate) } then {set LUT1(2:$vsn:$subaddr) "$item $group"}
        if {$camaccrate >= $preferences(FVI.Module2.Branch2minCrate) \
            &&  $camaccrate <= $preferences(FVI.Module2.Branch2maxCrate) } then {set LUT2(2:$vsn:$subaddr) "$item $group"}
        incr subaddr
        incr item $iincr
        if {$items >= $ipg} then {set item [nineth $data]; incr group; set items 0}
    }
  }

#    finally write the LUT to file
  catch {exec mkdir $LUTdir} zzz
  set z [catch {open $LUTdir/LUT.fvi w} file]
  if {$z != 0} then {set-footer "$file"; return}

#    make LUT1 for Module1
  puts $file "module 1"
  puts $file "#  Branch 1"
  set offset 0
  for {set vsn 0} {$vsn < 256} {incr vsn} {
    set vsnhdr 0
    for {set subaddr 0} {$subaddr < 16} {incr subaddr} {
       if {$LUT1(1:$vsn:$subaddr) != 0} then {
          if {$vsnhdr == 0} then {puts $file "# vsn $vsn"; incr vsnhdr}
          puts $file "lut1+$offset $LUT1(1:$vsn:$subaddr)"
       }
       incr offset 2
    }
  }
#    make LUT2 for Module 1
  puts $file "#  Branch 2"
  set offset 0
  for {set vsn 0} {$vsn < 256} {incr vsn} {
    set vsnhdr 0
    for {set subaddr 0} {$subaddr < 16} {incr subaddr} {
       if {$LUT2(1:$vsn:$subaddr) != 0} then {
          if {$vsnhdr == 0} then {puts $file "# vsn $vsn"; incr vsnhdr}
          puts $file "lut2+$offset $LUT2(1:$vsn:$subaddr)"
       }
       incr offset 2
    }
  }

#    make LUT1 for Module 2
  puts $file "module 2"
  puts $file "#  Branch 1"
  set offset 0
  for {set vsn 0} {$vsn < 256} {incr vsn} {
    set vsnhdr 0
    for {set subaddr 0} {$subaddr < 16} {incr subaddr} {
       if {$LUT1(2:$vsn:$subaddr) != 0} then {
          if {$vsnhdr == 0} then {puts $file "# vsn $vsn"; incr vsnhdr}
          puts $file "lut1+$offset $LUT1(2:$vsn:$subaddr)"
       }
       incr offset 2
    }
  }
#    make LUT2 for Module 2
  puts $file "#  Branch 2"
  set offset 0
  for {set vsn 0} {$vsn < 256} {incr vsn} {
    set vsnhdr 0
    for {set subaddr 0} {$subaddr < 16} {incr subaddr} {
       if {$LUT2(2:$vsn:$subaddr) != 0} then {
          if {$vsnhdr == 0} then {puts $file "# vsn $vsn"; incr vsnhdr}
          puts $file "lut2+$offset $LUT2(2:$vsn:$subaddr)"
       }
       incr offset 2
    }
  }

  close $file
  set-footer "LUT written to $LUTdir/LUT.fvi"
}

proc make-frame {} {
   global env started 
   global modulechoice cratechoice stationchoice numchoice
  set started 1

  :frame [self frame-name] :label "[self frame-title]"     \
      :show-footer T  :resize F  \
      :icon-image "@$env(ICONHOME)/camac_crate" \
      :icon-name "[self icon-title]"

  :enable [self frame-name] Enter

   :panel CONFIGURATION  :layout T :padx 10 :pady 10
      :layout V; :row-gap 10; :col-gap 10
      :list M  :rows 3 :cols 130 :strings [list ""] :deaf T
      :list CONFIG-DATALIST  :rows 20 :cols 130 :exclusive T :strings [list ""] :double-click T
   :panel OPTIONS  :layout T :padx 10
      :layout H; :row-gap 10; :col-gap 10

      :choice-stack MODULE  :label "Module Type:"   :exclusive T  :strings "$modulechoice"  :value 0
      :choice-stack CRATE   :label "CAMAC Crate:"   :exclusive T  :strings "$cratechoice"   :value 0
      :choice-stack STATION :label "CAMAC Station:" :exclusive T  :strings "$stationchoice"   :value 0
#      :choice-stack VSN     :label VSN:             :exclusive T  :strings "$numchoice"       :value 0
      :text         VSN     :label VSN:  :w 5      :value ""
#      :choice-stack SUBADDR :label SubAddr:         :exclusive T  :strings "$numchoice"       :value 0
      :text         SUBADDR     :label SubAddr:  :w 5      :value ""
      :choice-stack CHANNELS :label Channels:       :exclusive T  :strings "$numchoice"       :value 0

      :next-row T
#      :choice-stack GROUP :label Group:             :exclusive T  :strings "$numchoice"  :value 1
      :text GROUP         :label Group:      :w 5    :value ""
      :choice-stack ITEM  :label Item:              :exclusive T  :strings "$numchoice"   :value 0
      :choice-stack IPG   :label "Items per group:" :exclusive T  :strings "$numchoice"    :value 2
      :choice-stack IINCR :label "Item increment:"  :exclusive T  :strings "$numchoice"  :value 2
      :text COMMENT       :label Comment:    :w 24   :value ""
      :button FVICONFIG   :label "Define FVI Configuration"
  
   :panel CLICK :layout T :padx 10
      :layout H; :row-gap 10; :col-gap 15
      :vskip 10
      :button ADD     :label Add
      :button REPLACE :label Replace
      :button DELETE  :label Delete
      :button CLEAR   :label Clear
      :button DEFAULT :label "Default Configuration"
      :button LOAD    :label "Load Configuration into Register Server"
      :button MAKELUT :label "Generate FVI LookUp table"
      :button PRINT   :label "Print Configuration"
      :next-row T
      :button REDISPLAY :label Redisplay
      :hskip 40
      :button RESET   :label "RESET electronics"
      :button GO      :label "GO electronics"
      :button HALT    :label "HALT electronics"
      :button SAVERESTORE    :label "Save/Restore Setup"
      :button SR             :label "Save/Restore Configuration"
      :button HELP

   click-default
   close-frame

}

proc fvi_config_frame {} {
  global preferences cratechoice

  :frame "[self frame-name].fviconfig" :label "FVI CAMAC Configuration" :show-footer T :resize F :transient T

    :panel FVIPANEL :layout T :padx 5 :pady 5
     :layout V; :row-gap 5

      :message FVIM1       :label "FVI Module 1 Branch 1" :bold T
      :choice-stack M1B1LOW  :label "Low CAMAC Station:"  :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module1.Branch1minCrate)]
      :choice-stack M1B1HIGH :label "High CAMAC Station:" :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module1.Branch1maxCrate)]

      :message FVIM2       :label "FVI Module 1 Branch 2" :bold T
      :choice-stack M1B2LOW  :label "Low CAMAC Station:"  :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module1.Branch2minCrate)]
      :choice-stack M1B2HIGH :label "High CAMAC Station:" :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module1.Branch2maxCrate)]

      :message FVIM3       :label "FVI Module 2 Branch 1" :bold T
      :choice-stack M2B1LOW  :label "Low CAMAC Station:"  :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module2.Branch1minCrate)]
      :choice-stack M2B1HIGH :label "High CAMAC Station:" :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module2.Branch1maxCrate)]

      :message FVIM4       :label "FVI Module 2 Branch 2" :bold T
      :choice-stack M2B2LOW  :label "Low CAMAC Station:"  :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module2.Branch2minCrate)]
      :choice-stack M2B2HIGH :label "High CAMAC Station:" :strings "$cratechoice"  \
              :value [lsearch $cratechoice $preferences(FVI.Module2.Branch2maxCrate)]

      :button FVIDEFAULT :label "restore default configuration"

      :show T
}

