########################################################################
# This patch file contains a new version of procedure
# proc script-neorc
# The old version of proc script-neorc produces  the
# script file neorc with information embedded in the code.
# The new version reads (1) the features of the available resources
# from the configuration file MIDAS and (2) the configuration of
# the current experiment with the procedure :experiment
# that uses the EG command.
#

##################### proc script-neorc NEW VERSION #####################
 
proc script-neorc {host prog proc_nbr} {
   global env
   global Farm_installdir
   global srchost srcgroup dsthost msrcport mdstport
   global ERROR
   
# I read the maximum configuration from file MIDAS
   set f [file join $env(EB_CONFIG)  MIDAS]
   
set err [set-resources $f]
   if {$err == $ERROR} {set-footer $err; return} 

# I read the current event collector configuration from the joined resources
# and write the configuration file src.conf
   set file [file join $env(EXPERIMENTDIR)  src.conf]

   set filecontents [read-ecConf]
   if {$filecontents == $ERROR} then {set-footer $err; return}

   set err [write-file $file $filecontents]
   if {$err == $ERROR} {set-footer $err; return}
 
# I write the script file neorc
# DEBUG (1 row)
   set data {}
   lappend data {#!/bin/csh} 
   lappend data "#BSUB -m $host"
   lappend data {set EB_DIST=/euroball/src/neo/cmd_server}
   lappend data "unsetenv ANALYSIS_NAME;setenv ANALYSIS_NAME $prog"
   lappend data {unsetenv HOST;         setenv HOST `/bin/hostname`}
   lappend data "unsetenv CMD_SERVER;   setenv CMD_SERVER [set prog]_cmd"
   lappend data {$EB_DIST/register $HOST $CMD_SERVER}
   lappend data "unsetenv IAM;          setenv IAM [expr $proc_nbr - 1]"
   lappend data {unsetenv BASE_SPY_PORT;setenv BASE_SPY_PORT 16000}
   lappend data "unsetenv SPY_SERVER_HOST; setenv SPY_SERVER_HOST $host"
   lappend data {unsetenv STRM1_LEN;    setenv STRM1_LEN 0} 
   lappend data {unsetenv STRM2_LEN;    setenv STRM2_LEN 0} 
   lappend data {unsetenv STRM3_LEN;    setenv STRM3_LEN 0} 
   lappend data {unsetenv STRM4_LEN;    setenv STRM4_LEN 0} 
   lappend data {unsetenv MRG_REPEAT;   setenv MRG_REPEAT 0} 
   lappend data {unsetenv MRG_TIMEOUT;  setenv MRG_TIMEOUT 10.0} 

# I read and set current farm configuration from file evFarm.conf

   set file [file join $env(EB_ETC)  evFarm.conf]
   set config [read-file $file]
   if {$config == $ERROR} {set-footer $err; return}

   set mrgdstnum 0
   set cpusnum 0
   foreach item $config {
      incr mrgdstnum
      set host [first $item]
      set numcpus [second $item]
      set mdstportname [lindex $mdstport $mrgdstnum]
      lappend data "unsetenv MRG_DSTHOST$mrgdstnum; setenv MRG_DSTHOST$mrgdstnum $dsthost($host)"
      lappend data "unsetenv MRG_TKPORT$mrgdstnum; setenv MRG_TKPORT$mrgdstnum $mdstportname"
      if {$numcpus == 2} then {
         incr mrgdstnum
         set mdstportname [lindex $mdstport $mrgdstnum]
         lappend data "unsetenv MRG_DSTHOST$mrgdstnum; setenv MRG_DSTHOST$mrgdstnum $dsthost($host)"
         lappend data "unsetenv MRG_TKPORT$mrgdstnum; setenv MRG_TKPORT$mrgdstnum $mdstportname"
      }
    }
    lappend data "unsetenv MRG_DSTNUM;   setenv MRG_DSTNUM $mrgdstnum"

# I read and set current front end configuration from file src.conf

  set file [file join $env(EXPERIMENTDIR)  src.conf]
   set config [read-file $file]
   if {$config == $ERROR} {set-footer $err; return}
   
   set mrgsrcnum 0
   foreach item $config {
      incr mrgsrcnum
      set host [first $item]
      set group [second $item]
      set msrcportname [lindex $msrcport $mrgsrcnum]
      lappend data "unsetenv MRG_SRCHOST$mrgsrcnum; setenv MRG_SRCHOST$mrgsrcnum $srchost($host)"
      lappend data "unsetenv MRG_SRCPORT$mrgsrcnum; setenv MRG_SRCPORT$mrgsrcnum $msrcportname"
      lappend data "unsetenv  MRG_GROUPNUM$mrgsrcnum; setenv MRG_GROUPNUM$mrgsrcnum $group"
      for {set i 1} {$i <= 3} {incr i} {
         set indice [expr $i+1]
         set roco [lindex $item $indice]
         if {$roco != ""} then {
            set envvarnameA MRG_$mrgsrcnum
            set envvarnameB ROCO$i
            set envvarname $envvarnameA$envvarnameB
            lappend data "unsetenv $envvarname; setenv $envvarname $roco"
         } else {
         break
         }
       }
       set fvnum [expr $i-1]
       lappend data "unsetenv MRG_FV$mrgsrcnum; setenv MRG_FV$mrgsrcnum $fvnum"
   }

   lappend data "unsetenv MRG_SRCNUM;   setenv MRG_SRCNUM $mrgsrcnum"

   lappend data "cd $Farm_installdir"
   lappend data	"exec ./$prog"
   lappend data	"sleep 60"
   return $data
}
 

##################### PROC ###################################

proc set-resources file  {
 
 global env
 global srchost srcgroup dsthost msrcport mdstport
 global ERROR

 set config [read-file $file]

# If I do not read teh configuraton file MIDAS I return from the procedure
 if {$config == {}} then {
  midas-warning "Error while reading configuration file \n$file\n
This must be corrected before you can continue"
  return $ERROR
 }

# Now I read from the file MIDAS the property (which I need) 
# of the configured resources which 
 foreach item $config {
   set command [first $item]
   switch $command {
    ec          { set srchost([second $item]) [fourth $item] 
                  set srcgroup([second $item]) [fifth $item] }
    host        { set dsthost([second $item]) [third $item] }
    mrgsrcports { set msrcport $item }
    mrgdstports { set mdstport $item } 
    default {}
   }
  }
  return 0

}   


proc read-ecConf {}  {

 global eclist vxilist srcgroup vxiofgroup
 global ERROR

 set err [read-fe-resources]

# I verify 1- if there are configured resources
#          2- if I can read correct the vxi's propertys
 if {$err != 0} then {
  midas-warning "There are not configured resources!\n 
configure resources for current experiment\n before you must continue"
 return $ERROR
 }

 set err [read-vxi-config $vxilist]
 if {$err != 0} then {return}

# Now I set the content of the file src.conf
 set data {}
 foreach item $eclist {
  set index $srcgroup($item)
  set wronggroup 1
  for {set i 0} {$i < 8 } {incr i} {
   set gnum [expr $i+248]
   if {$srcgroup($item) == $gnum} then {
    set wronggroup 0
    break
   } 
  }
  if {$wronggroup == 1} then {
   midas-warning "Wrong Group Number in file MIDAS\n
This must be corrected before you can continue"
   return $ERROR
  } 
  lappend data "$item $srcgroup($item) $vxiofgroup($index)"
 }

 return $data

}


proc read-fe-resources  {}  {
 
 global resources eclist vxilist ERROR 
 
 set resources [:experiment resources]
 if {$resources == {}} then  {
  return $ERROR
 }
 set srclist [make-source-list $resources]

# I create 2 lists: one of configured EC and
#                   one of configured VXI
 set eclist {}
 set vxilist {}
 foreach item $srclist  {
  switch [first $item]  {
   EC      {lappend eclist  [second $item]}
   VXI  -
   pci  -
   vme     {lappend vxilist [second $item]}
   default {}
  }
 }
# puts stdout "List of configured ec  = $eclist"
# puts stdout "List of configured vxi = $vxilist"
 return 0
 
}
 
proc make-source-list s  {
 
 set ll {}
 foreach p $s {lappend ll [format-source $p]}
 return $ll
 
}
 
proc format-source sr  {
 
set name "" 
set role ""

 foreach s $sr {
  switch [first $s]  {
   name     {set name [second $s]}
   role     {set role [second $s]}
   default  {}
  }
 }
 
 #format "%-10s%-10s%" $role $name
 format "%s  %s" $role $name
 
}

proc read-vxi-config vxi  {

 global env vxiofgroup
 global ERROR

# foreach group of VXI I create a variable thath, at the end,
# will contain, foreach group, the number of configured VXI 
 for {set i 0} {$i < 8 } {incr i} {
  set index [expr $i+248]
  set numvxiofgroup($index) 0
 }

 set workdir [file join $env(MIDASBASE) config VXI]

 foreach item $vxi  {
    if {$item == "vme1"} continue
  set vxinum 0
  set filename [file join $workdir $item]
  set config [read-file $filename]
  if {$config == {}} {
   midas-warning "Error while reading configuration file $filename\n
This must be corrected before you can continue"
   return $ERROR
  }
  foreach p $config  {
   set str [second $p]
   if {[string match VRE* $str] != 1} then {
    continue
   } else {
    set vxinum [second [split $str E]]
    break
   }
  }
 
  set lowerbound 1
  set upperbound 15
  set wrongvxinum 1
  for {set i 0} {$i < 8 } {incr i} {
   set num [expr $i+248]
   if {($vxinum >= $lowerbound) && ($vxinum <= $upperbound)} then {
    set numvxiofgroup($num) [expr $numvxiofgroup($num)+1]
    set wrongvxinum 0
    break
   }
   set lowerbound [expr $lowerbound+16]
   set upperbound [expr $upperbound+16]
  }
  if {$wrongvxinum != 0} then {
   midas-warning "Error in the file $filename\nyou must correct the VRE number before continue" 
  }
 }
 
 for {set i 0} {$i < 8} {incr i} {
  set index [expr $i+248]
  set vxiofgroup($index) ""
  if {$numvxiofgroup($index) == 1} then {
   lappend vxiofgroup($index) 15
  } else {
   if {$numvxiofgroup($index) > 1} then {
    for {set j 0} {$j < [expr $numvxiofgroup($index)-1]} {incr j} {
     lappend vxiofgroup($index) [expr $j+1]
    }
    incr j
    lappend vxiofgroup($index) 15
   }
  }
 }
 return 0

}

