 #
#    common.farm.tcl
#
#    procedures used by FARM

set sig_start  SIGUSR2
set sig_unload SIGHUP
set sig_term SIGKILL


set line_index   0
set workdir    ""
set prog_name   ""
set job_name   ""
set farm_state ""
set job_id      0
set defaultdir ""
set defaultfile ""



proc read-Farm_conf {} {
   global  env Farm
   return [read-file $env(EB_ETC)/$Farm.conf]
}

proc read-Farm_neo {} {
   global Farm  
   global Farm_workdir
   global workdir prog_name job_name farm_state

   set data [read-file $Farm_workdir/$Farm.neo]
   foreach item $data {
     if {$item != {}} then {
         set workdir   [first  $item]
         set prog_name [second $item]
         set farm_state [third $item]
	 break
     }
   }

   if {$workdir == ""} then {
      set workdir $Farm_workdir
      set prog_name ""
   }
   set job_name  [get-jobname $prog_name]
   return [list $workdir $prog_name $job_name $farm_state]
}   
   
proc write-Farm_neo {dir prog action} {
   global Farm
   global Farm_workdir
   global workdir prog_name job_name farm_state
   
   read-Farm_neo
   
   if {$dir != ""} then {set workdir $dir}
   if {$prog != ""} then {set prog_name $prog}
   set farm_state $action
   
   set header { \
      {# Definition of neo program loaded to Event Processor Farm }
      {#}
      {#    directory		  program_name   action}
      {#} }

   set data [concat $header [list "$workdir $prog_name $farm_state"]]
   write-file $Farm_workdir/$Farm.neo $data
}
   
proc signal-Processors {signal} {
   global  Farm job_name prog_name ERROR OK mytaskfirstname
   global  defaultdir defaultfile
   global  Farm_queue

   if {![bping]} then {return}
   
   if {$job_name == ""} then {
      read-Farm_neo
      if {$job_name == ""} then {
         if {[bjobs_run $Farm_queue] != $OK} then {return "No program loaded"}
      }
   }

   set names [bjobs_names $Farm_queue]
   set ok  0
   foreach name $names {if {$name == $job_name} then {set ok 1; break}}
   if {!$ok} then {
       midas-warning "The program \"$prog_name\" is not loaded."
       return $ERROR
   }
	
   set-busy-watch "Sending signal $signal to neo program \"$prog_name\" \
    		   from all event processors : Waiting..."
   set out [bkill $Farm_queue $job_name $signal]
   clear-busy-watch
   deactive-start_unload
   return $out
}

#------------------------------------------
# 
# Submit a job to the lsbatch system
#
# If script shell is used then the name host distant is included in the
# script and the option "-m" is not used.
#
# return  message job or ERROR if error
#

proc bsub-Farm {host prog proc_nbr} {
   global Farm env ERROR OK
   global Farm_queue Farm_installdir

   if {$host == "" || $prog == ""} then {
      set m "Error to submit a job.\n\
	     bad parameter: \n\
	     processor  = \"$host\"\n\
	     prog       = \"$prog\""
      midas-error "Bad parameter" $m
      return $ERROR
   }

   set file_exec [file join $env(MIDASTempDir) neorc$host$proc_nbr]

   switch $Farm {
     evFarm  {set script [script-neorc $host $prog $proc_nbr]}
     spyFarm {set rcfile [read-file $env(EB_CONFIG)/spy.rc]
              if {$rcfile == ""} then {return}
              set script {}
              foreach line $rcfile {
                 set z [catch {subst $line} m]
                 if {$z == 0} then {lappend script "$m"} else {lappend script "$line"}
              }
     }
   }

   set err [write-file $file_exec $script]
   if {$err == $ERROR} then {set-footer $err; return $ERROR}

   exec chmod +x $file_exec 
   set m [lsrcp $file_exec $host $Farm_installdir/neorc$proc_nbr]
   if {$m != ""} then {midas-warning "lsrcp returned $m"}
		     		
   set job      [get-jobname $prog]
   set output   $job$proc_nbr
   set cmd      $file_exec
  
   switch $Farm {
     evFarm  {set out [bsub-shell $Farm_queue       $job $output "" "" $cmd]}
     spyFarm {set out [bsub-shell $Farm_queue       $job $output "" "" $cmd]}
   }

#   remove-file $file_exec
   if {$out != $ERROR} then {
      set title "Processor $proc_nbr: \
      		 loadding neo program \"$prog\" in $host"
      set out [linsert $out 0 $title]
   }

   return [list [join $out "; "]]
}


#####################  PROC  #############################


proc unload-Processors  {} {
   global sig_unload ERROR
   
   set out [signal-Processors $sig_unload]
   if {$out == "No program loaded"} then {return $out}
   if {$out != $ERROR} {
      write-Farm_neo "" "" "unload"
      print-messages $out
      set-busy
      for {set i 0} {$i < 30} {incr i} {
         if {[display-status_jobs] == $ERROR} then {break}
         set-footer "Waiting for jobs to terminate"
         after 2000
         clear-footer
      }
      if {$i == 10} then {midas-warning "NEO++ programs have not unloaded after 60 secs\nYou will have to force them out manually before you can continue\n"}
   }
   clear-busy
}
   
proc start-Processors {} {
   global sig_start ERROR
   
   set out [signal-Processors $sig_start]
   if {$out == "No program loaded"} then {return $out}
   if {$out != $ERROR} {
      write-Farm_neo "" "" "start"
      print-messages $out
      display-status_jobs
      set out "processor farm start failed"
   } else {
      print-messages $out
      set out ""
   }
   return $out
}

proc stop-Processors {} {
   global sig_term ERROR
   
   set out [signal-Processors $sig_term]
   if {$out == "No program loaded"} then {return $out}
   if {$out != $ERROR} {
      write-Farm_neo "" "" "stop"
      print-messages $out
      display-status_jobs
      set out "processor farm stop failed"
   } else {
      print-messages $out
      set out ""
   }
   return $out
}












