#
#    common.lnl.tcl
#
#    Common procedures used to Legnaro


set ERROR -1
set OK     0
set NOPNOP 1

set directory       ""
set mytaskfirstname [lindex [split $mytaskname {-_}] 0]

set editor [file join $env(OPENWINHOME) bin textedit]


set users_queue ""


#-----------------------------------------
# 
# initialise environment
#
proc init-environment {} {
     global env directory
     
     if {![file isdirectory $directory]} then {if {[info exists env(HOME)]} then {set directory $env(HOME)}}

     loadsource EB_environment

     foreach variable {users_queue} {
         global $variable
         set $variable $env([string toupper $variable])
     }
}


#------------------------------------------
# 
# Returns 1 if "f" is a file C++, 0 otherwise.
#
proc isfileC {f} {
   set extension [file extension $f]
   if {$extension == ".cc" || $extension == ".C" || $extension == ".c"} \
   then {return 1} \
   else {return 0}
}

#------------------------------------------
# 
# Returns the name of C++ file executable.
#
proc get-execname {f} {return [first [split $f "."]]}

#------------------------------------------
# 
# return string
#
proc ttime {} {
    set z [catch {date \+%H:%M:%S} m]
    if {$z} then {return ""} else {return $m}
}


#------------------------------------------
# 
# return new list or hold list
#
proc ldelete {list val} {
    set i [lsearch -exact $list $val]
    if {$i >= 0} then {
	return [lreplace $list $i $i]
    }

    return $list
}

#------------------------------------------
# 
# Error Report
#
proc display-error {m} {midas-error "" $m}

proc whoami {} {
   global env
   if {$env(platform) == "windows"} then {
      set-footer "/usr/ucb/whoami not supported by windows"
      return "root"
   } else {
#    assume Unix
      if [catch {exec /usr/ucb/whoami} r] then {set-footer $r; return ""}
      return $r
   }
}

proc hostname {} {
   global env
   if {$env(platform) == "windows"} then {
      set-footer "/usr/ucb/hostname not supported by windows"
      return "windows"
   } else {
#    assume Unix
      if [catch {exec /usr/ucb/hostname} r] then {set-footer $r; return ""}
      return $r
   }
}

proc read-file {fn} {
   set data {}

   if [catch {open $fn r} f] then {
      midas-warning "Unable to read file $fn\n$f"
      set-footer "Unable to read file $fn"
      return {} 
   }
   set l [read $f]
   close $f

   set ll [split $l \n]
   foreach item $ll {
     if {$item != ""} then {
        set l [split $item]
        if {[first $l] != "#"} then {lappend data $item}
     }
   }
   return $data
}

proc write-file {fn data} {
    global ERROR OK

    set body [join $data \n]
    if [catch {open $fn w+} f] {
       midas-warning "Unable to write file $fn\n$f"
       set-footer "Unable to write file $fn"
       return $ERROR
    }
    puts $f "$body"
    close $f
    return $OK   
}

proc remove-file {f} {
    global ERROR OK

    if [catch {:file delete $f} out] then {return $ERROR}
    return $OK
}

proc set-busy-watch {m} {  
   set-footer $m
   set-busy
}

proc clear-busy-watch {} {
   clear-busy
   clear-footer
}

proc clear-busy-watch-error {} {
   clear-busy
}



#-----------------------------------------
# 
# Print to the main task the message "m"
#
proc print-messages {m} {
    global mytaskfirstname
    set message {}

    if {$m == {}} return
    foreach r $m {if {$r != {}} {lappend message $r}}
    inform $mytaskfirstname display-messages [list $message]
}

proc display-messages {m} {
    :modify MESSAGES  :append T :strings "[join $m \n] \n"
}


#-----------------------------------------
# 
# Editor 
#
proc edit {f} {
   global env ERROR OK editor

   if {$env(platform) == "windows"} then {
      midas-warning "edit not supported by windows"
      return $ERROR
   }
   
   set-footer "Starting $editor editor..."
         
   if {[file isfile $f]} {
      if [catch {exec $editor $f &} err] {
         midas-warning "Error executing command $editor\n\n$err"
	   set-footer "Error executing $editor command"
         return $ERROR
      }
   }  else {
      set-footer "Can't open \"$f\": no such file !!!"
      return $ERROR
   }
      
   return $OK
}
  
#-----------------------------------------
# 
# Editor VI
#   
proc vi {f} {
   global env ERROR OK
   
   if {$env(platform) == "windows"} then {
      midas-warning "vi not supported by windows - use another editor"
      return $ERROR
   }
   
   set i    [string last "/" $f]
   set dir  [string range $f 0 [expr $i-1]]       
   set file [string range $f [expr $i+1] end]
       
   set xterm  $env(OPENWINHOME)/bin/xterm
     
   if {[file isfile $f]} {
      if [catch {exec $xterm +sf -sb -sl 1000 -T "vi : \"$f\"" -n $file -e vi $f &} err] {
         midas-warning "Error executing command $xterm\n\n$err"
	   set-footer "Error executing $xterm command"
         return $ERROR
      }
   } else {
      set-footer "Can't open \"$f\": no such file !!!"
      return $ERROR
   }
      
   return $OK
}


proc pidexist {p} { 
  global env

   if {$env(platform) == "windows"} then {
      midas-warning "/bin/ps not currently supported by windows - unable to continue"
      return $ERROR
   }
            
   if {![catch {exec /bin/ps -o pid -p $p} pid]} {
       if {[second $pid] == $p} then {return 1} else {return 0}
   }
   return 0      
}  

proc progexist {p} {
  global env

   if {$env(platform) == "windows"} then {
      midas-warning "/bin/ps not currently supported by windows - unable to continue"
      return $ERROR
   }

   if {![catch {exec /bin/ps -e -o comm | grep $p} prog]}  {
       if {$prog == ""} then {return 0} else {return 1}
   }
   return 0      
}
   
proc rcp {source_file dest_host dest_file } {
   global env ERROR OK 

   if {$env(platform) == "windows"} then {
      midas-warning "/bin/rcp not currently supported by windows - unable to continue"
      return $ERROR
   }

   set rcp  /bin/rcp
   set z [catch {exec $rcp $source_file $dest_host:$dest_file} err]
   if {$z} {
        midas-warning "Error executing command $rcp $source_file $dest_host:$dest_file\n\n$err"
	  set-footer "Error executing rcp command"
        return $ERROR
   }
   return $OK
}

