#
#    common.orbix.tcl
#
#    procedures used to ORBIX 


#------------------------------------------
# 
# return Orbix psit list or empty list if error
#
proc psit {host} {
    global env ERROR

   if {$env(platform) == "windows"} then {
      midas-warning "Orbix functions not currently supported by windows - unable to continue"
      return $ERROR
   }

    set psit_list {}

    set command $env(ORBIXHOME)/bin/psit

    set z [catch {exec $command -h $host} m]
    if {$z} {
        midas-warning "Error executing Orbix command $command\n\n$m"
	set-footer "Error executing Orbix command $command"
        return $ERROR
    }
    set ll [split $m "\n"]
    if {[first [first $ll]] == "SYSTEM_EXCEPTION"} {
        midas-warning "Error executing Orbix command $command\n\n$m"
	set-footer "Error executing Orbix command $command"
        return $ERROR
    }
    set lg [llength $ll]
    if {$lg < 3} {return {}}
    for {set i 3} {$i < $lg} {incr i} {
        set tmp [first [lindex $ll $i]]
        set server [first [split $tmp "*"]]
        if {$server != ""} {lappend psit_list $server}
    }

    return [lsort $psit_list]
}

#-----------------------------------------
# 
# return messages Orbix killit list or ERROR if error
#
proc killit {host server} {
    global env ERROR OK

   if {$env(platform) == "windows"} then {
      midas-warning "Orbix functions not currently supported by windows - unable to continue"
      return $ERROR
   }

    set mess_list {}
    set active_servers    [psit $host]
    set implement_servers [lsit $host]

    set kill_command $env(ORBIXHOME)/bin/killit
    set rm_command   $env(ORBIXHOME)/bin/rmit

    foreach s $active_servers {
	if {$server == $s} {
    	    lappend mess_list "Killing $server on machine $host"
    	    set z [catch {exec $kill_command -h $host $server} m]
            if {$z} {
               midas-warning "Error executing Orbix command $kill_command\n\n$m"
	       set-footer "Error executing Orbix command $command"
               return $ERROR
            }
            lappend mess_list $m
	}
    }
    foreach s $implement_servers {
	if {$server == $s} {
    	    lappend mess_list "Removing $server on machine $host"
    	    set z [catch {exec $rm_command -h $host $server} m]
            if {$z} {
               midas-warning "Error executing Orbix command $rm_command\n\n$m"
	       set-footer "Error executing Orbix command $command"
               return $ERROR
            }
            lappend mess_list $m
	}
    }
    return $mess_list
}


#------------------------------------------
# 
# return Orbix lsit list or empty list if error
#
proc lsit {host} {
    global env ERROR

   if {$env(platform) == "windows"} then {
      midas-warning "Orbix functions not currently supported by windows - unable to continue"
      return $ERROR
   }

    set lsit_list {}

    set command $env(ORBIXHOME)/bin/lsit

    set z [catch {exec $command -h $host} m]
    if {$z} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }
    set ll [split $m \n]
    if {[first [first $ll]] == "SYSTEM_EXCEPTION"} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }

    set lg [llength $ll]
    for {set i 1} {$i < $lg} {incr i} {
        set server [first [lindex $ll $i]]
        if {$server != ""}  {lappend lsit_list $server}
    }

    return [lsort $lsit_list]
}

#------------------------------------------
# 
# return OK or ERROR if error
#
proc Rmit {host server} {
    global env ERROR OK

   if {$env(platform) == "windows"} then {
      midas-warning "Orbix functions not currently supported by windows - unable to continue"
      return $ERROR
   }

    set command $env(ORBIXHOME)/bin/rmit

    set z [catch {exec $command -h $host $server} m]
    if {$z} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }
    return $OK
}


#------------------------------------------
# 
# return OK or ERROR if error
#
proc Killit {host server} {
    global env ERROR OK

   if {$env(platform) == "windows"} then {
      midas-warning "Orbix functions not currently supported by windows - unable to continue"
      return $ERROR
   }

    set command $env(ORBIXHOME)/bin/killit

    set z [catch {exec $command -h $host $server} m]
    if {$z} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }
    return $OK
}
    
#------------------------------------------
# 
# return Orbix catit list or empty list if error
#
proc catit {host server} {
    global env ERROR

   if {$env(platform) == "windows"} then {
      midas-warning "Orbix functions not currently supported by windows - unable to continue"
      return $ERROR
   }

    set cl {}

    set command $env(ORBIXHOME)/bin/catit

    set z [catch {exec $command -h $host $server} m]
    if {$z} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }
    set ll [split $m \n]
    if {[first [first $ll]] == "SYSTEM_EXCEPTION"} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }

    lappend cl " Name\t\t: $server"
    lappend cl " Host\t\t: $host"
    set index 2;
    foreach line $ll {
        switch [first $line] {
	    Comms      {lappend cl " Comms Protocol\t: [lrange $line 2 end]"}
	    Code       {lappend cl " Data Encoder\t: [lrange $line 2 end]"}
	    Activation {lappend cl " Activation\t: [lrange $line 2 end]"}
	    Owner      {lappend cl " Owner\t\t: [lrange $line 2 end]"}
	    Launch     {lappend cl " Launch\t\t: [lrange $line 2 end]"}
	    Invoke     {lappend cl " Invoke\t\t: [lrange $line 2 end]"}
	    Marker     {break}
	}
        incr index
    }

    set line [lindex $ll $index]

    lappend cl " Marker\t\t: [first $line]"
    lappend cl " Launch Command\t: [lrange $line 1 end]"

    return $cl
}

#------------------------------------------
# 
# return Orbix catit list or empty list if error
#
proc catit_process {host server} {
    global env ERROR

   if {$env(platform) == "windows"} then {
      midas-warning "Orbix functions not currently supported by windows - unable to continue"
      return $ERROR
   }

    set cl {}
    set begin 0

    set command $env(ORBIXHOME)/bin/psit

    set z [catch {exec $command -h $host} m]
    if {$z} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }
    set ll [split $m \n]
    if {[first [first $ll]] == "SYSTEM_EXCEPTION"} {
       midas-warning "Error executing Orbix command $command\n\n$m"
       set-footer "Error executing Orbix command $command"
       return $ERROR
    }

    lappend cl " Name\t\t: $server"
    lappend cl " Host\t\t: $host"

    set lg [llength $ll]
    if {$lg < 3} {return {}}
    for {set i 3} {$i < $lg} {incr i} {
        set line [lindex $ll $i]
        set tmp [first $line]
        set srv [first [split $tmp "*"]]
        if {$srv == $server} {
 	   set begin [expr [llength $line] - 8]
	   lappend cl " Comms Protocol\t: [lindex $line [expr $begin + 3]]"
	   lappend cl " Data Encoder\t: [lindex $line [expr $begin + 2]]"
	   lappend cl " Port Number\t: [lindex $line [expr $begin + 4]]"
	   lappend cl " Launch Status\t: [lindex $line [expr $begin + 5]]"
	   lappend cl " Process Identifier\t: [lindex $line [expr $begin + 7]]"
	   lappend cl " PerClient?\t: [lindex $line [expr $begin + 6]]"
	}
    }
    return $cl
}



   
