# ec.tcl     Euroball Event Collector control

#  12-6-96   VP   first version
#  13-11-96  VP/SCL  modified to use EC register server

set debug 0
set started 0

set frame EC
set frametitle "Euroball Event Collector Control & Monitor"
set icontitle  EC

if {![info exists env(EB_ETC)]}    then {set env(EB_ETC) [file join euroball etc]}
if {![info exists env(EB_CONFIG)]} then {set env(EB_CONFIG) [file join euroball config]}

set maxec 6

#    ECs
set eclist {0 ec1 ec2 ec3 ec4 ec5 ec6}

#    available ECs - built dynamically
set ecchoice {ec1 ec2 ec3 ec4 ec5 ec6}

#    apparatus names for ECs
#    index 0 is not used because this is used in ecappname (see following)
#    to indicate item not in use
set appnames {dummy ec1 ec2 ec3 ec4 ec5 ec6}

#   define index into appnames for "apparatus name" of each EC
#     updated dynamically
for {set i 1} {$i <= $maxec} {incr i} {set ecappname($i)  0}

#    all generic register names start with "prefix"
set prefix EC

#    for each register there exists a list item
#    each item contains   register name
#                         VxWorks variable name
#                         variable type and further attributes
#                         associated widget (if any)


set EC_registers { {state      EC_STATE       int         ECSTATE}
                   {num_dest   EC_NUM_DEST    int         ""}
                   {num_block  EC_NUM_BLOCK   int         ""}
                   {buf_size   EC_BUF_SIZE    int         ""}
                   {ford       EC_FORD        int         ""}
                   {myaddr     EC_myaddr      ulong       ""}
                   {status     EC_status      int         ""}
                   {flag       EC_flag        int         ""}
                   {bootf      EC_bootf       string      ""}
                   {dumpf      EC_dumpf       string      ""}
                   {inetaddr   EC_inetaddr    "ulong -a 1 -i 8"   ""}
                   {fcs_port   EC_fcs_port    "ulong -a 1 -i 8"   ""}
                   {setup      EC_setup       function    ""}
                   {reset      EC_reset       function    ""}
                   {start      EC_start       function    ""}
                   {stop       EC_stop        function    ""}
                   {ping       EC_ping        function    ""}
                   {flush      EC_flush       function    ""}
                   {realign    EC_realign     function    ""}
                   {dump       EC_dump        function    ""}
                   {clear_stat EC_clear_stat  function    ""}
                 }

set EC_stats_registers {
                   {DS.state    DSstate        "int -a 1 -i 8"   DSSTATE}
                   {DS.nobs     DSnobs         "int -a 1 -i 8"   DSNOBS}
                   {DS.ave      DSave          "int -a 1 -i 8"   DSAVE}
                   {DD.state    DDstate        "int -a 1 -i 8"   DDSTATE}
                   {DD.nobs     DDnobs         "int -a 1 -i 8"   DDNOBS}
                   {DD.ave      DDave          "int -a 1 -i 8"   DDAVE}
                   {DD.noreqs   DDnoreqs       "int -a 1 -i 8"   DDNOREQS}
                   {DD.aver     DDaver         "int -a 1 -i 8"   DDAVER}
                   {DSP.state   DSPstate       "int -a 1 -i 8"   DSPSTATE}
                   {DSP.pack    DSPpack        "int -a 1 -i 8"   DSPPACK}
                   {DSP.avep    DSPavep        "int -a 1 -i 8"   DSPAVEP}
                   {DSP.noev    DSPnoev         int              DSPNOEV}
                   {DSP.ave     DSPave          int              DSPAVE}
                   {DSP.maxl    DSPmaxl         int              DSPMAXL}
                   {DSP.avel    DSPavel         int              DSPAVEL}
                 }

#    apparatus role for ECs
set resource EC

#    maximum available destinations
set maxdestn 12

set maxqsize 64

set STATEchoice {Idle Configured Running Active}

set buffchoice {16000 32000 64000}

set rochoice {"old FERA" "new FERA" "DT32"}

#    choice of ethernet IP names/address for ECs
set enetnames {dummy 255.255.255.255}

#    choice of FC IP names/address for ECs
set fcnames   {dummy 255.255.255.255}

#    currently selected EC
set ecnumber 1

#   define index into enetnames for IP address on ethernet of each EC
for {set i 1} {$i <= $maxec} {incr i} {set ecenetaddr($i)  $i}

#   define index into fcnames for IP address on fibre channel of each EC
for {set i 1} {$i <= $maxec} {incr i} {set ecfcaddr($i)  $i}

#    define index into rochoice for each EC (first is 0)
for {set i 1} {$i <= $maxec} {incr i} {set ecromode($i)  2}

#    default value for buffsize (16000) (this is index into buffchoice list)
set ecbuffsize 1

#    default value for q size
set ecqsize 20

#    default value for number of destinations
set ecnumdest 6

#    default destinations - should be maxdestn items
for {set i 1} {$i <= $maxdestn} {incr i} {set ecdestn($i) ""}

#    Data Source switches
for {set i 1} {$i <= $maxdestn} {incr i} {set ecds($i) 1}

#    Data Destination switches
for {set i 1} {$i <= $maxdestn} {incr i} {set ecdd($i) 1}

#    default file path names
for {set i 1} {$i <= $maxec} {incr i} {
   set ecbootfile($i) "/MIDAS/boot/ec/ec_dsp.out"
   set ecdumpfile($i) "/tmp/EC[set i]-dump"
}

#    options for formatted dump
set dumpstart 0
set dumpwords 512
set dumpopt 2
set destntodump 0

set dumpdata ""

set allecs 0

#proc EG {args} {insert-debug "executing: $args"; return 2}
#proc EG {args} {return 2}

#    The following procedure obtains details of the current
#    EC configuration from the file /euroball/config/MIDAS (by default)
#    It also contains the FCS IP addresses of the processor farm from this file
#    The current event farm configuration is obtained from the file
#    /euroball/etc/evFarm.conf (by default)
proc set-config {} {
   global env
   global enetnames fcnames
   global ecdestn ecnumdest maxdestn
   global ecds ecdd
   set file [file join $env(EB_CONFIG) MIDAS]
   set z [catch {open $file r} f]
   if {$z != 0} then {midas-warning "Unable to read configuration file $file\n$f\n\n\
      This must be corrected before you can continue"; return}
   set config [split [read $f] \n]
   close $f
   set enetnames {dummy}
   set fcnames {dummy}
   foreach item $config {
      set l [string trimleft [string trimright $item]]
      if {$l == ""} then continue
      if {[string range $l 0 0] == "#"} then continue
      set command [first $l]
      switch $command {
         ec  {
            lappend enetnames [third $l]
            lappend fcnames [fourth $l]
         }
         host {
            set hostaddr([second $l]) [third $l]
         }
         default {}
      }
   }
   :modify ENETIPADDR :strings $enetnames :value 1
   :modify FCIPADDR :strings $fcnames :value 1
   set file [file join $env(EB_ETC) evFarm.conf]
   set z [catch {open $file r} f]
   if {$z != 0} then {midas-warning "Unable to read configuration file $file\n$f\n\n\
      This must be corrected before you can continue"; return}
   set config [split [read $f] \n]
   close $f
   set ecnumdest 0
   foreach item $config {
      set l [string trimleft [string trimright $item]]
      if {$l == ""} then continue
      if {[string range $l 0 0] == "#"} then continue
      incr ecnumdest
      set host [first $l]
      set numcpus [second $l]
      set port [last [split $hostaddr($host) .]]
      set ecdestn($ecnumdest) "$hostaddr($host) $port"
      if {$numcpus == 2} then {
         incr ecnumdest
         set ecdestn($ecnumdest) "$hostaddr($host) $port"
      }
   }
   :modify NUMDEST :value $ecnumdest

   for {set i 1} {$i <= $maxdestn} {incr i} {
      :modify DESTN$i  :value "" :inactive T
      set ecds($i) 0
      set ecdd($i) 0
      :modify DS$i     :inactive F :value 0 :inactive T
      :modify DD$i     :inactive F :value 0 :inactive T
   }
   for {set i 1} {$i <= $ecnumdest} {incr i} {
      :modify DESTN$i  :inactive F :value $ecdestn($i)
      set ecds($i) 1
      set ecdd($i) 1
      :modify DS$i     :inactive F :value "$ecds($i)"
      :modify DD$i     :inactive F :value "$ecdd($i)"
   }
}


#    the following are used if the EC cannot be reached or
#    if it is currently in the Idle state
proc set-defaults {} {
  global ecnumber maxdestn ecnumdest
  global ecbuffsize ecqsize ecromode ecbootfile ecdumpfile
  global ecds ecdd
      set ecbuffsize 0
      set ecqsize 20
      set ecromode($ecnumber)  2
      set ecbootfile($ecnumber) "/MIDAS/boot/ec/ec_dsp.out"
      set ecdumpfile($ecnumber) "/tmp/EC[set ecnumber]-dump"
  
      for {set i 0; set j 1} {$i < $maxdestn} {incr i; incr j} {
         set ecds($j) 0
         set ecdd($j) 0
      }
      for {set i 0; set j 1} {$i < $ecnumdest} {incr i; incr j} {
         set ecds($j) 1
         set ecdd($j) 1
      }
}

proc make-EC-list {} {
  global ecchoice
   set z [catch {eg "(experiment ?resources)"} m]
   if {$z==0} then {set rs $m} else {return $ecchoice}
   set ll {}
   foreach r $rs {
#    first scan through resources and all servers of role EC to the list
      set name [second [first $r]]
      set type [second [fourth $r]]
      set role [second [fifth $r]]
      if {$type == "register" && $role == "EC"} then {lappend ll $name}
   }
   return $ll
}

proc create-register {name class} {
   global prefix appnames ecappname ecnumber
   set resource [lindex $appnames $ecappname($ecnumber)]
   set reg /$resource/[set prefix].$name
   return [catch {EG "register :name '$reg' :class '$class' !new"} m ]
}

proc write-attribute {name attr} {
   global prefix appnames ecappname ecnumber
   set resource [lindex $appnames $ecappname($ecnumber)]
   set reg /$resource/[set prefix].$name
   EG "register :name '$reg' :object '$attr'"
   clear-footer
}

proc read-attribute {name} {
   global prefix appnames ecappname ecnum
   set resource [lindex $appnames $ecappname($ecnum)]
   set reg /$resource/[set prefix].$name
   EG "register :name '$reg' ?object"
   clear-footer
}

proc read-register {name} {
   global prefix appnames ecappname ecnumber
   set resource [lindex $appnames $ecappname($ecnumber)]
   set reg /$resource/[set prefix].$name
   set value [EG "register :name '$reg' ?value"]
   clear-footer
   return $value
}

proc function-register {name} {
   global prefix appnames ecappname ecnumber
   global STATEchoice

   set resource [lindex $appnames $ecappname($ecnumber)]
   set reg /$resource/[set prefix].$name
#    change EC state to Active to prevent simultaneous access
   set currentstate [read-register state]
   write-register state [lsearch $STATEchoice Active]
   :modify ECSTATE :value Active
#    clear EC_flag
   write-register flag 0
#    execute the function
   set value [read-register $name]
#    wait for EC_flag to be set when function ends - also set timeout
   set flag 0; set count 0
   while {$flag == 0 && $count < 100} {set flag [read-register flag]; incr count} 
   if {$count > 99} then {midas-warning "EC response timeout - function $name will not complete"}
   set status [read-register status]

   if {$status != 1} then {midas-warning "EC function $name failed - EC return status is $status"}

#   if {$status != 1} then {
#        if {$name == "reset" && $status == 134} then {continue} \
#          else {midas-warning "EC function $name failed - EC return status is $status"}
#   }

#  set EC state back 
   write-register state $currentstate
   clear-footer
   return $value
}

proc read-registers {pattern} {
   global prefix appnames ecappname ecnumber
   set resource [lindex $appnames $ecappname($ecnumber)]
   set rr /$resource/[set prefix].$pattern
   set values [EG "register :pattern '$rr' ?names-and-values"]
   clear-footer
   return $values
}

proc write-register {name value} {
   global prefix appnames ecappname ecnumber
   set resource [lindex $appnames $ecappname($ecnumber)]
   set reg /$resource/[set prefix].$name
   EG "register :name '$reg' :value '$value'"
   clear-footer
}

proc IP2hex {ip} {
   set v 0x

   foreach n [split $ip .] {
      set v [concat $v[format "%02X" $n]]
   }
   return $v
}

proc do-initialise {ecnum} {
# based on eb10.config.sh
global EC_registers EC_stats_registers
global ecappname

   if {$ecappname($ecnum) == 0} then {return}

   set class generic

   foreach item $EC_registers {
      set reg [first $item]
      set variable [second $item]
      set qualifier [third $item]
      set attribute "-n $variable -t $qualifier"

      create-register $reg $class
      write-attribute $reg $attribute
   }

   foreach item $EC_stats_registers {
      set reg [first $item]
      set variable [second $item]
      set qualifier [third $item]
      set attribute "-n $variable -t $qualifier"

      create-register $reg $class
      write-attribute $reg $attribute
   }
}

#    select ec to access - for use by external tasks
proc select-ec {v} {
    global ecnumber
    set ecnumber $v
    redisplay
}
    
proc do-setup {ecnum} {
# based on eb10.setup.sh
global ecappname 
global maxec maxdestn buffchoice enetnames fcnames
global ecenetaddr ecfcaddr ecromode
global ecbuffsize ecqsize ecnumdest ecbootfile ecdumpfile ecdestn
global STATEchoice

   if {$ecappname($ecnum) == 0} then {return}
   
#    setup can only be issued in EC state idle

   set state [read-register state]
   if {$state != [lsearch $STATEchoice Idle]} then {
      set ecstate [lindex $STATEchoice $state]
      midas-warning "You cannot use Setup when EC is in state $ecstate"
      return
   }

# number of destinations
   write-register num_dest $ecnumdest
# number of memory blocks in one queue
   write-register num_block $ecqsize
# the size of one buffer (in bytes, without the UDP header)
   write-register buf_size [lindex $buffchoice $ecbuffsize]
# code for read-out modules: 0: old FERA 1: new FERA 2: DT32
   write-register ford $ecromode($ecnum)

# 192.168.1.10 (0xC0A8010A)
   set myaddr [lindex $fcnames $ecfcaddr($ecnum)]
# IP address (Fibre Channel)
   write-register myaddr [IP2hex $myaddr]
   write-register status 0
   write-register flag 0

# boot file
   write-register bootf $ecbootfile($ecnum)
# dump file
   write-register dumpf $ecdumpfile($ecnum)

# internet addresses and FCS physical port numbers of destinations
#destination 192.168.1.3 (0xC0A80103) 3
   for {set i 0; set j 1} {$i < $ecnumdest} {incr i; incr j} {
      write-attribute inetaddr "-i $i"
      write-register inetaddr [IP2hex [first $ecdestn($j)]]
      write-attribute fcs_port "-i $i"
      write-register fcs_port [second $ecdestn($j)]
   }

#call setup function
   read-register setup
#    change EC state
   write-register state [lsearch $STATEchoice Configured]
}

proc do-start {ecnum} {
global ecappname
global STATEchoice

#    start can only be issued in EC state Configured

   set action start
   if {$ecappname($ecnum) != 0} then {
      set state [read-register state]
      if {$state == [lsearch $STATEchoice Configured]} then {
         function-register $action
#    change EC state
         write-register state [lsearch $STATEchoice Running]
      } else {
         set ecstate [lindex $STATEchoice $state]
         midas-warning "You cannot use $action when EC is in state $ecstate"
      }
   }
}

proc do-stop {ecnum} {
global ecappname
global STATEchoice

#    stop can only be issued in EC state Running
#    but we should ignore request in states Idle & Configured

   set action stop
   if {$ecappname($ecnum) != 0} then {
      set state [read-register state]
      if {$state == [lsearch $STATEchoice Running]} then {
         function-register $action
#    change EC state
         write-register state [lsearch $STATEchoice Configured]
      } else {
         set ecstate [lindex $STATEchoice $state]
         if {$state != [lsearch $STATEchoice Idle] && $state != [lsearch $STATEchoice Configured]} then {
            midas-warning "You cannot use $action when EC is in state $ecstate"
         }
      }
   }
}

proc do-reset {ecnum} {
global ecappname
global STATEchoice

#    this action can only be issued in EC state Configured and Running
#    state becomes Idle

   set action reset
   if {$ecappname($ecnum) != 0} then {
      set state [read-register state]
      if {$state == [lsearch $STATEchoice Running] || $state == [lsearch $STATEchoice Configured]} then {
         function-register $action
         write-register state [lsearch $STATEchoice Idle]
      } else {
         set ecstate [lindex $STATEchoice $state]
         if {$ecstate != "Idle"} then {
            midas-warning "You cannot use $action when EC is in state $ecstate"
         }
      }
   }
}

proc do-common-function {action ecnum} {
global ecappname
global STATEchoice

#    this action can only be issued in EC state Configured and Running

   if {$ecappname($ecnum) != 0} then {
      set state [read-register state]
      if {$state == [lsearch $STATEchoice Running] || $state == [lsearch $STATEchoice Configured]} then {
         function-register $action
      } else {
         set ecstate [lindex $STATEchoice $state]
         midas-warning "You cannot use $action when EC is in state $ecstate"
      }
   }
}

proc do-ping {ecnum}    {do-common-function ping $ecnum}
proc do-flush {ecnum}   {do-common-function flush $ecnum}
proc do-realign {ecnum} {do-common-function realign $ecnum}
proc do-dump {ecnum}    {do-common-function dump $ecnum}
proc do-clear_stat {ecnum}   {do-common-function clear_stat $ecnum}

proc do-dd {ecnum} {
   global ecnumdest ecdd ecappname
#    write the data flow switches
   if {$ecappname($ecnum) != 0} then {
      for {set i 0; set j 1} {$i < $ecnumdest} {incr i; incr j} {
         write-attribute DD.state "-i $i"
         write-register DD.state $ecdd($j)
      }
   }
}

proc do-ds {ecnum} {
   global ecnumdest ecds ecappname
#    write the data flow switches
   if {$ecappname($ecnum) != 0} then {
      for {set i 0; set j 1} {$i < $ecnumdest} {incr i; incr j} {
         write-attribute DS.state "-i $i"
         write-register DS.state $ecds($j)
      }
   }
}

proc update-stats {} {
  global statsvisible statsupdate
  global ecnumdest ecnumber
  global EC_stats_registers

  if {$statsvisible == 0} then {return}

  :modify SECNUM :value $ecnumber

#    first update stats per destination
   for {set i 0} {$i < $ecnumdest} {incr i} {
      foreach item $EC_stats_registers {
         set reg [first $item]
         set type [third $item]
         if {"$type" != "int"} then {
            write-attribute $reg "-i $i"
            set vl [read-register $reg]
            set w [fourth $item]$i
            :modify $w :value $vl
         }
      }
   }
#    then global DSP stats
   foreach item $EC_stats_registers {
      set reg [first $item]
      set type [third $item]
      if {"$type" == "int"} then {
            set vl [read-register $reg]
            set w [fourth $item]
            :modify $w :value $vl
      }
   }
   if {$statsupdate > 0} then {after [expr $statsupdate * 10000] update-stats}
}

proc click-action action {
  global ecnumber ecdumpfile
#    perform EC action
   do-[lowercase $action] $ecnumber
   if {$action == "DUMP"} then {set-footer "data dump is left in file $ecdumpfile($ecnumber)"} \
      else {clear-footer}
}

proc click-onlinedump {} {
  global destntodump dumpdata dumpstart dumpwords

   set-footer "Starting Online Data Dump"
#    wait for EC_dump_full to be zero - also set timeout
   set flag 1; set count 0
   while {$flag != 0 && $count < 100} {set flag [read-register dump_full]; incr count} 
   if {$count > 99} then {
     write-register dump_request 0
     midas-warning "EC dump function in-use"
     return
   }
#    set which destination buffer to dump
   write-register dump_index $destntodump 
#    request buffer copy
   set-footer "Requesting Data Copy"
   write-register dump_request 1
#    wait for EC_dump_full to be set - also set timeout
   set flag 0; set count 0
   while {$flag != 1 && $count < 1000} {set flag [read-register dump_full]; incr count} 
   if {$count > 999} then {
      write-register dump_request 0
      midas-warning "EC dump function not responding"
      return
   }
#    get address pointer to data buffer
   set dumpaddr [read-register dump_addr]
#    read data
   set-footer "Fetching data"
   set fragsize 64
   set dumpend  [expr $dumpstart + $dumpwords]
   set dumpdata ""
   for {set i $dumpstart} {$i <= $dumpend} {incr i $fragsize} {
      write-attribute dump_data "-a $dumpaddr -o [expr $i * 4] -l $fragsize"
      append dumpdata [read-register dump_data]
   }
#    free dump buffer
   write-register dump_request 0
#
   clear-footer
   format_the_dump
}

proc click-formatdump {} {
  global ecnumber ecdumpfile dumpdata

   set-footer "reading dump"
#    read the dump file
   exec /usr/bin/od -Xv $ecdumpfile($ecnumber) > /tmp/ECdump[pid]
   set dumpdata [exec /usr/bin/head -2000 /tmp/ECdump[pid]]
#
   clear-footer
   format_the_dump
}

proc format_the_dump {} {
  global dumpdata
  global dumpopt dumpstart dumpwords
  global ancillary1 ancillary2 ancillary3 ancillaryBase1 ancillaryBase2 ancillaryBase3

#    format dumpdata held in global variable dumpdata

   set-footer "formatting dump"

   set frame "dump"
   :frame [self frame-name].$frame :label  "Event Collector Dump"  :transient T
   wm protocol [self frame-name].$frame WM_DELETE_WINDOW {destroy [self frame-name].dump}

   :font local text n :fmly courier

   switch $dumpopt {
   0  {
        :galley G :rows 50 :cols 50 :deaf T :strings "" :font fixed
        :show T
        :modify G :strings "raw dump starting at word offset 0\n"
        :modify G :strings $dumpdata
      }

   1  {
        :galley G :rows 50 :cols 50 :deaf T :strings "" :font fixed
        :show T
        set end [expr $dumpstart + $dumpwords]
        :modify G :strings "dump starting at word offset $dumpstart for $dumpwords words\n"
        set dump ""
        for {set i 0} {$i <= $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           if {$group >= 248 && $item == 63} then {
#    last end token for event
             append dump "$word "
             :modify G :strings $dump
             :modify G :strings "\n\n"
             set dump ""
           } else {
             append dump "$word "
           }
        }
        :modify G :strings $dump
     }
   2  {
        :galley G :rows 50 :cols 60 :deaf T :strings "" :font fixed
        :show T
        set end [expr $dumpstart + $dumpwords]
        :modify G :strings "dump starting at word offset $dumpstart for $dumpwords words\n"
        for {set i 0} {$i <= $dumpwords} {incr i} {
           set word [lindex $dumpdata $i]
           set group [expr $word >> 16 & 255]
           set item  [expr $word >> 24 & 63]
           set qbits [expr $word >> 30 & 3]
           set data  [expr $word & 0x0000ffff]
           set title "Unknown"
           if {$group <= 0} then {
#              invalid
               set title "illegal"
           } elseif {$group <= 20} then {
#               Euroball III cards (Cluster)
              set detector [expr $group - 1]
              switch $item {
              0   {set title "Cluster$detector.GeA.20MeV"}
              1   {set title "Cluster$detector.GeA.4MeV"}
              2   {set title "Cluster$detector.GeA.FT"}
              3   {set title "Cluster$detector.GeB.20MeV"}
              4   {set title "Cluster$detector.GeB.4MeV"}
              5   {set title "Cluster$detector.GeB.FT"}
              6   {set title "Cluster$detector.GeC.20MeV"}
              7   {set title "Cluster$detector.GeC.4MeV"}
              8   {set title "Cluster$detector.GeC.FT"}
              9   {set title "Cluster$detector.GeD.20MeV"}
              10  {set title "Cluster$detector.GeD.4MeV"}
              11  {set title "Cluster$detector.GeD.FT"}
              12  {set title "Cluster$detector.GeE.20MeV"}
              13  {set title "Cluster$detector.GeE.4MeV"}
              14  {set title "Cluster$detector.GeE.FT"}
              15  {set title "Cluster$detector.GeF.20MeV"}
              16  {set title "Cluster$detector.GeF.4MeV"}
              17  {set title "Cluster$detector.GeF.FT"}
              18  {set title "Cluster$detector.GeG.20MeV"}
              19  {set title "Cluster$detector.GeG.4MeV"}
              20  {set title "Cluster$detector.GeG.FT"}
              21  {set title "Cluster$detector.BGOSum.Energy"}
              22  {set title "Cluster$detector.BGOSum.TAC"}
              23  {set title "Cluster$detector.BGO.Pattern.High"}
              24  {set title "Cluster$detector.BGO.Pattern.Low"}
              25  {set title "Cluster$detector.BGOA.Energy"}
              26  {set title "Cluster$detector.BGOA.TAC"}
              27  {set title "Cluster$detector.BGOB.Energy"}
              28  {set title "Cluster$detector.BGOB.TAC"}
              29  {set title "Cluster$detector.BGOC.Energy"}
              30  {set title "Cluster$detector.BGOC.TAC"}
              31  {set title "Cluster$detector.BGOD.Energy"}
              32  {set title "Cluster$detector.BGOD.TAC"}
              33  {set title "Cluster$detector.BGOE.Energy"}
              34  {set title "Cluster$detector.BGOE.TAC"}
              35  {set title "Cluster$detector.BGOF.Energy"}
              36  {set title "Cluster$detector.BGOF.TAC"}
              37  {set title "Cluster$detector.BGOG.Energy"}
              38  {set title "Cluster$detector.BGOG.TAC"}
              39  {set title "Cluster$detector.BGOH.Energy"}
              40  {set title "Cluster$detector.BGOH.TAC"}
              41  {set title "Cluster$detector.BGOI.Energy"}
              42  {set title "Cluster$detector.BGOI.TAC"}
              43  {set title "Cluster$detector.BGOJ.Energy"}
              44  {set title "Cluster$detector.BGOJ.TAC"}
              45  {set title "Cluster$detector.BGOK.Energy"}
              46  {set title "Cluster$detector.BGOK.TAC"}
              47  {set title "Cluster$detector.BGOL.Energy"}
              48  {set title "Cluster$detector.BGOL.TAC"}
              49  {set title "Cluster$detector.BGOM.Energy"}
              50  {set title "Cluster$detector.BGOM.TAC"}
              51  {set title "Cluster$detector.BGON.Energy"}
              52  {set title "Cluster$detector.BGON.TAC"}
              53  {set title "Cluster$detector.BGOO.Energy"}
              54  {set title "Cluster$detector.BGOO.TAC"}
              55  {set title "Cluster$detector.BGOP.Energy"}
              56  {set title "Cluster$detector.BGOP.TAC"}
              57  {set title "Cluster$detector.BGOQ.Energy"}
              58  {set title "Cluster$detector.BGOQ.TAC"}
              59  {set title "Cluster$detector.BGOR.Energy"}
              60  {set title "Cluster$detector.BGOR.TAC"}
              62  {set title "Cluster$detector.Ge.GIR.EvNum"}
              61  {set title "Cluster$detector.BGO.GIR.EvNum"}
              default  {}
              }
           } elseif {$group <= 50} then {
#               Eurogam II cards (Clover)
              set detector [expr $group - 21]
              switch $item {
              0   {}
              1   {}
              2   {}
              3   {}
              4   {set title "Clover$detector.BGO.Energy"}
              5   {set title "Clover$detector.BGO.TAC"}
              6   {set title "Clover$detector.BGO.Pattern"}
              7   {}
              8   {set title "Clover$detector.GeA.20MeV"}
              9   {set title "Clover$detector.GeA.4MeV"}
              10  {set title "Clover$detector.GeA.FT"}
              11  {set title "Clover$detector.GeA.CO"}
              12  {set title "Clover$detector.GeB.20Mev"}
              13  {set title "Clover$detector.GeB.4MeV"}
              14  {set title "Clover$detector.GeB.FT"}
              15  {set title "Clover$detector.GeB.CO"}
              16  {set title "Clover$detector.GeC.20MeV"}
              17  {set title "Clover$detector.GeC.4MeV"}
              18  {set title "Clover$detector.GeC.FT"}
              19  {set title "Clover$detector.GeC.CO"}
              20  {set title "Clover$detector.GeD.20MeV"}
              21  {set title "Clover$detector.GeD.4MeV"}
              22  {set title "Clover$detector.GeD.FT"}
              23  {set title "Clover$detector.GeD.CO"}
              default  {}
              }
           } elseif {$group <= 90} then {
#               Eurogam I cards  (Ge and BGO)
              set detector [expr $group - 51]
              switch $item {
              0   {set title "Ge$detector.20MeV"}
              1   {set title "Ge$detector.4MeV"}
              2   {set title "Ge$detector.FT"}
              3   {set title "Ge$detector.CO"}
              4   {set title "BGO$detector.Energy"}
              5   {set title "BGO$detector.TAC"}
              6   {set title "BGO$detector.Pattern"}
              default  {}
              }  
           } elseif {$group < 109} then {
#               BGO inner ball detectors
               set detector [expr $group - 91]
              switch $item {
              0   {set title "BGOBall$detector.Energy"}
              1   {set title "BGOBall$detector.Pattern"}
              2   {set title "BGOBall$detector.TAC"}
              default  {}
              }
           } elseif {$group < $ancillaryBase2} then {
#               ancillary detector type I
               set detector [expr $group - $ancillaryBase1]
               set title "$ancillary1$detector.item$item"
           } elseif {$group < $ancillaryBase3} then {
#               ancillary detector type II
               set detector [expr $group - $ancillaryBase2]
               set title "$ancillary2$detector.item$item"
           } elseif {$group < 246} then {
#               ancillary detector type III
               set detector [expr $group - $ancillaryBase3]
               set title "$ancillary3$detector.item$item"
            } elseif {$group == 246} then {
#               trigger
              switch $item {
              0   {set title "MasterTrigger.SumBus1"}
              1   {set title "MasterTrigger.SumBus2"}
              2   {set title "MasterTrigger.SumBus3"}
              3   {set title "MasterTrigger.SumBus4"}
              4   {set title "MasterTrigger.TAC"}
              5   {set title "MasterTrigger.Pattern"}
              6   {set title "MasterTrigger.EvNumLSB"}
              7   {set title "MasterTrigger.EvNumMSB"}
              default  {}
              }
           } elseif {$group == 247} then {
#               VRE + other diagnostics including Pulser
              switch $item {
              0   {set title "VXI Diag.   VRE Time Out"}
              1   {set title "Neutron Wall Pulser Adc1"}
              2   {set title "Neutron Wall Pulser Adc2"}
              3   {set title "Neutron Wall Pulser Adc3"}
              4   {set title "Neutron Wall Pulser Adc4"}
              5   {set title "Neutron Wall Pulser Adc5"}
              6   {set title "Euclides Pulser Crate1"}
              7   {set title "Euclides Pulser Crate2"}
              10   {set title "VRS (NWall) LUT failure"}
              11   {set title "VRS (NWall) error"}
              12   {set title "FVI LUT failure"}
              default  {set title "diagnostic"}
              }
           } else {
#               tokens 
              if {$item <= 15} then {set title "start token"} \
              elseif {$item <= 31} then {set title "event number lsw"} \
              elseif {$item <= 47} then {set title "event number msw"} \
              else {set title "end token"}
           }
           :modify G :strings "[format "%10s %3d %3d %4d %6x %24s" $word $qbits $item $group $data $title] \n"
           if {$item == 63} then {:modify G :strings "\n"}
       }
     }

   }
   :modify G :see 0
   clear-footer
}

proc redisplay {} {
  global EGreport eclist ecnumber ecenetaddr ecfcaddr 
  global maxdestn ecdestn ecdd ecds
  global ecappname
  global STATEchoice buffchoice ecchoice
  global ecnumdest ecbuffsize ecqsize ecromode ecbootfile ecdumpfile

   if {![winfo ismapped .]} then {return}

   set EGreport_save $EGreport
   if {$EGreport == 0} then {set EGreport -1}
   set-busy

   set class generic
   if {$ecappname($ecnumber) == 0} then {return}

   set z [catch {read-register state} m]
   if {$z == 0} then {
      set ecstate [lindex $STATEchoice $m]
   } else {
      set ecstate "Unknown"
      set-defaults
   }

   if {$ecstate != "Idle" && $ecstate != "Unknown"} then {
#    update parameters with current server values - (if configured)
      set ecnumdest [read-register num_dest]
      set ecbuffsize  [lsearch $buffchoice [read-register buf_size]]
      set ecqsize [read-register num_block]
      set ecromode($ecnumber)  [read-register ford]
      set ecbootfile($ecnumber) [read-register bootf]
      set ecdumpfile($ecnumber) [read-register dumpf]
  
      for {set i 0; set j 1} {$i < $ecnumdest} {incr i; incr j} {
         write-attribute DS.state "-i $i"
         set ecds($j) [read-register DS.state]
         write-attribute DD.state "-i $i"
         set ecdd($j) [read-register DD.state]
      }
   }

   :modify ECNUM  :value  [lsearch $ecchoice ec$ecnumber]
   :modify ECSTATE  :value $ecstate
   :modify NUMDEST  :value $ecnumdest
   :modify BUFFSIZE :value $ecbuffsize
   :modify QSIZE    :value $ecqsize
#   :modify ROMODE  :value $ecromode($ecnumber) 

   :modify ENETIPADDR :value $ecenetaddr($ecnumber)
   :modify FCIPADDR   :value $ecfcaddr($ecnumber)

   :modify BOOTFILE   :value $ecbootfile($ecnumber)
#   :modify DUMPFILE   :value $ecdumpfile($ecnumber)

#    enable inuse destinations
   for {set i 1} {$i <= $maxdestn} {incr i}  {:modify DESTN$i  :inactive F :value "" :inactive T}
   for {set i 1} {$i <= $ecnumdest} {incr i} {:modify DESTN$i  :inactive F :value "$ecdestn($i)"}
   for {set i 1} {$i <= $maxdestn} {incr i}  {:modify DS$i     :inactive F :value 0 :inactive T}
   for {set i 1} {$i <= $ecnumdest} {incr i} {:modify DS$i     :inactive F :value "$ecds($i)"}
   for {set i 1} {$i <= $maxdestn} {incr i}  {:modify DD$i     :inactive F :value 0 :inactive T}
   for {set i 1} {$i <= $ecnumdest} {incr i} {:modify DD$i     :inactive F :value "$ecdd($i)"}

   clear-busy
   set EGreport $EGreport_save
}


proc do-all action {
  global EGreport maxec ecenetaddr ecappname ecnumber

   set EGreport_save $EGreport
   if {$EGreport == 0} then {set EGreport -1}
   set-busy

   set ecnumsave $ecnumber
   for {set i 1} {$i <= $maxec} {incr i} {
      if {$ecenetaddr($i) != 0 && $ecappname($i) != 0} then {
         set ecnumber $i
         click-action $action
      }
   }
   set ecnumber $ecnumsave

   clear-busy
   set EGreport $EGreport_save
}

#    procedures called from SYS window

proc start-all {} {do-all START; redisplay; return}
  
proc stop-all {}  {do-all STOP; redisplay; return}

proc reset-all {} {do-all RESET; redisplay; return}

proc setup-all {} {do-all SETUP; redisplay; return}

proc set-DS {v} {
  global maxdestn ecds
   for {set i 1} {$i <= $maxdestn} {incr i} {set ecds($i) $v}
   do-all DS
   redisplay
   return
}

proc set-DD {v} {
  global maxdestn ecdd
   for {set i 1} {$i <= $maxdestn} {incr i} {set ecdd($i) $v}
   do-all DD
   redisplay
   return
}


proc EC-state {state} {
  global ecnumber maxec ecenetaddr ecappname STATEchoice

   set ecnumsave $ecnumber
   set ecstate 0
   for {set i 1} {$i <= $maxec} {incr i} {
      if {$ecenetaddr($i) != 0 && $ecappname($i) != 0} then {
         set ecnumber $i
         set ecstate [lindex $STATEchoice [read-register state]]
         if {$ecstate != "$state"} then {set ecnumber $ecnumsave; return 0}
      }
   }
   set ecnumber $ecnumsave
#    if we get here all ECs are in the required state
   return 1
}



proc click arg {
   global debug
   global ecnumber ecenetaddr ecfcaddr ecromode ecbuffsize ecqsize ecnumdest ecdestn ecdd ecds
   global ecdumpfile ecbootfile ecchoice appnames
   global dumpstart dumpwords dumpopt destntodump allecs
   global statsupdate
   global maxdestn

   if {$debug} then {insert-debug "called proc click with argument \{$arg\}"}
#  insert-debug "entered proc click with argument \{$arg\}"

   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   if {$c==":quit"} then {return [click-quit $f]}
   switch -glob $w {
      ECNUM       {set ecnumber [lsearch $appnames [lindex $ecchoice $v]]}
      ALL         {set allecs $v}

      ENETIPADDR  {set ecenetaddr($ecnumber) $v}
      FCIPADDR    {set ecfcaddr($ecnumber) $v}
      BOOTFILE    {set ecbootfile($ecnumber) $v}
      DUMPFILE    {set ecdumpfile($ecnumber) $v}
      BUFFSIZE    {set ecbuffsize $v}
      QSIZE       {set ecqsize $v}
      NUMDEST     {set ecnumdest $v}

      DESTN*      {set ecdestn([string trimleft $w DESTN]) "$v"}
      DD*         {
         set ecdd([string trimleft $w DD]) $v
         if {$allecs == 0} then {click-action DD} else {do-all DD}
      }
      DS*         {
         set ecds([string trimleft $w DS]) $v
         if {$allecs == 0} then {click-action DS} else {do-all DS}
      }

      PING        -
      CLEAR_STAT  -
      REALIGN     -
      FLUSH       -
      DUMP        -
      SETUP       -
      RESET       -
      START       -
      STOP        {if {$allecs == 0} then {click-action $w} else {do-all $w}}

      ALLDS       {
          :modify ALLDS :value -1
          for {set i 1} {$i <= $maxdestn} {incr i} {set ecds($i) $v}
          if {$allecs == 0} then {click-action DS} else {do-all DS}
      }
      ALLDD       {
          :modify ALLDS :value -1
          for {set i 1} {$i <= $maxdestn} {incr i} {set ecdd($i) $v}
          if {$allecs == 0} then {click-action DD} else {do-all DD}
      }

      FORMATDUMP  click-formatdump
      ONLINEDUMP  click-onlinedump
      DUMPSTART   {set dumpstart $v}
      DUMPWORDS   {set dumpwords $v}
      DUMPOPT     {set dumpopt $v}
      DUMPBUFFER  {set destntodump $v}

      STATISTICS  {make-stats}
      SUPDATE     {set statsupdate $v}

      HELP        -
      SHELP       -
      REDISPLAY   -
      SREDISPLAY  {update-stats}
      default     {insert-debug "called proc click with argument \{$arg\}"}
   }
#     do a redisplay after all clicks
      redisplay
}


proc set-experiment args {start}

proc start args {
   global  started ecchoice ecnumber appnames ecappname maxec
   if {$started == 0} then {make-frame}
   open-frame
   set-config
   set ecchoice [make-EC-list]
   :modify ECNUM :strings $ecchoice :value 0
   for {set i 1} {$i <= $maxec} {incr i} {set ecappname($i) 0}
   foreach ec $ecchoice {
      set ecappname([string range $ec 2 end]) [lsearch $appnames $ec]
   }
   set ecnumber [lsearch $appnames [first $ecchoice]]
   redisplay
}

proc make-frame {} {
   global env started
   global enetnames fcnames maxdestn maxqsize buffchoice rochoice
   global dumpstart dumpwords dumpopt allecs ecqsize ecnumdest

 :frame [self frame-name] :label "[self frame-title]" :show-footer T :resize F  \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 20

      :choice-stack ECNUM :label "EC Number:" :valuex 15c
      :text ECSTATE :label "State" :w 12 :value "Idle" :deaf T
      :hskip 100
      :checkbox ALL :label "Act on all ECs:" :value $allecs
      :next-row T
      :col-gap 0
      :choice-stack  ENETIPADDR   :label "name/IP address (ethernet):"       :strings $enetnames :value 1
      :choice-stack  FCIPADDR     :label "name/IP address (fibre channel):"  :strings $fcnames :value 1
#      :next-row T
#      :choice  ROMODE       :label "read out mode"  :strings $rochoice

      :next-row T
      :text BOOTFILE        :label "DSP program file:"    :w 70  :value "" :valuex 20c
#      :next-row T
#      :text DUMPFILE        :label "dump file:"    :w 80  :value "" :valuex 10c

    :panel PANEL2 :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 20

      :number NUMDEST :label "Number of destinations"  :min 0 :max $maxdestn  :value $ecnumdest
      :number QSIZE   :label "Queue size (blocks)"     :min 1 :max $maxqsize  :value $ecqsize
      :choice-stack BUFFSIZE      :label "Buffer size (bytes)"   :strings $buffchoice
      :next-row T

      :message M1 :label "IP address and FCS physical port for each destination" :bold T
      :next-row T

      :col-gap 10
      set j 0
      for {set i 1} {$i <= $maxdestn} {incr i} {
         :text DESTN$i  :label "#$i" :w 18 :value "" :inactive T :valuex 4c
         :checkbox DS$i :label "DS" :inactive T
         :checkbox DD$i :label "DD"  :inactive T
         :hskip 20
         incr j
         if {$j == 2} then {:next-row T; set j 0}
      }

    :panel BUTTONS :layout T :padx 5 :pady 5
     :layout H; :row-gap 10; :col-gap 10

      :message M2 :label "Event Collector Control" :bold T
      :next-row T
      :button SETUP; :button RESET; :button START; :button STOP; :button PING; :button REALIGN; :button FLUSH
      :button CLEAR_STAT :label "Clear Statistics"
      :next-row T
      :hskip 50
      :choice ALLDS  :label "All DS"  :strings [list Off On] :value -1
      :hskip 50
      :choice ALLDD  :label "All DD"  :strings [list Off On] :value -1
      :next-row T
      :message M3 :label "Data Buffer Dump Options" :bold T

      :next-row T
      :number DUMPBUFFER :label "buffer index"  :min 0 :max [expr $maxdestn - 1] :value 0
      :number DUMPSTART  :label "start (words)" :w 5 :min 0 :max 8192 :value $dumpstart
      :number DUMPWORDS  :label "size (words)"  :w 5 :min 0 :max 8192 :value $dumpwords
      :choice-stack DUMPOPT :label "format option" :strings [list raw partial full] :value $dumpopt

      :next-row T
      :button ONLINEDUMP :label "Online Dump"
#      :button DUMP  :label "Dump to file"
#      :button FORMATDUMP :label "Format Dump"
      :button UPDATE :label "Update Configuration" :command set-config

      :hskip 50
      :button REDISPLAY; :button STATISTICS; :button HELP

   set started 1
   close-frame
}



#    statistics control
set statsupdate 0
set statsframe "[self frame-name].stats"
set statsvisible 0

proc make-stats {} {
  global env statsframe statsvisible statsupdate ecnumdest

:frame $statsframe :label "Event Collector Statistics" :show-footer T :resize F :transient T

:panel S1 :layout T 
  :layout H
    :message DSM1 :label "Data Sources" :bold T
    :hskip 300
    :text SECNUM :label "EC Number" :deaf T :w 5
  :next-row T
    :message DSM2 :label "Destination Active"     :w 20
    :message DSM3 :label "Number of Blocks"       :w 15
    :message DSM4 :label "Data Rate (Kbytes/sec)" :w 20
  :next-row T
    for {set i 0} {$i < $ecnumdest} {incr i} {
        :checkbox DSSTATE$i :label "[expr $i + 1]" :deaf T  :valuex 10c
        :text   DSNOBS$i  :label ""   :deaf T    :w 12 :valuex 10c
        :text   DSAVE$i   :label ""   :deaf T    :w 12 :valuex 5c
      :next-row T
    }

:panel S2 :layout T
  :layout H
    :message DDM1 :label "Data Destinations" :bold T
  :next-row T
    :message DDM2 :label "Destination Active"  :w 20
    :message DDM3 :label "Number of Blocks"   :w 15
    :message DDM4 :label "Data Rate (blocks/sec)"  :w 20
    :message DDM5 :label "Number of Requests"  :w 16
    :message DDM6 :label "Data Rate (requests/sec)"  :w 20
  :next-row T
    for {set i 0} {$i < $ecnumdest} {incr i} {
        :checkbox DDSTATE$i :label "[expr $i + 1]" :deaf T  :valuex 10c
        :text   DDNOBS$i  :label ""   :deaf T  :w 12 :valuex 10c
        :text   DDAVE$i   :label ""   :deaf T :w 12 :valuex 5c
        :text   DDNOREQS$i  :label ""   :deaf T :w 12 :valuex 5c
        :text   DDAVER$i    :label ""   :deaf T :w 12 :valuex 5c
      :next-row T
    }
    
:panel S3 :layout T
  :layout H
    :message DSPM1 :label "DSP" :bold T
  :next-row T
    :message DSPM2 :label "Destination Active"  :w 20
    :message DSPM3 :label "Number of Packets"  :w 15
    :message DSPM4 :label "Data Rate (packets/sec)"  :w 20
  :next-row T
    for {set i 0} {$i < $ecnumdest} {incr i} {
        :checkbox DSPSTATE$i :label "[expr $i + 1]" :deaf T  :valuex 10c
        :text   DSPPACK$i  :label ""   :deaf T :w 12 :valuex 10c
        :text   DSPAVEP$i   :label ""   :deaf T :w 12 :valuex 5c
      :next-row T
    }

:panel S4 :layout L :pady 10
    :layout V
    :text DSPNOEV :label "number of incoming events"  :w 12 :valuex 30c
    :text DSPAVE  :label "event rate (events/sec)" :w 12 :valuex 30c
    :text DSPMAXL :label "maximum event length (bytes)" :w 12 :valuex 30c
    :text DSPAVEL :label "average event length (bytes)" :w 12 :valuex 30c

:panel S5 :layout L :pady 10
  :layout V; :row-gap 10
    :choice-stack SUPDATE    :label "Update"   :strings [list Off On]
    :button SREDISPLAY :label "Redisplay"
    :button SHELP      :label "Help"

:show T
set statsvisible 1
set statsupdate 0
}
