
set frame NEOGEN
set frametitle "Neo++ Code Generator"
set icontitle Neo++Gen

set VXIServer "VXI"

set NEOGENBASE [file join $env(MIDASBASE) euroball tcl neogen]


set currentpage 0
set lastpage 10


#    list of card types supported by the wizard
set VXICardChoice [list  trigger tapered clover cluster bgoball saphir]

#    for each card type a description for the choice widget
set VXICardID(tapered) "Tapered Detector (Ge and BGO cards)"
set VXICardID(clover)  "Clover Detector"
set VXICardID(cluster) "Cluster Detector (ClusterGE and ClusterBGO)"
set VXICardID(bgoball) "BGO Inner Ball"
set VXICardID(saphir)  "Saphir Detector"
set VXICardID(trigger) "Euroball Master Trigger"

#    for each card type the default state of InUse option (1 = InUse)
set VXICardInuse(tapered) 1
set VXICardInuse(clover) 1
set VXICardInuse(cluster) 1
set VXICardInuse(bgoball) 1
set VXICardInuse(trigger) 1
set VXICardInuse(saphir) 0

#    global NEO++ options

set NeoOptionChoice [list totege totebgo totebgoball tote gm tape]

set NeoOptionID(totege)      "Total Ge energy"
set NeoOptionID(totebgo)     "Total BGO energy"
set NeoOptionID(totebgoball) "Total BGOBall energy"
set NeoOptionID(tote)        "Total energy"
set NeoOptionID(gm)          "Gain Matching"
set NeoOptionID(tape)        "Tape Output"

set NeoOptions(totege) 1
set NeoOptions(totebgo) 1
set NeoOptions(totebgoball) 1
set NeoOptions(tote) 1
set NeoOptions(gm) 1
set NeoOptions(tape) 1
set NeoOptions(spystrms) 2


#    for saving the  NEO++ source
set browserdirectory [file join $env(HOME) experiments]
set browserfile      "neo.cc"

#    card specific options are in the card specific files


proc make_page {n} {
   global env _variable currentpage
   upvar #0 page[set n]text text

  set currentpage $n

  :frame "[self frame-name]" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :galley GALLEY  :scroll N :rows 15 :cols 80 :deaf T
      [:path GALLEY].text configure -background $env(tkframebackground)
      :modify GALLEY  :createtag "title -foreground red -background $env(tkframebackground)"
      :modify GALLEY  :createtag "body  -foreground black -background $env(tkframebackground)"
      set ttext [first [split $text \n]]
      :modify GALLEY  :usingtag title :strings "$ttext\n"
      :modify GALLEY  :usingtag body  :strings  "[string range $text [string length $ttext] end]"
}

proc make_buttons {} {

    :panel BUTTONS :layout B :padx 10 :pady 10 :after FOOTER
    :layout H; :col-gap 10; :row-gap 10
      :button CANCEL   :label "Cancel"  :command cancel
      :button BACK     :label "<Back"   :command goto_previouspage
      :button NEXT     :label "Next>"   :command goto_nextpage
      :button FINISH   :label "Finish"  :command goto_finish  :inactive T

   centre-frame
   :show T
}

proc get_text {w} {
   global _variable
   set v [string tolower $w]
   set _variable($v) "[[:path $w].text get 1.0 end]"
}

proc value {w} {
   global _variable
   set v [string tolower $w]
   if {[info exists _variable($v)]} then {return $_variable($v)} else {return ""}
}

proc goto_nextpage {} {
   global currentpage lastpage

   if {$currentpage == $lastpage} then {goto_finish; return}

#    first complete processing for the current page
   if {$currentpage > 0} then {finish_page$currentpage}

   incr currentpage

#    next any preprocessing for the new page

   start_page$currentpage

   make_page[set currentpage]
   if {$currentpage == $lastpage} then {
     :modify NEXT :inactive T
     :modify FINISH :inactive F
   }
   if {$currentpage == 1} then {
     :modify BACK :inactive T
   }
}

proc goto_previouspage {} {
   global currentpage lastpage
   incr currentpage -1
   if {$currentpage == 0} then {set currentpage 1}

   start_page$currentpage
   make_page[set currentpage]
}

#    Now each page definition

#    page 1 - Introduction

proc start_page1 {} {}

set page1text "Welcome to the NEO++ Code Generator wizard\nThis software will attempt to generate NEO++ code suitable for your experiment and check that the options you request are compatible with the current VXI hardware setup\nTo be most useful the VXI hardware should be setup before you continue and in particular all VXI cards you intend to use should be online and all data sources (adcs/tacs) you intend to read out should be enabled\n\nYou will be asked a number of questions in order to determine the options you require and NEO++ code will be generated for each option. At each step you will be given the opportunity to manually modify the code generated if you feel (and are sure) it is needed.\n\nClick on the Next> button to start."

proc make_page1 {} {
   make_page 1
   make_buttons 
}

proc finish_page1 {} {}


#   page 2 

#    This page allows selection of which card types are being used by the experimemt

proc start_page2 {} {}

set page2text "Please select all VXI card types which are to be used by your experiment.\nNEO++ statements will only be generated for the selected card types. Later you will be able to check that the hardware setup is consistent with your choices.\n\nClick on the Next> button to continue."

proc make_page2 {} {
   global VXICardChoice VXICardInuse VXICardID
   make_page 2

   :panel PANEL2 :layout T :padx 10 :pady 0
     foreach item $VXICardChoice {
         :choice [uppercase $item] :label "$VXICardID($item)" :valuex 40c  :strings [list "Ignore" "In Use"] :w 8 \
             :value $VXICardInuse($item) :command click_page2
     }

   make_buttons
}

proc click_page2 {w v} {
    global VXICardInuse
    set VXICardInuse([lowercase $w]) $v
}

proc finish_page2 {} {}


#  page 3

#    For each card type in use show the available data sources (ADCs TDCs etc) and allow the user to select those
#    which are required and basic options which are applied per data source

proc start_page3 {} {
   global resume_page3

   set resume_page3 0
}

proc make_page3 {} {
   global NEOGENBASE
   global VXICardChoice VXICardInuse
   global DataSources DataSourceID 
   global DataSourceOptions DataHistOptions DataTapeOptions
   global vxicard resume_page3 page3text

   incr resume_page3

   set index 0
   foreach vxicard $VXICardChoice {
      incr index
      if {$index < $resume_page3} then {continue}
      if {$VXICardInuse([lowercase $vxicard]) == 1} then {

         set page3text "Options for VXI card type $vxicard.\nPlease select all data items you wish to use on this card and choose from the processing options available.\nThen click on the button \"Generate NEO++\".\n\nYou may view the generated NEO++ statements by clicking on the \"View NEO++\" button.\n\nYou can check for any inconsistences between your choices here and the current hardware setup by clicking on the \"Check hardware setup\" button.\n\nClick on the \"Next Card\" button to continue to the next card\n."

         set f [file join $NEOGENBASE $vxicard.tcl]
         set z [catch {uplevel #0 source $f} m]
         if {$z != 0} then {
             midas-warning "Unable to read NEO++ source file for VXI card type $vxicard from file $f\nReason - $m"
             return
         }

         make_page 3

         :panel PANEL2 :layout T :padx 10 :pady 0
           :layout H; :col-gap 15
         foreach datasource $DataSources($vxicard) {
            :choice [uppercase $datasource]S :label "$DataSourceID($vxicard,$datasource)" :valuex 16c \
                :strings [list "Ignore" "In Use"] :w 8 \
                :value $DataSourceOptions($vxicard,$datasource) :command click_page3
            :choice [uppercase $datasource]H :label "Histogram" \
                :strings [list "No" "Yes"] :w 4 \
                :value $DataHistOptions($vxicard,$datasource) :command click_page3
            :choice [uppercase $datasource]T :label "=> tape" \
                :strings [list "No" "Yes"] :w 4 \
                :value $DataTapeOptions($vxicard,$datasource) :command click_page3
            :next-row T
         }
           :next-row T
           :layout H
           :col-gap 25
           :button GEN   :label "Generate NEO++" :command click_neogen1
           :button VIEW  :label "View NEO++" :command click_neoview1 :inactive T
           :button SETUP :label "Check hardware setup" :command click_check1
           :button NEXTCARD :label "Next Card" :command make_page3 :inactive T
         make_buttons
         :modify NEXT :inactive T
         :modify BACK :inactive T

         return
      }
   }

   set page3text "\nAll VXI cards in use have now been handled.\n\nClick on the Next> button to continue."
   make_page 3
   make_buttons
}

proc click_check1 {args} {
    global vxicard
    probe_$vxicard
}

proc click_page3 {w v} {
    global VXICardInuse
    global DataSources DataSourceID
    global DataSourceOptions DataHistOptions DataTapeOptions
    global vxicard

    foreach datasource $DataSources($vxicard) {
        if {"[uppercase $datasource]S" == $w} then {set DataSourceOptions($vxicard,$datasource) $v; return}
    }

    foreach datasource $DataSources($vxicard) {
        if {"[uppercase $datasource]H" == $w} then {set DataHistOptions($vxicard,$datasource) $v; return}
    }

    foreach datasource $DataSources($vxicard) {
        if {"[uppercase $datasource]T" == $w} then {set DataTapeOptions($vxicard,$datasource) $v; return}
    }
}

proc finish_page3 {} {}


proc NEO {s} {
    global vxicard neo
    append neo($vxicard) "$s\n"
}

proc HISTH {s} {
    global vxicard histh
    append histh($vxicard) "$s\n"
}

proc EVENTH {s} {
    global vxicard eventh
    append eventh($vxicard) "$s\n"
}

proc click_neogen1 {args} {
   global vxicard eventh histh neo

   set eventh($vxicard) ""
   set histh($vxicard) ""
   set neo($vxicard) ""

   neogen1_$vxicard

   :modify VIEW :inactive F
   :modify NEXTCARD :inactive F
}

proc click_neoview1 {args} {
    global vxicard 
    global eventh histh

    :frame [self frame-name].neo :label  "NEO++ code contribution to file event.h for VXI card type $vxicard"  :transient T
    wm protocol [self frame-name].neo WM_DELETE_WINDOW {destroy [self frame-name].neo}
    :galley G :rows 50 :cols 80 :strings "$eventh($vxicard)\n\n\n" :font fixed
    :show T
}

#   page 4

#    selection of NEO++ options which are applied globally

proc start_page4 {} {}

set page4text "Global NEO++ options.\nPlease select those options you require to be performed by the NEO++ program.\n\nClick on the Next> button to continue."

proc make_page4 {} {
   global NeoOptions NeoOptionChoice NeoOptionID
   make_page 4

   :panel PANEL2 :layout T :padx 10 :pady 0
     foreach item $NeoOptionChoice {
         :choice [uppercase $item] :label "$NeoOptionID($item)" :valuex 40c  :strings [list "No" "Yes"] :w 4 \
             :value $NeoOptions($item) :command click_page4
     }
     :choice SPYSTRMS :label "Number of Spy streams" :valuex 40c  :strings [list 0 1 2 3] :w 4  \
             :value $NeoOptions(spystrms) :command click_page4

   make_buttons
   :modify NEXT :inactive F
}

proc click_page4 {w v} {
    global NeoOptions
    set NeoOptions([lowercase $w]) $v
}


proc finish_page4 {} {}


#  page 5

proc start_page5 {} {
   global resume_page5 

   set resume_page5 0
}

proc make_page5 {} {
   global VXICardChoice VXICardInuse
   global NeoCardOptionChoice NeoCardOptionID NeoCardOptions
   global vxicard resume_page5 page5text

   incr resume_page5

   set index 0
   foreach vxicard $VXICardChoice {
      incr index
      if {$index < $resume_page5} then {continue}
      if {$VXICardInuse([lowercase $vxicard]) == 1} then {

         set page5text "Options for VXI card type $vxicard.\n\nPlease choose from the processing options available for this card.\nThen click on the button \"Generate NEO++\".\n\nYou may view the generated NEO++ statements by clicking on the \"View NEO++\" button.\n\nClick on the \"Next Card\" button to continue to the next card\n."

         make_page 5

         :panel PANEL2 :layout T :padx 10 :pady 0
           :layout V
         foreach neooption $NeoCardOptionChoice($vxicard) {
            :choice [uppercase $neooption] :label "$NeoCardOptionID($vxicard,$neooption)" :valuex 40c \
                :strings [list "No" "Yes"] :w 4 \
                :value $NeoCardOptions($vxicard,$neooption) :command click_page5
         }
           :next-row T
           :next-row T
           :layout H
           :col-gap 25
           :button GEN  :label "Generate NEO++" :command click_neogen2
           :button VIEW :label "View NEO++" :command click_neoview2 :inactive T
           :button NEXTCARD :label "Next Card" :command make_page5 :inactive T
         make_buttons
         :modify NEXT :inactive T
         :modify BACK :inactive T

         return
      }
   }

   set page5text "All VXI cards in use have now been handled.\n\nClick on the Next> button to continue."
   make_page 5
   make_buttons
}

proc click_page5 {w v} {
    global NeoCardOptionChoice NeoCardOptions
    global vxicard

    foreach neooption $NeoCardOptionChoice($vxicard) {
        if {[uppercase $neooption] == $w} then {break}
    }
    set $NeoCardOptions($vxicard,$neooption) $v
}

proc click_neogen2 {args} {
   global vxicard neo

   set neo($vxicard) ""
   neogen2_$vxicard

   :modify VIEW :inactive F
   :modify NEXTCARD :inactive F
}

proc click_neoview2 {args} {
    global vxicard 
    global neo

    :frame [self frame-name].neo :label  "NEO++ code contribution for VXI card type $vxicard"  :transient T
    wm protocol [self frame-name].neo WM_DELETE_WINDOW {destroy [self frame-name].neo}
    :galley G :rows 50 :cols 80 :strings "$neo($vxicard)\n\n\n" :font fixed
    :show T
}

#    

proc finish_page5 {} {}


#  page 6

set viewwindow 0

proc start_page6 {} {global viewwindow; set viewwindow 0}

proc make_page6 {} {
   global page6text

   set page6text "\nAll detector specific NEO++ has now been generated. The final step is to generate the containing code and merge to create a single C++ source file.\n\nClick on the button \"Generate NEO++\" to continue.\n\nYou may view the generated NEO++ statements by clicking on the \"View NEO++\" button.\n\nYou may modify the final NEO++ program by just overtyping in the View window.\n\nClick on the Next> button to continue."

   make_page 6

   :panel PANEL2 :layout T :padx 10 :pady 0
     :layout H; :col-gap 25
     :button GEN  :label "Generate NEO++" :command click_neogen
     :button VIEW :label "View NEO++" :command click_neoview :inactive T

   make_buttons
   :modify NEXT :inactive T
}

proc finish_page6 {} {
    global viewwindow neo

    if {$viewwindow == 1} then {
        set neo(complete) [get_text G]
    }
}

proc click_neogen {args} {
    global vxicard neo NEOGENBASE

    set vxicard complete
    set neo(complete) ""

    set f [file join $NEOGENBASE neo.tcl]
    set z [catch {uplevel #0 source $f} m]
    if {$z != 0} then {
       midas-warning "Unable to read NEO++ source file from file $f\nReason - $m"
       return
    }

    :modify VIEW :inactive F
    :modify NEXT :inactive F
}

proc click_neoview {args} {
    global neo viewwindow

    :frame [self frame-name].neo :label  "Completed NEO++ Program"  :transient T
    wm protocol [self frame-name].neo WM_DELETE_WINDOW {destroy [self frame-name].neo}
    :galley G :rows 50 :cols 100 :strings "$neo(complete)\n\n\n" :font fixed
    :show T

    set viewwindow 1
}

#  page 7

proc start_page7 {} {goto_nextpage}

proc make_page7 {} {
   global page7text

   set page7text "An empty page."

   make_page 7

   :panel PANEL2 :layout T :padx 10 :pady 0

   make_buttons
}

proc finish_page7 {} {}



#  page 8

proc start_page8 {} {}

proc make_page8 {} {
   global page8text

   set page8text "\nYour NEO++ program has now been completed.\n\nClick on the \"Save NEO++ program\" to write your NEO++ program to disc.\n\nAfter you have saved the NEO++ program file to disc a suitable Makefile can be generated.\n\nClick on the Next> button to continue."

   make_page 8

   :panel PANEL2 :layout T :padx 10 :pady 0
     :layout H; :col-gap 25
     :button SAVE  :label "Save NEO++ program" :command "midas-browser W save_neo"
     :button MAKE  :label "Generate Makefile for the NEO++ program" :command neo_makefile

   make_buttons
    :modify NEXT :inactive T
    :modify MAKE :inactive T
}

proc finish_page8 {} {}

proc save_neo {args} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file
   global  neo

   set z [catch {open $directory/$file w} f]
   if {$z != 0} then {midas-warning "Unable to write $directory/$file\n$f"; return}
   puts $f $neo(complete)
   close $f

   :modify NEXT :inactive F
   :modify MAKE :inactive F
}

proc neo_makefile {args} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file
   global NEOGENBASE

   set prg [first [split $file .]]

   set source2 [file join $NEOGENBASE Makefile]
   set target [file join $directory Makefile]

   if {[midas-check-for-temp] == 0} then {return}

   set source1 [file join neogen-prg]

   set z [catch {open $source1 w} f]
   if {$z != 0} then {midas-warning "Unable to write file $source1\n$f"; return}
   puts $f "PRG = $prg"
   close $f

   set z [catch {exec cat $source1 $source2 > $target} f]
   if {$z != 0} then {midas-warning "Error creating Makefile\n$f"; return}
}

#  page 9

proc start_page9 {} {goto_nextpage}

proc make_page9 {} {
   global page9text

   set page9text "An empty page."

   make_page 9

   :panel PANEL2 :layout T :padx 10 :pady 0

   make_buttons
}

proc finish_page9 {} {}


#  page 10

proc start_page10 {} {}

proc make_page10 {} {
   global page10text

   set page10text "Well done - You have now created a NEO++ program.\n Click on the Compile button to start the make procedure.\n"

   make_page 10

   :panel PANEL2 :layout T :padx 10 :pady 0
    :button COMPILE :command click-compile

   make_buttons
}

proc click-compile args {
    inform evFarm click-compile
    goto_finish
}

proc finish_page10 {} {}


proc start args {
   global _variable currentpage
    set currentpage 0
    goto_nextpage
}

proc goto_finish {} {complete}

proc complete args {close-frame}

proc cancel {} {complete}

proc null args {}




