


#    for bgoball detectors a list of data sources
#    Note - these must be in order of Item number
set DataSources(bgoball) [list "BGOE" "BGOT" "BGOP"]

#    for bgoball detectors a description of each data source for the choice widget
set DataSourceID(bgoball,BGOE)   "BGO Energy"
set DataSourceID(bgoball,BGOT)   "BGO TAC"
set DataSourceID(bgoball,BGOP)   "BGO Pattern"

#    the name of the register
set DataSourceReg(bgoball,BGOE)   "Energy"
set DataSourceReg(bgoball,BGOT)   "TAC"
set DataSourceReg(bgoball,BGOP)   "Pattern"

#    for bgoball detectors the default state of the InUse option
set DataSourceOptions(bgoball,BGOE)   1
set DataSourceOptions(bgoball,BGOT)   1
set DataSourceOptions(bgoball,BGOP)   1

#    for bgoball detectors the default state of the Hist option
set DataHistOptions(bgoball,BGOE)   1
set DataHistOptions(bgoball,BGOT)   1
set DataHistOptions(bgoball,BGOP)   1

#    for bgoball detectors the default state of the Tape option
set DataTapeOptions(bgoball,BGOE)   1
set DataTapeOptions(bgoball,BGOT)   1
set DataTapeOptions(bgoball,BGOP)   1

set NeoCardOptionChoice(bgoball) [list sum hist tape1 tape2]

set NeoCardOptionID(bgoball,sum)       "Generate BGO Inner Ball Sum Energy and Multiplicity"
set NeoCardOptionID(bgoball,hist)      "Histogram sum energy and fold"
set NeoCardOptionID(bgoball,tape1)     "Output BGO Inner Ball Sum to tape"
set NeoCardOptionID(bgoball,tape2)     "Suppress BGO Inner Ball raw data to tape"

set NeoCardOptions(bgoball,sum)           1
set NeoCardOptions(bgoball,hist)          1
set NeoCardOptions(bgoball,tape1)         1
set NeoCardOptions(bgoball,tape2)         1




#    hardware probe to check setup
   
proc probe_bgoball {} {
   global env VXIServer DataSources DataSourceReg DataSourceOptions DataSourceID

  :frame "[self frame-name].probe" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :galley REPORT :rows 15 :cols 80 :deaf T

    :modify REPORT :strings "\nLooking for BgoBall VXI Card data word status\n\n"
    :show T

    set num 0

    for {set c 0} {$c < 12} {incr c} {
    foreach item $DataSources(bgoball) {
      set VXIregister /$VXIServer/BgoBall$c.$DataSourceReg(bgoball,$item).ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(bgoball,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(bgoball,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware for channel $c.\n"
             incr num
          }
      }
      if {$DataSourceOptions(bgoball,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(bgoball,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware for channel $c.\n"
             incr num
          }
      }
    }
    set VXIregister /$VXIServer/BgoBall$c.Disable
    set VXIdata [EG "register :name '$VXIregister' ?value"]
    if {$VXIdata == 1} then {
        :modify REPORT :strings "Warning - BGO channel $c IS NOT enabled\n"
    }
    }

    if {$num == 0} then {:modify REPORT :strings "\nYour NEO++ statements for this card are compatible with the hardware setup - Well done!\n"}
    if {$num != 0} then {:modify REPORT :strings "\nThere are $num inconsistences between your NEO++ statements for this card and the hardware setup\n"}
}


#    phase 1 code generation - declarations

proc neogen1_bgoball {} {
    global DataSourceOptions DataHistOptions

#    generate subdetector definitions

    EVENTH "//    Bgo Inner Ball detector section generated [date]\n\n\n\n"

    EVENTH "//   detector definition for bgoball detector"

     EVENTH "format (BgoballRaw)"
     EVENTH "    item     energy;"
     EVENTH "    item     pattern;"
     EVENTH "    item     timing;"
     EVENTH "end_format"
     EVENTH ""
     EVENTH "format (Bgoball)"
     EVENTH "    item     energy;"
     EVENTH "    item     mult;"
     EVENTH "end_format"

     EVENTH "Detector<BgoballRaw, BGOBALL_RAW> bgoballraw;"
     EVENTH "Detector<Bgoball, BGOBALL> bgoball;"
 
}


proc neogen2_bgoball {} {
   global DataSourceOptions DataHistOptions NeoOptions NeoCardOptions

     HISTH "//    Bgo Inner Ball detector section generated [date]\n\n\n"

#    generate histograms for online monitoring

     HISTH "VectHist1D<4> B_stat(128,\"Astat_BgoBall\");"

    if {$DataSourceOptions(bgoball,BGOE) == 1 && $DataHistOptions(bgoball,BGOE) == 1} then {
       HISTH "VectHist1D<18> B_Energy(8192,\"BgoBall_Energy\");"
    }
    if {$DataSourceOptions(bgoball,BGOT) == 1 && $DataHistOptions(bgoball,BGOT) == 1} then {
       HISTH "VectHist1D<18> B_TAC(8192,\"BgoBall_TAC\");"
    }
    if {$DataSourceOptions(bgoball,BGOP) == 1 && $DataHistOptions(bgoball,BGOP) == 1} then {
       HISTH "VectHist1D<18> B_Pattern(2048,\"BgoBall_Pattern\");"
    }

#    create sum histograms  

  if {$NeoCardOptions(bgoball,hist) == 1} then {
     HISTH "Hist1D B_Sum(8192,\"BgoBall_Sum_Energy\");"
     HISTH "Hist1D B_fold(128,\"Asat_BgoBallFold\");"
  }

     NEO "//    Bgo Inner Ball detector section generated [date]\n\n\n\n"
     NEO "    BgoBallEnergy = 0;"
     NEO "    if (bgoballraw.howManyFired()) \{       //   there are bgoball detectors in this event"
     NEO "        foldBgoBall = 0;"
     NEO "        Astat.inc(74);       //  increment statistics histogram"
     NEO "        jj=0;"
     NEO "        while (jj < bgoballraw.howManyFired()) \{   //  loop for each detector"
     NEO "            keyDet = bgoballraw(jj).keyGet();"
     NEO "            B_stat\[0\].inc(keyDet);       //  increment statistics histogram for detector"
     NEO "            keySpec = keyDet;"
     NEO "            valid = FALSE;"
     NEO ""

   if {$DataSourceOptions(bgoball,BGOE) == 1} then {
     NEO "            temp = bgoballraw(jj).energy & 8191;   // extract energy from 13 bit adc"
     NEO "            if (temp) \{"
   if {$DataHistOptions(bgoball,BGOE) == 1} then {
     NEO "                B_Energy\[keySpec\].inc(temp); // increment raw data histogram"
   }
     NEO "                BgoBallEnergy += temp;"
     NEO "                valid = TRUE;"
     NEO "            \}"
     NEO ""
   }

   if {$DataSourceOptions(bgoball,BGOT) == 1 && $DataHistOptions(bgoball,BGOT) == 1} then {
     NEO "            temp = bgoballraw(jj).timing & 8191;   // extract data from 13 bit adc"
     NEO "            if (temp) B_TAC\[keySpec\].inc(temp); // increment raw data histogram"
     NEO ""
   }

   if {$DataSourceOptions(bgoball,BGOP) == 1 && $DataHistOptions(bgoball,BGOP) == 1} then {
     NEO "            temp = bgoballraw(jj).pattern & 2047;   // extract pattern data"
     NEO "            if (temp) B_Pattern\[keySpec\].inc(temp); // increment raw data histogram"
     NEO ""
   }

    NEO "         if(valid) \{      //  there is an energy data item"
    NEO "            B_stat\[2\].inc(keyDet);"
    NEO "            temp = bgoballraw(jj).pattern & 2047;"

    NEO "            if (temp) \{   //  count bits in the pattern to obtain fold"
    NEO "               for (int j=1; j<=10; j++) \{"
    NEO "                  if (temp >> j & 1) \{"
    NEO "                     foldBgoBall++;"
    NEO "                     keySpec = keyDet * 10 + j - 1;"
    NEO "                     B_stat\[3\].inc(keySpec);"
    NEO "                  \}"
    NEO "               \}"
    NEO "            \} else B_stat\[1\].inc(keyDet);"
    NEO "         \};"
  if {$NeoCardOptions(bgoball,tape2) == 1} then {              
    NEO "	  bgoballraw(jj).invalidate();"
  }
    NEO ""
    NEO "        \}     //  end of detector"

    NEO "	 if (foldBgoBall || BgoBallEnergy) \{"
    NEO "	   bgoball.hasFired(0);        //  to activate the bgoball (sum) detector"
    NEO "	 \};"

    NEO "        if (BgoBallEnergy) \{"
    NEO "	   bgoball(0).energy = (unsigned short)((BgoBallEnergy >> 2) & 8191);"
    NEO "	 \}"

    NEO "        if (foldBgoBall) \{"
    NEO "	   B_fold.inc(foldBgoBall);"
    NEO "	   bgoball(0).mult = (unsigned short)foldBgoBall;"
    NEO "	 \}"

     NEO "   \}        //  end of all detectors"

}


