


#    for clover detectors a list of data sources
#    Note - these must be in order of Item number
set DataSources(clover) [list "Ge20" "Ge4" "FT" "CO" "BGOE" "BGOT" "BGOP"]

set DataSources(cloverGe)  [list "Ge20" "Ge4" "FT" "CO"]
set DataSources(cloverBGO) [list "BGOE" "BGOT" "BGOP"]

#    for clover detectors a description of each data source for the choice widget
set DataSourceID(clover,Ge20)   "Ge 20MeV ADC"
set DataSourceID(clover,Ge4)    "Ge 4MeV ADC"
set DataSourceID(clover,FT)     "FT TAC"
set DataSourceID(clover,CO)     "CO TAC"
set DataSourceID(clover,BGOE)   "BGO Energy"
set DataSourceID(clover,BGOT)   "BGO TAC"
set DataSourceID(clover,BGOP)   "BGO Pattern"

#    the name of the register
set DataSourceReg(clover,Ge20)   "20MeV"
set DataSourceReg(clover,Ge4)    "4MeV"
set DataSourceReg(clover,FT)     "FT"
set DataSourceReg(clover,CO)     "CO"
set DataSourceReg(clover,BGOE)   "Energy"
set DataSourceReg(clover,BGOT)   "TAC"
set DataSourceReg(clover,BGOP)   "Pattern"

#    for clover detectors the default state of the InUse option
set DataSourceOptions(clover,Ge20)   1
set DataSourceOptions(clover,Ge4)    1
set DataSourceOptions(clover,FT)     1
set DataSourceOptions(clover,CO)     0
set DataSourceOptions(clover,BGOE)   1
set DataSourceOptions(clover,BGOT)   1
set DataSourceOptions(clover,BGOP)   1

#    for clover detectors the default state of the Hist option
set DataHistOptions(clover,Ge20)   1
set DataHistOptions(clover,Ge4)    1
set DataHistOptions(clover,FT)     1
set DataHistOptions(clover,CO)     1
set DataHistOptions(clover,BGOE)   1
set DataHistOptions(clover,BGOT)   1
set DataHistOptions(clover,BGOP)   1

set DataHistOptions(clover,Ge20T)   1
set DataHistOptions(clover,Ge4T)   1


#    for clover detectors the default state of the Tape option
set DataTapeOptions(clover,Ge20)   1
set DataTapeOptions(clover,Ge4)    1
set DataTapeOptions(clover,FT)     1
set DataTapeOptions(clover,CO)     1
set DataTapeOptions(clover,BGOE)   1
set DataTapeOptions(clover,BGOT)   1
set DataTapeOptions(clover,BGOP)   1


set NeoCardOptionChoice(clover) [list acrej purej addback gm gmhist]

set NeoCardOptionID(clover,acrej)       "Reject Ge data with anti-compton"
set NeoCardOptionID(clover,purej)       "Reject Ge data with pile-up"
set NeoCardOptionID(clover,addback)     "Perform Ge crystal energy addback"
set NeoCardOptionID(clover,gm)          "Gain match Ge detectors"
set NeoCardOptionID(clover,gmhist)      "Histogram gain matched data"

set NeoCardOptions(clover,acrej)       1
set NeoCardOptions(clover,purej)       1
set NeoCardOptions(clover,addback)     1
set NeoCardOptions(clover,gm)          1
set NeoCardOptions(clover,gmhist)      1




#    hardware probe to check setup
   
proc probe_clover {} {
   global env VXIServer DataSources DataSourceReg DataSourceOptions DataSourceID

  :frame "[self frame-name].probe" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :galley REPORT :rows 15 :cols 80 :deaf T

    :modify REPORT :strings "\nLooking for Clover Ge VXI Card data word status\n\n"
    :show T

    set num 0
  for {set c 0} {$c < 26} {incr c} {
    set numseg ""
   foreach s {A B C D E} {
    foreach item $DataSources(cloverGe) {
      set VXIregister /$VXIServer/Clover$c.Ge$s.$DataSourceReg(clover,$item).ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(clover,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(clover,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware for Ge channel $c segment $s.\n"
             incr num
          }
      }
      if {$DataSourceOptions(clover,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(clover,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware for Ge channel $c segment $s.\n"
             incr num
          }
      }
    }
    set VXIregister /$VXIServer/Clover$c.Ge$s.Enable
    set VXIdata [EG "register :name '$VXIregister' ?value"]
    if {$VXIdata == 1} {append numseg "$s "}
   }
    if {[string length $numseg] != 8} then {
       :modify REPORT :strings "Warning - Ge channel $c; segments $numseg are enabled which is incorrect. Exactly 4 segments should be enabled\n"
    }
  }

    :modify REPORT :strings "\nLooking for Clover BGO VXI Card data word status\n\n"

  for {set c 0} {$c < 26} {incr c} {
    foreach item $DataSources(cloverBGO) {
      set VXIregister /$VXIServer/Clover$c.BGO.$DataSourceReg(clover,$item).ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(clover,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(clover,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware for BGO channel $c.\n"
             incr num
          }
      }
      if {$DataSourceOptions(clover,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(clover,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware for BGO channel $c.\n"
             incr num
          }
      }
    }
  }

#    some checks for things not used

    :modify REPORT :strings "\nLooking for Clover VXI Card RdOut data word status\n\n"

  for {set c 0} {$c < 26} {incr c} {

   set VXIregister /$VXIServer/Clover$c.RdOut.ROCR.EvNumData.Enable
   set VXIdata [EG "register :name '$VXIregister' ?value"]
   if {$VXIdata == 1} then {
      :modify REPORT :strings "Warning - data item Clover$c.RdOut.ROCR.EvNumData IS enabled.\n"
      incr num
   }
  }

    if {$num == 0} then {:modify REPORT :strings "\nYour NEO++ statements for this card are compatible with the hardware setup - Well done!\n"}
    if {$num != 0} then {:modify REPORT :strings "\nThere are $num inconsistences between your NEO++ statements for this card and the hardware setup\n"}
}



#    phase 1 code generation - declarations

proc neogen1_clover {} {
    global DataSourceOptions DataHistOptions

#    generate subdetector definitions

    EVENTH "//    Clover detector section generated [date]\n\n"

#    first do the Ge part

    EVENTH "//   Ge detector definition for clover detector"
    EVENTH "subdetector (GeC)"
    if {$DataSourceOptions(clover,Ge20) == 1} then {
       EVENTH "    item_q E20;"
    } else {
       EVENTH "    dummy_item E20;"
    }
    if {$DataSourceOptions(clover,Ge4) == 1} then {
       EVENTH "    item_q E4;"
    } else {
       EVENTH "    dummy_item E4;"
    }
    if {$DataSourceOptions(clover,FT) == 1} then {
       EVENTH "    item ft;"
    } else {
       EVENTH "    dummy_item ft;"
    }
    if {$DataSourceOptions(clover,CO) == 1} then {
       EVENTH "    item CO;"
    } else {
       EVENTH "    dummy_item CO;"
    }
    EVENTH "end_subdetector"

#    then do the BGO part

    set bgopresent 0

    EVENTH "//   BGO detector definition for clover detector"
    EVENTH "subdetector (BGOC)"
    if {$DataSourceOptions(clover,BGOE) == 1} then {
       EVENTH "    item energy;"
       set bgopresent 1
    } else {
       EVENTH "    dummy_item energy;"
    }
    if {$DataSourceOptions(clover,BGOT) == 1} then {
        EVENTH "    item timing;"
        set bgopresent 1
    } else {
        EVENTH "    dummy_item timing;"
    }
    if {$DataSourceOptions(clover,BGOP) == 1} then {
         EVENTH "    item pattern;"
         set bgopresent 1
    } else {
         EVENTH "    dummy_item pattern;"
    }
    EVENTH "end_subdetector"

#    generate detector format

     EVENTH "format (Clover)"
     EVENTH "    GeC     ge\[4\];"
     if {$bgopresent != 0} then {EVENTH "    BGOC     bgo;"}
     EVENTH "end_format"

     EVENTH "Detector<Clover, CLOVER> clover;"
}


proc neogen2_clover {} {
   global DataSourceOptions DataHistOptions NeoOptions NeoCardOptions

     HISTH "//    Clover detector section generated [date]\n\n\n"

#    generate histograms for online monitoring

     HISTH "VectHist1D<4> Q_stat(128,\"Astat_Clover\");"

    if {$DataSourceOptions(clover,Ge20) == 1 && $DataHistOptions(clover,Ge20) == 1} then {
       HISTH "MatHist1D<26,4> Q_E20(8192,\"Clover_Ge_20MeV\");"
    }
    if {$DataSourceOptions(clover,Ge4) == 1 && $DataHistOptions(clover,Ge4) == 1} then {
       HISTH "MatHist1D<26,4> Q_E4(8192,\"Clover_Ge_4MeV\");"
    }
    if {$DataSourceOptions(clover,BGOE) == 1 && $DataHistOptions(clover,BGOE) == 1} then {
       HISTH "VectHist1D<26> Q_E(8192,\"Clover_BGO_Energy\");"
    }
    if {$DataSourceOptions(clover,BGOT) == 1 && $DataHistOptions(clover,BGOT) == 1} then {
       HISTH "VectHist1D<26> Q_t(8192,\"Clover_BGO_TAC\");"
    }
    if {$DataSourceOptions(clover,BGOP) == 1 && $DataHistOptions(clover,BGOP) == 1} then {
       HISTH "VectHist1D<26> Q_p(2048,\"Clover_BGO_Pattern\");"
    }   

    if {$DataSourceOptions(clover,FT) == 1 && $DataHistOptions(clover,FT) == 1} then {
       HISTH "MatHist1D<26,4> Q_ft(8192,\"Clover_Ge_FT\");"
    }
    if {$DataSourceOptions(clover,CO) == 1 && $DataHistOptions(clover,CO) == 1} then {
       HISTH "MatHist1D<26,4> Q_co(8192,\"Clover_Ge_CO\");"
    }

#    create gain matched histograms as required for 4 and 20 MeV 

  if {$NeoCardOptions(clover,gmhist) == 1} then {
  if {$DataSourceOptions(clover,Ge20) == 1} then {
  if {$NeoCardOptions(clover,acrej) == 1} then {
     HISTH "MatHist1D<26,4> Q_E20_GM(8192,\"Clover_Ge_20MeV_G_Adj_Clean\");"
  } else {
     HISTH "MatHist1D<26,4> Q_E20_GM(8192,\"Clover_Ge_20MeV_G_Adj\");"
  }
  }
  if {$DataSourceOptions(clover,Ge4) == 1} then {
  if {$NeoCardOptions(clover,acrej) == 1} then {
     HISTH "MatHist1D<26,4> Q_E4_GM(8192,\"Clover_Ge_4MeV_G_Adj_Clean\");"
  } else {
     HISTH "MatHist1D<26,4> Q_E4_GM(8192,\"Clover_Ge_4MeV_G_Adj\");"
  }
  }
  }

#    create total energy histograms

    if {$DataSourceOptions(clover,Ge20) == 1 && $DataHistOptions(clover,Ge20T) == 1} then {
       HISTH "VectHist1D<26> Q_E20_TOTAL_RAW(8192,\"Clover_Ge_20MeV_Total_Raw\");"
    if {$NeoCardOptions(clover,acrej) == 1} then {
       HISTH "VectHist1D<26> Q_E20_TOTAL_CLEAN(8192,\"Clover_Ge_20MeV_Total_Clean\");"
    }
    }
    if {$DataSourceOptions(clover,Ge4) == 1 && $DataHistOptions(clover,Ge4T) == 1} then {
       HISTH "VectHist1D<26> Q_E4_TOTAL_RAW(8192,\"Clover_Ge_4MeV_Total_Raw\");"
    if {$NeoCardOptions(clover,acrej) == 1} then {
       HISTH "VectHist1D<26> Q_E4_TOTAL_CLEAN(8192,\"Clover_Ge_4MeV_Total_Clean\");"
    }
    }


     NEO "//    Clover detector section generated [date]\n\n\n\n"
     NEO "    if (clover.howManyFired()) \{       //   there are clover detectors in this event"
     NEO "        Astat.inc(72);       //  increment statistics histogram"
     NEO "        jj=0;"
     NEO "        while (jj < clover.howManyFired()) \{   //  loop for each detector"
     NEO "            keyDet = clover(jj).keyGet();"
     NEO "            Q_stat\[0\].inc(keyDet);       //  increment statistics histogram for detector"
     NEO "            valid = FALSE;"
     NEO ""
     NEO "//    BGO section"
     NEO ""
   if {$DataSourceOptions(clover,BGOE) == 1} then {
     NEO "            BgoEnergy = clover(jj).bgo.energy & 8191;   // extract energy from 13 bit adc"
   if {$DataHistOptions(clover,BGOE) == 1} then {
     NEO "            if (BgoEnergy) \{"
     NEO "                Q_E\[keyDet\].inc(BgoEnergy); // increment raw data histogram"
     NEO "                valid = TRUE;"
     NEO "            \}"
   }
     NEO ""
   }
   if {$DataSourceOptions(clover,BGOT) == 1 && $DataHistOptions(clover,BGOT) == 1} then {
     NEO "            temp = clover(jj).bgo.timing & 8191;   // extract data from 13 bit adc"
     NEO "            if (temp) Q_t\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
   if {$DataSourceOptions(clover,BGOP) == 1 && $DataHistOptions(clover,BGOP) == 1} then {
     NEO "            temp = clover(jj).bgo.pattern & 2047;   // extract pattern data"
     NEO "            if (temp) Q_p\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
     NEO ""
     NEO "//    Ge section"
     NEO ""
     NEO "        anti_compton = FALSE;"
     NEO "        pile_up = FALSE;"
     NEO "        one_ac = FALSE;"
     NEO "        all_ac = TRUE;"
     NEO "        E20 = 0;"
     NEO "        E4 = 0;"
     NEO " "
  if {$NeoCardOptions(clover,addback) == 1} then {
     NEO " //    initialise variables used for addback"
     NEO "	  for (j=0; j<4; j++) \{"
     NEO "	    tmp_gm4\[j\]=0;"
     NEO "	    tmp_gm20\[j\]=0;"
     NEO "	  \};"
     NEO ""
  }
     NEO "        for (j=0; j<4; j++) \{    // this loops over the 4 Ge crystals in a clover detector"
     NEO "        if(clover(jj).isSubdetFired(j) ) \{      // for the crystals which have fired"
     NEO "            Q_stat\[1\].inc(keyDet);       //  increment statistics histogram for detector "
     NEO "            keySpec = keyDet*4+j;"
     NEO "            Q_stat\[3\].inc(keySpec);       //  increment statistics histogram for crystal"

  if {$NeoCardOptions(clover,purej) == 1} then {
  if {$DataSourceOptions(clover,Ge20) == 1 || $DataSourceOptions(clover,Ge4) == 1} then {
     NEO "//    look for the q2 (pile-up) bit"
  if {$DataSourceOptions(clover,Ge20) == 1 && $DataSourceOptions(clover,Ge4) == 1} then {
     NEO "            if ((clover(jj).ge\[j\].E4 & 32768) || (clover(jj).ge\[j\].E20 & 32768)) \{"
  }
  if {$DataSourceOptions(clover,Ge20) == 1 && $DataSourceOptions(clover,Ge4) == 0} then {
     NEO "            if (clover(jj).ge\[j\].E20 & 32768) \{"
  }
  if {$DataSourceOptions(clover,Ge20) == 0 && $DataSourceOptions(clover,Ge4) == 1} then {
     NEO "            if (clover(jj).ge\[j\].E4 & 32768) \{"
  }
     NEO ""
     NEO "                 pile_up = TRUE;   //    the q2 (pile-up) bit is set"
     NEO "            \} else \{"
     NEO " //    no Pile Up"
  }
  }

  if {$NeoCardOptions(clover,acrej) == 1 || $NeoOptions(totebgo) == 1} then {
  if {$DataSourceOptions(clover,Ge20) == 1 || $DataSourceOptions(clover,Ge4) == 1} then {
     NEO "//    look for the q1 (anti-compton) bit"
  if {$DataSourceOptions(clover,Ge20) == 1 && $DataSourceOptions(clover,Ge4) == 1} then {
     NEO "            if ((clover(jj).ge\[j\].E4 & 16384) || (clover(jj).ge\[j\].E20 & 16384)) \{"
  }
  if {$DataSourceOptions(clover,Ge20) == 1 && $DataSourceOptions(clover,Ge4) == 0} then {
     NEO "            if (clover(jj).ge\[j\].E20 & 16384) \{"
  }
  if {$DataSourceOptions(clover,Ge20) == 0 && $DataSourceOptions(clover,Ge4) == 1} then {
     NEO "            if (clover(jj).ge\[j\].E4 & 16384) \{"
  }
     NEO ""
     NEO "//    the q1 (anti-compton) bit is set"
  if {$NeoCardOptions(clover,acrej) == 1} then {
     NEO "                 anti_compton = TRUE;"
  }
  if {$NeoOptions(totebgo) == 1 && $DataSourceOptions(clover,BGOE) == 1} then {
     NEO " //    We sum the Bgo anti-compton Energy converting from 8MeV range to 20MeV range"
     NEO "                 TotalBgoEnergy += (BgoEnergy * 8/20);    // We sum the Bgo anti-compton Energy"
  }
     NEO "            \}"
  }
  }

#    Section for a 20MeV Ge ADC

  if {$DataSourceOptions(clover,Ge20) == 1} then {
     NEO "//    Section for a 20MeV Ge ADC"
     NEO ""
     NEO "            E20 = clover(jj).ge\[j\].E20 & 8191;   // extract energy from 13 bit adc"
     NEO "            if (E20) \{"
  if {$NeoCardOptions(clover,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
  if {$DataHistOptions(clover,Ge20) == 1} then {
     NEO "              Q_E20\[keyDet\]\[j\].inc(E20);     //    histogram data (raw or clean)"
  }      
  if {$NeoCardOptions(clover,gm) == 1} then {
     NEO "              rnd++;rnd = rnd % 1000;     //  apply gain match correction"
     NEO "              E20 = (coeff20_a\[keySpec+105\]*(E20+rnd/1000.0)+coeff20_b\[keySpec+105\])/2.5;"
  }
     NEO "              if (E20 > 5) \{"
  if {$NeoCardOptions(clover,gmhist) == 1} then {
  if {$NeoCardOptions(clover,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "                Q_E20_GM\[keyDet\]\[j\].inc(E20);     //    histogram gain matched data"
  }
  if {$NeoOptions(tote) == 1} then {
     NEO "                tmp_gm20\[j\] = E20;           // to increment the total spectrum "
  }
     NEO "              \}"
     NEO "            \}"
  }

#    Section for a 4MeV Ge ADC

  if {$DataSourceOptions(clover,Ge4) == 1} then {
     NEO "//    Section for a 4MeV Ge ADC"
     NEO ""
     NEO "            E4 = clover(jj).ge\[j\].E4 & 8191;   // extract energy from 13 bit adc"
     NEO "            if (E4) \{"
  if {$DataHistOptions(clover,Ge4) == 1} then {
  if {$NeoCardOptions(clover,acrej) == 1} then {
     NEO "            if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "            Q_E4\[keyDet\]\[j\].inc(E4);     //    histogram data (raw or clean)"
  }
  if {$NeoCardOptions(clover,gm) == 1} then {
     NEO "              rnd++;rnd = rnd % 1000;     //  apply gain match correction"
     NEO "              E4 = 2*(coeff_a\[keySpec+105\]*(E4+rnd/1000.0)+coeff_b\[keySpec+105\]);"
  }
     NEO "              if (E4 > 5) \{"
  if {$NeoCardOptions(clover,gmhist) == 1} then {
  if {$NeoCardOptions(clover,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "                Q_E4_GM\[keyDet\]\[j\].inc(E4);     //    histogram gain matched data"  
  }
  if {$NeoOptions(tote) == 1} then {
     NEO "                tmp_gm4\[j\] = E4;           // to increment the total spectrum "
  }
     NEO "              \}"
     NEO "            \}"
     NEO ""
  }

#    Section for a Ge TAC

  if {$DataSourceOptions(clover,FT) == 1} then {
     NEO "//    Section for a Ge TAC"
     NEO ""
     NEO "            ft = clover(jj).ge\[j\].ft & 8191;   // extract data from 13 bit adc"
     NEO "            if (ft) \{"
  if {$DataHistOptions(clover,FT) == 1} then {
  if {$NeoCardOptions(clover,acrej) == 1} then {
     NEO "              if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "              Q_ft\[keyDet\]\[j\].inc(ft);     //    histogram data (raw or clean)"
  }
     NEO "            \}"
     NEO ""
  }

#    Section for a Ge CrossOver TAC

  if {$DataSourceOptions(clover,CO) == 1} then {
     NEO "//    Section for a Ge CrossOver TAC"
     NEO ""
     NEO "            co = clover(jj).ge\[j\].co & 8191;"
     NEO "            if (co) \{"
  if {$DataHistOptions(clover,FT) == 1} then {
  if {$NeoCardOptions(clover,acrej) == 1} then {
     NEO "              if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "              Q_co\[keyDet\]\[j\].inc(co);     //    histogram data (raw or clean)"
  }
     NEO "            \}"
     NEO ""
  }

     NEO "            if (E4 || E20 || BgoEnergy) valid = TRUE;   // data is valid if an energy item is present"

     NEO "		if (anti_compton == FALSE && (E4 || E20))"
     NEO "		  one_ac = TRUE;"
     NEO "		else"
     NEO "		  all_ac = FALSE;"

  if {$NeoCardOptions(clover,purej) == 1} then {
  if {$DataSourceOptions(clover,Ge20) == 1 || $DataSourceOptions(clover,Ge4) == 1} then {
     NEO "            \}   // end of pile-up "
  }
  }
     NEO "         \}      // end of subdetector fired"
     NEO "      \}         // end of all subdetectors"
     NEO ""
     NEO "      if(valid) \{"
     NEO "          Q_stat\[2\].inc(keyDet);       //  increment statistics histogram for good event"

   if {$DataSourceOptions(clover,Ge4) == 1 && $DataHistOptions(clover,Ge4T) == 1} then {
     NEO "	    temp_gm = 0;  // to build Total 4MeV Histograms"
     NEO "	    for (j=0; j<4; j++) "
     NEO "	      temp_gm += tmp_gm4\[j\];"
     NEO "	    if (temp_gm < 8192) \{"
     NEO "	      if (all_ac && one_ac) \{"
     NEO "		Q_E4_TOTAL_CLEAN\[keyDet\].inc(temp_gm);"
     NEO "	      \}"
     NEO "	      Q_E4_TOTAL_RAW\[keyDet\].inc(temp_gm);"
     NEO "	    \}"
   }
	    
   if {$DataSourceOptions(clover,Ge20) == 1 && $DataHistOptions(clover,Ge20T) == 1} then {
     NEO "	    temp_gm = 0;  // to build Total 20MeV Histograms"
     NEO "	    for (j=0; j<4; j++)"
     NEO "	      temp_gm += tmp_gm20\[j\];"
     NEO "	    if (temp_gm < 8192) \{"
     NEO "	       if (all_ac && one_ac) \{"
     NEO "	         Q_E20_TOTAL_CLEAN\[keyDet\].inc(temp_gm);"
     NEO "	       \};"
     NEO "	    Q_E20_TOTAL_RAW\[keyDet\].inc(temp_gm);"
  if {$NeoOptions(tote) == 1} then {
     NEO "	    tot_20MeV += temp_gm;    //  for the total energy histogram"
  }
     NEO "          \}"
  }	

     NEO "          jj++;"
     NEO "      \} else clover(jj).invalidate();   // this detector data is not written to tape"
     NEO ""
     NEO "      \}     //  end of detector"
     NEO "   \}        //  end of all detectors"
 
}

