

#    for cluster detectors a list of data sources
#    Note - these must be in order of Item number
set DataSources(cluster) [list "Ge20" "Ge4" "FT" "BGOE" "BGOT" "BGOPL" "BGOPM"]

set DataSources(clusterGe)  [list "Ge20" "Ge4" "FT"]
set DataSources(clusterBGO) [list "BGOE" "BGOT" "BGOPL" "BGOPM"]

#    for cluster detectors a description of each data source for the choice widget
set DataSourceID(cluster,Ge20)   "Ge 20MeV ADC"
set DataSourceID(cluster,Ge4)    "Ge 4MeV ADC"
set DataSourceID(cluster,FT)     "FT TAC"
set DataSourceID(cluster,BGOE)   "BGO Sum Energy"
set DataSourceID(cluster,BGOT)   "BGO Sum TAC"
set DataSourceID(cluster,BGOPL)   "BGO Pattern LSW"
set DataSourceID(cluster,BGOPM)   "BGO Pattern MSW"

#    the name of the register
set DataSourceReg(cluster,Ge20)   "20MeV"
set DataSourceReg(cluster,Ge4)    "4MeV"
set DataSourceReg(cluster,FT)     "FT"
set DataSourceReg(cluster,BGOE)   "BGOSum.Energy"
set DataSourceReg(cluster,BGOT)   "BGOSum.TAC"
set DataSourceReg(cluster,BGOPL)  "BGOPattern.Low"
set DataSourceReg(cluster,BGOPM)  "BGOPattern.High"

#    for cluster detectors the default state of the InUse option
set DataSourceOptions(cluster,Ge20)   1
set DataSourceOptions(cluster,Ge4)    1
set DataSourceOptions(cluster,FT)     1
set DataSourceOptions(cluster,BGOE)   1
set DataSourceOptions(cluster,BGOT)   1
set DataSourceOptions(cluster,BGOPL)   1
set DataSourceOptions(cluster,BGOPM)   1

#    for cluster detectors the default state of the Hist option
set DataHistOptions(cluster,Ge20)   1
set DataHistOptions(cluster,Ge4)    1
set DataHistOptions(cluster,FT)     1
set DataHistOptions(cluster,BGOE)   1
set DataHistOptions(cluster,BGOT)   1
set DataHistOptions(cluster,BGOPL)   0
set DataHistOptions(cluster,BGOPM)   0

set DataHistOptions(cluster,Ge20T)   1
set DataHistOptions(cluster,Ge4T)   1


#    for cluster detectors the default state of the Tape option
set DataTapeOptions(cluster,Ge20)   1
set DataTapeOptions(cluster,Ge4)    1
set DataTapeOptions(cluster,FT)     1
set DataTapeOptions(cluster,BGOE)   1
set DataTapeOptions(cluster,BGOT)   1
set DataTapeOptions(cluster,BGOPL)   1
set DataTapeOptions(cluster,BGOPM)   1


set NeoCardOptionChoice(cluster) [list acrej purej addback gm gmhist]

set NeoCardOptionID(cluster,acrej)       "Reject Ge data with anti-compton"
set NeoCardOptionID(cluster,purej)       "Reject Ge data with pile-up"
set NeoCardOptionID(cluster,addback)     "Perform Ge crystal energy addback"
set NeoCardOptionID(cluster,gm)          "Gain match Ge detectors"
set NeoCardOptionID(cluster,gmhist)      "Histogram gain matched data"

set NeoCardOptions(cluster,acrej)       1
set NeoCardOptions(cluster,purej)       1
set NeoCardOptions(cluster,addback)     1
set NeoCardOptions(cluster,gm)          1
set NeoCardOptions(cluster,gmhist)      1





#    hardware probe to check setup
   
proc probe_cluster {} {
   global env VXIServer DataSources DataSourceReg DataSourceOptions DataSourceID

  :frame "[self frame-name].probe" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :galley REPORT :rows 15 :cols 80 :deaf T

    :modify REPORT :strings "\nLooking for Cluster Ge VXI Card data word status\n\n"
    :show T

    set num 0

  for {set c 0} {$c < 15} {incr c} {
    set numseg ""
   foreach s {A B C D E F G H} {
    foreach item $DataSources(clusterGe) {
      set VXIregister /$VXIServer/ClusterGe$c.Ge$s.$DataSourceReg(cluster,$item).ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(cluster,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(cluster,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware for Ge channel $c segment $s.\n"
             incr num
          }
      }
      if {$DataSourceOptions(cluster,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(cluster,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware for Ge channel $c segment $s.\n"
             incr num
          }
      }
    }
    set VXIregister /$VXIServer/ClusterGe$c.Ge$s.Enable
    set VXIdata [EG "register :name '$VXIregister' ?value"]
    if {$VXIdata == 1} {append numseg "$s "}
   }
    if {[string length $numseg] != 14} then {
       :modify REPORT :strings "Warning - Ge channel $c; segments $numseg are enabled which is incorrect. Exactly 7 segments should be enabled\n"
    }
  }

#    some checks for things not used

    :modify REPORT :strings "\nLooking for Cluster Ge VXI Card GIR data word status\n\n"

  for {set c 0} {$c < 15} {incr c} {

   set VXIregister /$VXIServer/ClusterGe$c.GIR.ROCR.EvNumData.Disable
   set VXIdata [EG "register :name '$VXIregister' ?value"]
   if {$VXIdata == 0} then {
      :modify REPORT :strings "Warning - data item ClusterGe$c.GIR.ROCR.EvNumData IS enabled.\n"
      incr num
   }
  }

    :modify REPORT :strings "\nLooking for Cluster BGO VXI Card Sum channel data word status\n\n"

  for {set c 0} {$c < 15} {incr c} {
    foreach item $DataSources(clusterBGO) {
      set VXIregister /$VXIServer/ClusterBGO$c.$DataSourceReg(cluster,$item).ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(cluster,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(cluster,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware for BGO channel $c.\n"
             incr num
          }
      }
      if {$DataSourceOptions(cluster,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(cluster,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware for BGO channel $c.\n"
             incr num
          }
      }
    }
    set VXIregister /$VXIServer/ClusterBGO$c.BGOSum.Enable
    set VXIdata [EG "register :name '$VXIregister' ?value"]
    if {$VXIdata == 0} then {
       :modify REPORT :strings "Warning - BGO channel $c is NOT enabled.\n"
    }
  }

#    some checks for things not used

    :modify REPORT :strings "\nLooking for Cluster BGO VXI Card GIR data word status\n\n"

  for {set c 0} {$c < 15} {incr c} {

   set VXIregister /$VXIServer/ClusterBGO$c.GIR.ROCR.EvNumData.Disable
   set VXIdata [EG "register :name '$VXIregister' ?value"]
   if {$VXIdata == 0} then {
      :modify REPORT :strings "Warning - data item ClusterBGO$c.GIR.ROCR.EvNumData IS enabled.\n"
      incr num
   }
  }
 
    :modify REPORT :strings "\nLooking for Cluster BGO VXI Card individual channel data word status\n\n"

  for {set c 0} {$c < 15} {incr c} {
   foreach s {A B C D E F G H I J K L M N O P Q R S} {
    foreach item {Energy TAC} {
      set VXIregister /$VXIServer/ClusterBGO$c.BGO$s.$item.ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$VXIdata == 1} then {
          :modify REPORT :strings "Warning - data item ClusterBGO$c.BGO$s.$item IS enabled.\n"
          incr num
      }
    }
   }
  }

    if {$num == 0} then {:modify REPORT :strings "\nYour NEO++ statements for this card are compatible with the hardware setup - Well done!\n"}
    if {$num != 0} then {:modify REPORT :strings "\nThere are $num inconsistences between your NEO++ statements for this card and the hardware setup\n"}
}


#    phase 1 code generation - declarations

proc neogen1_cluster {} {
    global DataSourceOptions DataHistOptions

#    generate subdetector definitions

    EVENTH "//    Cluster detector section generated [date]\n\n"

#    first do the Ge part

    EVENTH "//   Ge detector definition for cluster detector"
    EVENTH "subdetector (ClusterGe)"
    if {$DataSourceOptions(cluster,Ge20) == 1} then {
       EVENTH "    item_q E20;"
    } else {
       EVENTH "    dummy_item E20;"
    }
    if {$DataSourceOptions(cluster,Ge4) == 1} then {
       EVENTH "    item_q E4;"
    } else {
       EVENTH "    dummy_item E4;"
    }
    if {$DataSourceOptions(cluster,FT) == 1} then {
       EVENTH "    item ft;"
    } else {
       EVENTH "    dummy_item ft;"
    }
    EVENTH "end_subdetector"

#    then do the BGO part

    set bgopresent 0

    EVENTH "//   BGO Sum Channel detector definition for cluster detector"
    EVENTH "subdetector (ClusterBGO)"
    if {$DataSourceOptions(cluster,BGOE) == 1} then {
       EVENTH "    item energy;"
       set bgopresent 1
    } else {
       EVENTH "    dummy_item energy;"
    }
    if {$DataSourceOptions(cluster,BGOT) == 1} then {
        EVENTH "    item timing;"
        set bgopresent 1
    } else {
        EVENTH "    dummy_item timing;"
    }
    if {$DataSourceOptions(cluster,BGOPL) == 1} then {
         EVENTH "    item patternLSW;"
         set bgopresent 1
    } else {
         EVENTH "    dummy_item patternLSW;"
    }
    if {$DataSourceOptions(cluster,BGOPM) == 1} then {
         EVENTH "    item patternMSW;"
         set bgopresent 1
    } else {
         EVENTH "    dummy_item patternMSW;"
    }
    EVENTH "end_subdetector"

#    generate detector format

     EVENTH "format (Cluster)"
     EVENTH "    ClusterGe     ge\[7\];"
     if {$bgopresent != 0} then {EVENTH "    ClusterBGO     bgo;"}
     EVENTH "end_format"

     EVENTH "Detector<Cluster, CLUSTER> cluster;"
}


proc neogen2_cluster {} {
   global DataSourceOptions DataHistOptions NeoOptions NeoCardOptions

     HISTH "//    Cluster detector section generated [date]\n\n\n"

#    generate histograms for online monitoring

     HISTH "VectHist1D<4> C_stat(128,\"Astat_Cluster\");"

    if {$DataSourceOptions(cluster,Ge20) == 1 && $DataHistOptions(cluster,Ge20) == 1} then {
       HISTH "MatHist1D<15,7> C_E20(8192,\"Cluster_Ge_20MeV\");"
    }
    if {$DataSourceOptions(cluster,Ge4) == 1 && $DataHistOptions(cluster,Ge4) == 1} then {
       HISTH "MatHist1D<15,7> C_E4(8192,\"Cluster_Ge_4MeV\");"
    }
    if {$DataSourceOptions(cluster,BGOE) == 1 && $DataHistOptions(cluster,BGOE) == 1} then {
       HISTH "VectHist1D<15> C_E(8192,\"Cluster_BGO_Energy\");"
    }
    if {$DataSourceOptions(cluster,BGOT) == 1 && $DataHistOptions(cluster,BGOT) == 1} then {
       HISTH "VectHist1D<15> C_t(8192,\"Cluster_BGO_TAC\");"
    }
    if {$DataSourceOptions(cluster,BGOPL) == 1 && $DataHistOptions(cluster,BGOPL) == 1} then {
       HISTH "VectHist1D<15> C_pLSW(1024,\"Cluster_BGO_PatternLSW\");"
    }
    if {$DataSourceOptions(cluster,BGOPM) == 1 && $DataHistOptions(cluster,BGOPM) == 1} then {
       HISTH "VectHist1D<15> C_pMSW(1024,\"Cluster_BGO_PatternMSW\");"
    }   

    if {$DataSourceOptions(cluster,FT) == 1 && $DataHistOptions(cluster,FT) == 1} then {
       HISTH "MatHist1D<15,7> C_ft(8192,\"Cluster_Ge_FT\");"
    }

#    create gain matched histograms as required for 4 and 20 MeV 

  if {$NeoCardOptions(cluster,gmhist) == 1} then {
  if {$DataSourceOptions(cluster,Ge20) == 1} then {
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     HISTH "MatHist1D<15,7> C_E20_GM(8192,\"Cluster_Ge_20MeV_G_Adj_Clean\");"
  } else {
     HISTH "MatHist1D<15,7> C_E20_GM(8192,\"Cluster_Ge_20MeV_G_Adj\");"
  }
  }
  if {$DataSourceOptions(cluster,Ge4) == 1} then {
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     HISTH "MatHist1D<15,7> C_E4_GM(8192,\"Cluster_Ge_4MeV_G_Adj_Clean\");"
  } else {
     HISTH "MatHist1D<15,7> C_E4_GM(8192,\"Cluster_Ge_4MeV_G_Adj\");"
  }
  }
  }

#    create total energy histograms

    if {$DataSourceOptions(cluster,Ge20) == 1 && $DataHistOptions(cluster,Ge20T) == 1} then {
       HISTH "VectHist1D<15> C_E20_TOTAL_RAW(8192,\"Cluster_Ge_20MeV_Total_Raw\");"
    if {$NeoCardOptions(cluster,acrej) == 1} then {
       HISTH "VectHist1D<15> C_E20_TOTAL_CLEAN(8192,\"Cluster_Ge_20MeV_Total_Clean\");"
    }
    }
    if {$DataSourceOptions(cluster,Ge4) == 1 && $DataHistOptions(cluster,Ge4T) == 1} then {
       HISTH "VectHist1D<15> C_E4_TOTAL_RAW(8192,\"Cluster_Ge_4MeV_Total_Raw\");"
    if {$NeoCardOptions(cluster,acrej) == 1} then {
       HISTH "VectHist1D<15> C_E4_TOTAL_CLEAN(8192,\"Cluster_Ge_4MeV_Total_Clean\");"
    }
    }


     NEO "//    Cluster detector section generated [date]\n\n\n\n"
     NEO "    if (cluster.howManyFired()) \{       //   there are cluster detectors in this event"
     NEO "        Astat.inc(71);       //  increment statistics histogram"
     NEO "        jj=0;"
     NEO "        while (jj < cluster.howManyFired()) \{   //  loop for each detector"
     NEO "            keyDet = cluster(jj).keyGet();"
     NEO "            C_stat\[0\].inc(keyDet);       //  increment statistics histogram for detector"
     NEO "            valid = FALSE;"
     NEO ""
     NEO "//    BGO section"
     NEO ""
   if {$DataSourceOptions(cluster,BGOE) == 1} then {
     NEO "            BgoEnergy = cluster(jj).bgo.energy & 8191;   // extract energy from 13 bit adc"
   if {$DataHistOptions(cluster,BGOE) == 1} then {
     NEO "            if (BgoEnergy) \{"
     NEO "                C_E\[keyDet\].inc(BgoEnergy); // increment raw data histogram"
     NEO "                valid = TRUE;"
     NEO "            \}"
   }
     NEO ""
   }
   if {$DataSourceOptions(cluster,BGOT) == 1 && $DataHistOptions(cluster,BGOT) == 1} then {
     NEO "            temp = cluster(jj).bgo.timing & 8191;   // extract data from 13 bit adc"
     NEO "            if (temp) C_t\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
   if {$DataSourceOptions(cluster,BGOPL) == 1 && $DataHistOptions(cluster,BGOPL) == 1} then {
     NEO "            temp = cluster(jj).bgo.patternLSW & 1023;   // extract pattern data"
     NEO "            if (temp) C_pLSW\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
   if {$DataSourceOptions(cluster,BGOPM) == 1 && $DataHistOptions(cluster,BGOPM) == 1} then {
     NEO "            temp = cluster(jj).bgo.patternMSW & 1023;   // extract pattern data"
     NEO "            if (temp) C_pMSW\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
     NEO ""
     NEO "//    Ge section"
     NEO ""
     NEO "        anti_compton = FALSE;"
     NEO "        pile_up = FALSE;"
     NEO "        one_ac = FALSE;"
     NEO "        all_ac = TRUE;"
     NEO "        E20 = 0;"
     NEO "        E4 = 0;"
     NEO " "
  if {$NeoCardOptions(cluster,addback) == 1} then {
     NEO " //    initialise variables used for addback"
     NEO "	  for (j=0; j<7; j++) \{"
     NEO "	    tmp_gm4\[j\]=0;"
     NEO "	    tmp_gm20\[j\]=0;"
     NEO "	  \};"
     NEO ""
  }
     NEO "        for (j=0; j<7; j++) \{    // this loops over the 7 Ge crystals in a cluster detector"
     NEO "        if(cluster(jj).isSubdetFired(j) ) \{      // for the crystals which have fired"
     NEO "            C_stat\[1\].inc(keyDet);       //  increment statistics histogram for detector "
     NEO "            keySpec = keyDet*7+j;"
     NEO "            C_stat\[3\].inc(keySpec);       //  increment statistics histogram for crystal"

  if {$NeoCardOptions(cluster,purej) == 1} then {
  if {$DataSourceOptions(cluster,Ge20) == 1 || $DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "//    look for the q2 (pile-up) bit"
  if {$DataSourceOptions(cluster,Ge20) == 1 && $DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "            if ((cluster(jj).ge\[j\].E4 & 32768) || (cluster(jj).ge\[j\].E20 & 32768)) \{"
  }
  if {$DataSourceOptions(cluster,Ge20) == 1 && $DataSourceOptions(cluster,Ge4) == 0} then {
     NEO "            if (cluster(jj).ge\[j\].E20 & 32768) \{"
  }
  if {$DataSourceOptions(cluster,Ge20) == 0 && $DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "            if (cluster(jj).ge\[j\].E4 & 32768) \{"
  }
     NEO ""
     NEO "                 pile_up = TRUE;   //    the q2 (pile-up) bit is set"
     NEO "            \} else \{"
     NEO " //    no Pile Up"
  }
  }

  if {$NeoCardOptions(cluster,acrej) == 1 || $NeoOptions(totebgo) == 1} then {
  if {$DataSourceOptions(cluster,Ge20) == 1 || $DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "//    look for the q1 (anti-compton) bit"
  if {$DataSourceOptions(cluster,Ge20) == 1 && $DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "            if ((cluster(jj).ge\[j\].E4 & 16384) || (cluster(jj).ge\[j\].E20 & 16384)) \{"
  }
  if {$DataSourceOptions(cluster,Ge20) == 1 && $DataSourceOptions(cluster,Ge4) == 0} then {
     NEO "            if (cluster(jj).ge\[j\].E20 & 16384) \{"
  }
  if {$DataSourceOptions(cluster,Ge20) == 0 && $DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "            if (cluster(jj).ge\[j\].E4 & 16384) \{"
  }
     NEO ""
     NEO "//    the q1 (anti-compton) bit is set"
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     NEO "                 anti_compton = TRUE;"
  }
  if {$NeoOptions(totebgo) == 1 && $DataSourceOptions(cluster,BGOE) == 1} then {
     NEO "                 TotalBgoEnergy += BgoEnergy;    // We sum the Bgo anti-compton Energy"
  }
     NEO "            \}"
  }
  }

#    Section for a 20MeV Ge ADC

  if {$DataSourceOptions(cluster,Ge20) == 1} then {
     NEO "//    Section for a 20MeV Ge ADC"
     NEO ""
     NEO "            E20 = cluster(jj).ge\[j\].E20 & 8191;   // extract energy from 13 bit adc"
     NEO "            if (E20) \{"
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
  if {$DataHistOptions(cluster,Ge20) == 1} then {
     NEO "              C_E20\[keyDet\]\[j\].inc(E20);     //    histogram data (raw or clean)"
  }      
  if {$NeoCardOptions(cluster,gm) == 1} then {
     NEO "              rnd++;rnd = rnd % 1000;     //  apply gain match correction"
     NEO "              E20 = (coeff20_a\[keySpec\]*(E20+rnd/1000.0)+coeff20_b\[keySpec\])/2.5;"
  }
     NEO "              if (E20 > 5) \{"
  if {$NeoCardOptions(cluster,gmhist) == 1} then {
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "                C_E20_GM\[keyDet\]\[j\].inc(E20);     //    histogram gain matched data"
  }
  if {$NeoOptions(tote) == 1} then {
     NEO "                tmp_gm20\[j\] = E20;           // to increment the total spectrum "
  }
     NEO "              \}"
     NEO "            \}"
  }

#    Section for a 4MeV Ge ADC

  if {$DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "//    Section for a 4MeV Ge ADC"
     NEO ""
     NEO "            E4 = cluster(jj).ge\[j\].E4 & 8191;   // extract energy from 13 bit adc"
     NEO "            if (E4) \{"
  if {$DataHistOptions(cluster,Ge4) == 1} then {
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     NEO "            if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "            C_E4\[keyDet\]\[j\].inc(E4);     //    histogram data (raw or clean)"
  }
  if {$NeoCardOptions(cluster,gm) == 1} then {
     NEO "              rnd++;rnd = rnd % 1000;     //  apply gain match correction"
     NEO "              E4 = 2*(coeff_a\[keySpec\]*(E4+rnd/1000.0)+coeff_b\[keySpec\]);"
  }
     NEO "              if (E4 > 5) \{"
  if {$NeoCardOptions(cluster,gmhist) == 1} then {
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "                C_E4_GM\[keyDet\]\[j\].inc(E4);     //    histogram gain matched data"  
  }
  if {$NeoOptions(tote) == 1} then {
     NEO "                tmp_gm4\[j\] = E4;           // to increment the total spectrum "
  }
     NEO "              \}"
     NEO "            \}"
     NEO ""
  }

#    Section for a Ge TAC

  if {$DataSourceOptions(cluster,FT) == 1} then {
     NEO "//    Section for a Ge TAC"
     NEO ""
     NEO "            ft = cluster(jj).ge\[j\].ft & 8191;   // extract data from 13 bit adc"
     NEO "            if (ft) \{"
  if {$DataHistOptions(cluster,FT) == 1} then {
  if {$NeoCardOptions(cluster,acrej) == 1} then {
     NEO "              if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "              C_ft\[keyDet\]\[j\].inc(ft);     //    histogram data (raw or clean)"
  }
     NEO "            \}"
     NEO ""
  }

     NEO "            if (E4 || E20 || BgoEnergy) valid = TRUE;   // data is valid if an energy item is present"

     NEO "		if (anti_compton == FALSE && (E4 || E20))"
     NEO "		  one_ac = TRUE;"
     NEO "		else"
     NEO "		  all_ac = FALSE;"

  if {$NeoCardOptions(cluster,purej) == 1} then {
  if {$DataSourceOptions(cluster,Ge20) == 1 || $DataSourceOptions(cluster,Ge4) == 1} then {
     NEO "            \}   // end of pile-up "
  }
  }
     NEO "         \}      // end of subdetector fired"
     NEO "      \}         // end of all subdetectors"
     NEO ""
     NEO "      if(valid) \{"
     NEO "          C_stat\[2\].inc(keyDet);       //  increment statistics histogram for good event"

   if {$DataSourceOptions(cluster,Ge4) == 1 && $DataHistOptions(cluster,Ge4T) == 1} then {
     NEO "	    temp_gm = 0;  // to build Total 4MeV Histograms"
     NEO "	    for (j=0; j<7; j++) "
     NEO "	      temp_gm += tmp_gm4\[j\];"
     NEO "	    if (temp_gm < 8192) \{"
     NEO "	      if (all_ac && one_ac) \{"
     NEO "		C_E4_TOTAL_CLEAN\[keyDet\].inc(temp_gm);"
     NEO "	      \}"
     NEO "	      C_E4_TOTAL_RAW\[keyDet\].inc(temp_gm);"
     NEO "	    \}"
   }
	    
   if {$DataSourceOptions(cluster,Ge20) == 1 && $DataHistOptions(cluster,Ge20T) == 1} then {
     NEO "	    temp_gm = 0;  // to build Total 20MeV Histograms"
     NEO "	    for (j=0; j<7; j++)"
     NEO "	      temp_gm += tmp_gm20\[j\];"
     NEO "	    if (temp_gm < 8192) \{"
     NEO "	       if (all_ac && one_ac) \{"
     NEO "	         C_E20_TOTAL_CLEAN\[keyDet\].inc(temp_gm);"
     NEO "	       \};"
     NEO "	    C_E20_TOTAL_RAW\[keyDet\].inc(temp_gm);"
  if {$NeoOptions(tote) == 1} then {
     NEO "	    tot_20MeV += temp_gm;    //  for the total energy histogram"
  }
     NEO "          \}"
  }	

     NEO "          jj++;"
     NEO "      \} else cluster(jj).invalidate();   // this detector data is not written to tape"
     NEO ""
     NEO "      \}     //  end of detector"
     NEO "   \}        //  end of all detectors"
 
}

