


#    for tapered detectors a list of data sources
#    Note - these must be in order of Item number
set DataSources(tapered) [list "Ge20" "Ge4" "FT" "CO" "BGOE" "BGOT" "BGOP"]

set DataSources(taperedGe)  [list "Ge20" "Ge4" "FT" "CO"]
set DataSources(taperedBGO) [list "BGOE" "BGOT" "BGOP"]

#    for tapered detectors a description of each data source for the choice widget
set DataSourceID(tapered,Ge20)   "Ge 20MeV ADC"
set DataSourceID(tapered,Ge4)    "Ge 4MeV ADC"
set DataSourceID(tapered,FT)     "FT TAC"
set DataSourceID(tapered,CO)     "CO TAC"
set DataSourceID(tapered,BGOE)   "BGO Energy"
set DataSourceID(tapered,BGOT)   "BGO TAC"
set DataSourceID(tapered,BGOP)   "BGO Pattern"

#    the name of the register
set DataSourceReg(tapered,Ge20)   "20MeV"
set DataSourceReg(tapered,Ge4)    "4MeV"
set DataSourceReg(tapered,FT)     "FT"
set DataSourceReg(tapered,CO)     "CO"
set DataSourceReg(tapered,BGOE)   "Energy"
set DataSourceReg(tapered,BGOT)   "TAC"
set DataSourceReg(tapered,BGOP)   "Pattern"

#    for tapered detectors the default state of the InUse option
set DataSourceOptions(tapered,Ge20)   1
set DataSourceOptions(tapered,Ge4)    1
set DataSourceOptions(tapered,FT)     1
set DataSourceOptions(tapered,CO)     0
set DataSourceOptions(tapered,BGOE)   1
set DataSourceOptions(tapered,BGOT)   1
set DataSourceOptions(tapered,BGOP)   1

#    for tapered detectors the default state of the Hist option
set DataHistOptions(tapered,Ge20)   1
set DataHistOptions(tapered,Ge4)    1
set DataHistOptions(tapered,FT)     1
set DataHistOptions(tapered,CO)     1
set DataHistOptions(tapered,BGOE)   1
set DataHistOptions(tapered,BGOT)   1
set DataHistOptions(tapered,BGOP)   1

#    for tapered detectors the default state of the Tape option
set DataTapeOptions(tapered,Ge20)   1
set DataTapeOptions(tapered,Ge4)    1
set DataTapeOptions(tapered,FT)     1
set DataTapeOptions(tapered,CO)     1
set DataTapeOptions(tapered,BGOE)   1
set DataTapeOptions(tapered,BGOT)   1
set DataTapeOptions(tapered,BGOP)   1

set NeoCardOptionChoice(tapered) [list acrej gm gmhist]

set NeoCardOptionID(tapered,acrej)       "Reject Ge data with anti-compton"
set NeoCardOptionID(tapered,gm)          "Gain match Ge detectors"
set NeoCardOptionID(tapered,gmhist)      "Histogram gain matched data"

set NeoCardOptions(tapered,acrej)       1
set NeoCardOptions(tapered,gm)          1
set NeoCardOptions(tapered,gmhist)      1




#    hardware probe to check setup
   
proc probe_tapered {} {
   global env VXIServer DataSources DataSourceReg DataSourceOptions DataSourceID

  :frame "[self frame-name].probe" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :galley REPORT :rows 15 :cols 80 :deaf T

    :modify REPORT :strings "\nLooking for Tapered Ge VXI Card data word status\n\n"
    :show T

    set num 0
    for {set c 0} {$c < 36} {incr c} {
    foreach item $DataSources(taperedGe) {
      set VXIregister /$VXIServer/Ge$c.$DataSourceReg(tapered,$item).ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(tapered,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(tapered,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware for channel $c.\n"
             incr num
          }
      }
      if {$DataSourceOptions(tapered,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(tapered,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware for channel $c.\n"
             incr num
          }
      }
    }
    set VXIregister /$VXIServer/Ge$c.Enable
    set VXIdata [EG "register :name '$VXIregister' ?value"]
    if {$VXIdata == 0} then {
       :modify REPORT :strings "Warning - Ge channel $c IS NOT enabled\n"
    }
    }

    :modify REPORT :strings "\nLooking for Tapered BGO VXI Card data word status\n\n"

    for {set c 0} {$c < 36} {incr c} {
    foreach item $DataSources(taperedBGO) {
      set VXIregister /$VXIServer/BGO$c.$DataSourceReg(tapered,$item).ROEnable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(tapered,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(tapered,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware for channel $c.\n"
             incr num
          }
      }
      if {$DataSourceOptions(tapered,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(tapered,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware for channel $c.\n"
             incr num
          }
      }
    }
    set VXIregister /$VXIServer/BGO$c.Disable
    set VXIdata [EG "register :name '$VXIregister' ?value"]
    if {$VXIdata == 1} then {
        :modify REPORT :strings "Warning - BGO channel $c IS NOT enabled\n"
    }
    }

    if {$num == 0} then {:modify REPORT :strings "\nYour NEO++ statements for this card are compatible with the hardware setup - Well done!\n"}
    if {$num != 0} then {:modify REPORT :strings "\nThere are $num inconsistences between your NEO++ statements for this card and the hardware setup\n"}
}


#    phase 1 code generation - declarations

proc neogen1_tapered {} {
    global DataSourceOptions DataHistOptions

#    generate subdetector definitions

    EVENTH "//    Tapered detector section generated [date]\n\n\n\n"

#    first do the Ge part

    EVENTH "//   Ge detector definition for tapered detector"
    EVENTH "subdetector (GeT)"
    if {$DataSourceOptions(tapered,Ge20) == 1} then {
       EVENTH "    item_q E20;"
    } else {
       EVENTH "    dummy_item E20;"
    }
    if {$DataSourceOptions(tapered,Ge4) == 1} then {
       EVENTH "    item_q E4;"
    } else {
       EVENTH "    dummy_item E4;"
    }
    if {$DataSourceOptions(tapered,FT) == 1} then {
       EVENTH "    item ft;"
    } else {
       EVENTH "    dummy_item ft;"
    }
    if {$DataSourceOptions(tapered,CO) == 1} then {
       EVENTH "    item CO;"
    } else {
       EVENTH "    dummy_item CO;"
    }
    EVENTH "end_subdetector"

#    then do the BGO part

    set bgopresent 0

    EVENTH "//   BGO detector definition for tapered detector"
    EVENTH "subdetector (BGOT)"
    if {$DataSourceOptions(tapered,BGOE) == 1} then {
       EVENTH "    item energy;"
       set bgopresent 1
    } else {
       EVENTH "    dummy_item energy;"
    }
    if {$DataSourceOptions(tapered,BGOT) == 1} then {
        EVENTH "    item timing;"
        set bgopresent 1
    } else {
        EVENTH "    dummy_item timing;"
    }
    if {$DataSourceOptions(tapered,BGOP) == 1} then {
         EVENTH "    item pattern;"
         set bgopresent 1
    } else {
         EVENTH "    dummy_item pattern;"
    }
    EVENTH "end_subdetector"

#    generate detector format

     EVENTH "format (Tapered)"
     EVENTH "    GeT     ge\[1\];"
     if {$bgopresent != 0} then {EVENTH "    BGOT     bgo;"}
     EVENTH "end_format"

     EVENTH "Detector<Tapered, TAPERED> tapered;"
 
}


proc neogen2_tapered {} {
   global DataSourceOptions DataHistOptions NeoOptions NeoCardOptions

     HISTH "//    Tapered detector section generated [date]\n\n\n"

#    generate histograms for online monitoring

     HISTH "VectHist1D<4> T_stat(128,\"Astat_Tapered\");"

    if {$DataSourceOptions(tapered,Ge20) == 1 && $DataHistOptions(tapered,Ge20) == 1} then {
       HISTH "VectHist1D<36> T_E20(8192,\"Ge_20MeV\");"
    }
    if {$DataSourceOptions(tapered,Ge4) == 1 && $DataHistOptions(tapered,Ge4) == 1} then {
       HISTH "VectHist1D<36> T_E4(8192,\"Ge_4MeV\");"
    }
    if {$DataSourceOptions(tapered,BGOE) == 1 && $DataHistOptions(tapered,BGOE) == 1} then {
       HISTH "VectHist1D<36> T_E(8192,\"BGO_Energy\");"
    }
    if {$DataSourceOptions(tapered,BGOT) == 1 && $DataHistOptions(tapered,BGOT) == 1} then {
       HISTH "VectHist1D<36> T_t(8192,\"BGO_TAC\");"
    }
    if {$DataSourceOptions(tapered,BGOP) == 1 && $DataHistOptions(tapered,BGOP) == 1} then {
       HISTH "VectHist1D<36> T_p(2048,\"BGO_Pattern\");"
    }

    if {$DataSourceOptions(tapered,FT) == 1 && $DataHistOptions(tapered,FT) == 1} then {
    if {$NeoCardOptions(tapered,acrej) == 1} then {
       HISTH "VectHist1D<36> T_ft(8192,\"Ge_FT\");"
    } else {
       HISTH "VectHist1D<36> T_ft(8192,\"Ge_FT_Clean\");"
    }
    }

    if {$DataSourceOptions(tapered,CO) == 1 && $DataHistOptions(tapered,CO) == 1} then {
    if {$NeoCardOptions(tapered,acrej) == 1} then {
       HISTH "VectHist1D<36> T_co(8192,\"Ge_CO\");"
    } else {
       HISTH "VectHist1D<36> T_co(8192,\"Ge_CO_Clean\");"
    }
    }

#    create gain matched histograms as required for 4 and 20 MeV 

  if {$NeoCardOptions(tapered,gmhist) == 1} then {
  if {$DataSourceOptions(tapered,Ge20) == 1} then {
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     HISTH "VectHist1D<36> T_E20_GM(8192,\"Ge_20MeV_G_Adj_Clean\");"
  } else {
     HISTH "VectHist1D<36> T_E20_GM(8192,\"Ge_20MeV_G_Adj\");"
  }
  }
  if {$DataSourceOptions(tapered,Ge4) == 1} then {
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     HISTH "VectHist1D<36> T_E4_GM(8192,\"Ge_4MeV_G_Adj_Clean\");"
  } else {
     HISTH "VectHist1D<36> T_E4_GM(8192,\"Ge_4MeV_G_Adj\");"
  }
  }
  }

     NEO "//    Tapered detector section generated [date]\n\n\n\n"
     NEO "    if (tapered.howManyFired()) \{       //   there are tapered detectors in this event"
     NEO "        Astat.inc(73);       //  increment statistics histogram"
     NEO "        jj=0;"
     NEO "        while (jj < tapered.howManyFired()) \{   //  loop for each detector"
     NEO "            keyDet = tapered(jj).keyGet();"
     NEO "            T_stat\[0\].inc(keyDet);       //  increment statistics histogram for detector"
     NEO "            valid = FALSE;"
     NEO ""
     NEO "//    BGO section"
     NEO ""
   if {$DataSourceOptions(tapered,BGOE) == 1} then {
     NEO "            BgoEnergy = tapered(jj).bgo.energy & 8191;   // extract energy from 13 bit adc"
   if {$DataHistOptions(tapered,BGOE) == 1} then {
     NEO "            if (BgoEnergy) \{"
     NEO "                T_E\[keyDet\].inc(BgoEnergy); // increment raw data histogram"
     NEO "                valid = TRUE;"
     NEO "            \}"
   }
     NEO ""
   }
   if {$DataSourceOptions(tapered,BGOT) == 1 && $DataHistOptions(tapered,BGOT) == 1} then {
     NEO "            temp = tapered(jj).bgo.timing & 8191;   // extract data from 13 bit adc"
     NEO "            if (temp) T_t\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
   if {$DataSourceOptions(tapered,BGOP) == 1 && $DataHistOptions(tapered,BGOP) == 1} then {
     NEO "            temp = tapered(jj).bgo.pattern & 2047;   // extract pattern data"
     NEO "            if (temp) T_p\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
     NEO ""
     NEO "//    Ge section"
     NEO ""
     NEO "        anti_compton = FALSE;"
     NEO "        E20 = 0;"
     NEO "        E4 = 0;"
     NEO "        for (j=0; j<1; j++) \{    // this loops over the 1 Ge crystal in a tapered detector"
     NEO "        if(tapered(jj).isSubdetFired(j) ) \{      // for the 1 crystal which has fired"
     NEO "            T_stat\[1\].inc(keyDet);       //  increment statistics histogram for detector "
     NEO "            keySpec = keyDet;"
     NEO "            T_stat\[3\].inc(keySpec);       //  increment statistics histogram for crystal"
  if {$NeoCardOptions(tapered,acrej) == 1 || $NeoOptions(totebgo) == 1} then {
  if {$DataSourceOptions(tapered,Ge20) == 1 || $DataSourceOptions(tapered,Ge4) == 1} then {
     NEO "//    look for the q1 (anti-compton) bit"
  if {$DataSourceOptions(tapered,Ge20) == 1 && $DataSourceOptions(tapered,Ge4) == 1} then {
     NEO "            if ((tapered(jj).ge\[j\].E4 & 16384) || (tapered(jj).ge\[j\].E20 & 16384)) \{"
  }
  if {$DataSourceOptions(tapered,Ge20) == 1 && $DataSourceOptions(tapered,Ge4) == 0} then {
     NEO "            if (tapered(jj).ge\[j\].E20 & 16384) \{"
  }
  if {$DataSourceOptions(tapered,Ge20) == 0 && $DataSourceOptions(tapered,Ge4) == 1} then {
     NEO "            if (tapered(jj).ge\[j\].E4 & 16384) \{"
  }
     NEO ""
     NEO "//    the q1 (anti-compton) bit is set"
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     NEO "                 anti_compton = TRUE;"
  }
  if {$NeoOptions(totebgo) == 1 && $DataSourceOptions(tapered,BGOE) == 1} then {
     NEO " //    We sum the Bgo anti-compton Energy converting from 8MeV range to 20MeV range"
     NEO "                 TotalBgoEnergy += (BgoEnergy * 8/20);    // We sum the Bgo anti-compton Energy"
  }
     NEO "            \}"
  }
  }

#    Section for a 20MeV Ge ADC

  if {$DataSourceOptions(tapered,Ge20) == 1} then {
     NEO "//    Section for a 20MeV Ge ADC"
     NEO ""
     NEO "            E20 = tapered(jj).ge\[j\].E20 & 8191;   // extract energy from 13 bit adc"
     NEO "            if (E20) \{"
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
  if {$DataHistOptions(tapered,Ge20) == 1} then {
     NEO "              T_E20\[keyDet\].inc(E20);     //    histogram data (raw or clean)"
  }      
  if {$NeoCardOptions(tapered,gm) == 1} then {
     NEO "              rnd++;rnd = rnd % 1000;     //  apply gain match correction"
     NEO "              E20 = (coeff20_a\[keySpec+209\]*(E20+rnd/1000.0)+coeff20_b\[keySpec+209\])/2.5;"
  }
     NEO "              if (E20 > 5) \{"
  if {$NeoCardOptions(tapered,gmhist) == 1} then {
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "                T_E20_GM\[keyDet\].inc(E20);     //    histogram gain matched data"
  }
  if {$NeoOptions(tote) == 1} then {
     NEO "                tot_20MeV += E20;           // to increment the total spectrum "
  }
     NEO "              \}"
     NEO "            \}"
  }

#    Section for a 4MeV Ge ADC

  if {$DataSourceOptions(tapered,Ge4) == 1} then {
     NEO "//    Section for a 4MeV Ge ADC"
     NEO ""
     NEO "            E4 = tapered(jj).ge\[j\].E4 & 8191;   // extract energy from 13 bit adc"
     NEO "            if (E4) \{"
  if {$DataHistOptions(tapered,Ge4) == 1} then {
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     NEO "            if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "            T_E4\[keyDet\].inc(E4);     //    histogram data (raw or clean)"
  }
  if {$NeoCardOptions(tapered,gm) == 1} then {
     NEO "              rnd++;rnd = rnd % 1000;     //  apply gain match correction"
     NEO "              E4 = 2*(coeff_a\[keySpec+209\]*(E4+rnd/1000.0)+coeff_b\[keySpec+209\]);"
  }
     NEO "              if (E4 > 5) \{"
  if {$NeoCardOptions(tapered,gmhist) == 1} then {
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     NEO "                if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "                T_E4_GM\[keyDet\].inc(E4);     //    histogram gain matched data"  
  }
     NEO "              \}"
     NEO "            \}"
     NEO ""
  }

#    Section for a Ge TAC

  if {$DataSourceOptions(tapered,FT) == 1} then {
     NEO "//    Section for a Ge TAC"
     NEO ""
     NEO "            ft = tapered(jj).ge\[j\].ft & 8191;   // extract data from 13 bit adc"
     NEO "            if (ft) \{"
  if {$DataHistOptions(tapered,FT) == 1} then {
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     NEO "              if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "              T_ft\[keyDet\].inc(ft);     //    histogram data (raw or clean)"
  }
     NEO "            \}"
     NEO ""
  }

#    Section for a Ge CrossOver TAC

  if {$DataSourceOptions(tapered,CO) == 1} then {
     NEO "//    Section for a Ge CrossOver TAC"
     NEO ""
     NEO "            co = tapered(jj).ge\[j\].co & 8191;"
     NEO "            if (co) \{"
  if {$DataHistOptions(tapered,FT) == 1} then {
  if {$NeoCardOptions(tapered,acrej) == 1} then {
     NEO "              if (anti_compton == FALSE)   //  ignore if anti-compton set"
  }
     NEO "              T_co\[keyDet\].inc(co);     //    histogram data (raw or clean)"
  }
     NEO "            \}"
     NEO ""
  }

     NEO "            if (E4 || E20 || BgoEnergy) valid = TRUE;   // data is valid if an energy item is present"
     NEO "         \}      // end of subdetector fired"
     NEO "      \}         // end of all subdetectors"
     NEO ""
     NEO "      if(valid) \{"
     NEO "          T_stat\[2\].inc(keyDet);       //  increment statistics histogram for good event"
     NEO "          jj++;"
     NEO "      \} else tapered(jj).invalidate();   // this detector data is not written to tape"
     NEO ""
     NEO "      \}     //  end of detector"
     NEO "   \}        //  end of all detectors"

}


