


#    for trigger detectors a list of data sources
#    Note - these must be in order of Item number
set DataSources(trigger) [list SB1 SB2 SB3 SB4 TAC PAT EVL EVM]

#    for trigger detectors a description of each data source for the choice widget
set DataSourceID(trigger,SB1)   "Sum Bus 1"
set DataSourceID(trigger,SB2)   "Sum Bus 2"
set DataSourceID(trigger,SB3)   "Sum Bus 3"
set DataSourceID(trigger,SB4)   "Sum Bus 4"
set DataSourceID(trigger,TAC)   "TAC"
set DataSourceID(trigger,PAT)   "Pattern"
set DataSourceID(trigger,EVL)   "Ev Num LSW"
set DataSourceID(trigger,EVM)   "Ev Num MSW"

#   for trigger the name of each register associated with a data source
set DataSourceReg(trigger,SB1)   "Sumbus1"
set DataSourceReg(trigger,SB2)   "Sumbus2"
set DataSourceReg(trigger,SB3)   "Sumbus3"
set DataSourceReg(trigger,SB4)   "Sumbus4"
set DataSourceReg(trigger,TAC)   "TAC"
set DataSourceReg(trigger,PAT)   "Pattern"
set DataSourceReg(trigger,EVL)   "EvNumLSB"
set DataSourceReg(trigger,EVM)   "EvNumMSB"

#    for trigger detectors the default state of the InUse option
set DataSourceOptions(trigger,SB1)   1
set DataSourceOptions(trigger,SB2)   1
set DataSourceOptions(trigger,SB3)   1
set DataSourceOptions(trigger,SB4)   1
set DataSourceOptions(trigger,TAC)   1
set DataSourceOptions(trigger,PAT)   1
set DataSourceOptions(trigger,EVL)   1
set DataSourceOptions(trigger,EVM)   1

#    for trigger detectors the default state of the Hist option
set DataHistOptions(trigger,SB1)   0
set DataHistOptions(trigger,SB2)   0
set DataHistOptions(trigger,SB3)   0
set DataHistOptions(trigger,SB4)   0
set DataHistOptions(trigger,TAC)   0
set DataHistOptions(trigger,PAT)   0
set DataHistOptions(trigger,EVL)   0
set DataHistOptions(trigger,EVM)   0

#    for trigger detectors the default state of the Tape option
set DataTapeOptions(trigger,SB1)   0
set DataTapeOptions(trigger,SB2)   0
set DataTapeOptions(trigger,SB3)   0
set DataTapeOptions(trigger,SB4)   0
set DataTapeOptions(trigger,TAC)   0
set DataTapeOptions(trigger,PAT)   0
set DataTapeOptions(trigger,EVL)   0
set DataTapeOptions(trigger,EVM)   0

set NeoCardOptionChoice(trigger) [list tp]

set NeoCardOptionID(trigger,tp)       "Suppress all output to tape"

set NeoCardOptions(trigger,tp)       1

#    hardware probe to check setup
   
proc probe_trigger {} {
   global env VXIServer DataSources DataSourceReg DataSourceOptions DataSourceID

  :frame "[self frame-name].probe" :label "[self frame-title]" :show-footer T  :resize F :toplevel T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]"

    :panel PANEL1 :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10
      :galley REPORT :rows 15 :cols 80 :deaf T

    :modify REPORT :strings "Looking for Trigger VXI Card data word status\n\n"
    :show T

    set num 0
    foreach item $DataSources(trigger) {
      set VXIregister /$VXIServer/TriggerMK2.RdOut.$DataSourceReg(trigger,$item).Enable
      set VXIdata [EG "register :name '$VXIregister' ?value"]
      if {$DataSourceOptions(trigger,$item) == 0} then {
          if {$VXIdata == 1} then {
             :modify REPORT :strings "data item $DataSourceID(trigger,$item) IS NOT defined to be In Use but IS enabled in the VXI hardware\n"
             incr num
          }
      }
      if {$DataSourceOptions(trigger,$item) == 1} then {
          if {$VXIdata == 0} then {
             :modify REPORT :strings "data item $DataSourceID(trigger,$item) IS defined to be In Use but IS NOT enabled in the VXI hardware\n"
             incr num
          }
      }
    }

#    some checks for things not used

   set VXIregister /$VXIServer/TriggerMK2.GIR.ROCR.EvNumData.Disable
   set VXIdata [EG "register :name '$VXIregister' ?value"]
   if {$VXIdata == 0} then {
      :modify REPORT :strings "Warning - data item TriggerMK2.GIR.ROCR.EvNumData IS enabled.\n"
      incr num
   }

    if {$num == 0} then {:modify REPORT :strings "\nYour NEO++ statements for this card are compatible with the hardware setup - Well done!"}
    if {$num != 0} then {:modify REPORT :strings "\nThere are $num inconsistences between your NEO++ statements for this card and the hardware setup"}
}


#    phase 1 code generation - declarations

proc neogen1_trigger {} {
    global DataSourceOptions DataHistOptions

#    generate subdetector definitions

    EVENTH "//    Trigger detector section generated [date]\n\n\n\n"

#    generate detector format

     EVENTH "format (MasterTrigger)"

    if {$DataSourceOptions(trigger,SB1) == 1} then {
       EVENTH "    item SumBus1;"
    } else {
       EVENTH "    dummy_item SumBus1;"
    }
    if {$DataSourceOptions(trigger,SB2) == 1} then {
       EVENTH "    item SumBus2;"
    } else {
       EVENTH "    dummy_item SumBus2;"
    }
    if {$DataSourceOptions(trigger,SB3) == 1} then {
       EVENTH "    item SumBus3;"
    } else {
       EVENTH "    dummy_item SumBus3;"
    }
    if {$DataSourceOptions(trigger,SB4) == 1} then {
       EVENTH "    item SumBus4;"
    } else {
       EVENTH "    dummy_item SumBus4;"
    }
    if {$DataSourceOptions(trigger,TAC) == 1} then {
       EVENTH "    item TAC;"
    } else {
       EVENTH "    dummy_item TAC;"
    }
    if {$DataSourceOptions(trigger,PAT) == 1} then {
       EVENTH "    item Pattern;"
    } else {
       EVENTH "    dummy_item Pattern;"
    }
    if {$DataSourceOptions(trigger,EVL) == 1} then {
       EVENTH "    item EvNumLSW;"
    } else {
       EVENTH "    dummy_item EvNumLSW;"
    }
    if {$DataSourceOptions(trigger,EVM) == 1} then {
       EVENTH "    item EvNumMSW;"
    } else {
       EVENTH "    dummy_item EvNumMSW;"
    }

     EVENTH "end_format"

     EVENTH "Detector<MasterTrigger, MASTER_TRIGGER> mastertrigger;"
}


proc neogen2_trigger {} {
   global DataSourceOptions DataHistOptions NeoOptions NeoCardOptions

     HISTH "//    Master Trigger section generated [date]\n\n\n"

#    generate histograms for online monitoring

   if {$DataSourceOptions(trigger,SB1) == 1 && $DataHistOptions(trigger,SB1) == 1} then {
       HISTH "Hist1D MT_SumBus1(256,\"MT_SumBus1\");"
   }
   if {$DataSourceOptions(trigger,SB2) == 1 && $DataHistOptions(trigger,SB2) == 1} then {
       HISTH "Hist1D MT_SumBus2(256,\"MT_SumBus2\");"
   }
   if {$DataSourceOptions(trigger,SB3) == 1 && $DataHistOptions(trigger,SB3) == 1} then {
       HISTH "Hist1D MT_SumBus3(256,\"MT_SumBus3\");"
   }
   if {$DataSourceOptions(trigger,SB4) == 1 && $DataHistOptions(trigger,SB4) == 1} then {
       HISTH "Hist1D MT_SumBus4(256,\"MT_SumBus4\");"
   }
   if {$DataSourceOptions(trigger,TAC) == 1 && $DataHistOptions(trigger,TAC) == 1} then {
       HISTH "Hist1D MT_TAC(4096,\"MT_TAC\");"
   }
   if {$DataSourceOptions(trigger,PAT) == 1 && $DataHistOptions(trigger,PAT) == 1} then {
       HISTH "Hist1D MT_Pattern(4096,\"MT_Pattern\");"
   }
   if {$DataSourceOptions(trigger,EVL) == 1 && $DataHistOptions(trigger,EVL) == 1} then {
       HISTH "Hist1D MT_EvNumLSW(4096,\"MT_EvNumLSW\");"
   }
   if {$DataSourceOptions(trigger,EVM) == 1 && $DataHistOptions(trigger,EVM) == 1} then {
       HISTH "Hist1D MT_EvNumMSW(4096,\"MT_EvNumMSW\");"
   }

     NEO "//    Master Trigger section generated [date]\n\n\n\n"
     NEO "    if (mastertrigger.howManyFired()) \{       //   there is trigger data in this event"
     NEO "        Astat.inc(75);       //  increment statistics histogram"

   if {$DataSourceOptions(trigger,SB1) == 1 && $DataHistOptions(trigger,SB1) == 1} then {
     NEO "            temp = mastertrigger(0).SumBus1 & 255;"
     NEO "            if (temp) T_t\[keyDet\].inc(temp); // increment histogram"
     NEO ""
   }
   if {$DataSourceOptions(trigger,SB2) == 1 && $DataHistOptions(trigger,SB2) == 1} then {
     NEO "            temp = mastertrigger(0).SumBus2 & 255;"
     NEO "            if (temp) T_t\[keyDet\].inc(temp); // increment histogram"
     NEO ""
   }
   if {$DataSourceOptions(trigger,SB3) == 1 && $DataHistOptions(trigger,SB3) == 1} then {
     NEO "            temp = mastertrigger(0).SumBus3 & 255;"
     NEO "            if (temp) T_t\[keyDet\].inc(temp); // increment histogram"
     NEO ""
   }
   if {$DataSourceOptions(trigger,SB4) == 1 && $DataHistOptions(trigger,SB4) == 1} then {
     NEO "            temp = mastertrigger(0).SumBus4 & 255;"
     NEO "            if (temp) T_t\[keyDet\].inc(temp); // increment histogram"
     NEO ""
   }
   if {$DataSourceOptions(trigger,TAC) == 1 && $DataHistOptions(trigger,TAC) == 1} then {
     NEO "            temp = mastertrigger(0).TAC & 4095;"
     NEO "            if (temp) T_t\[keyDet\].inc(temp); // increment histogram"
     NEO ""
   }
   if {$DataSourceOptions(trigger,PAT) == 1 && $DataHistOptions(trigger,PAT) == 1} then {
     NEO "            temp = mastertrigger(0).Pattern & 4095;"
     NEO "            if (temp) T_t\[keyDet\].inc(temp); // increment raw data histogram"
     NEO ""
   }
   if {$DataSourceOptions(trigger,EVL) == 1} then {
     NEO "            MTEvNumLSW = mastertrigger(0).EvNumLSW;"
   if {$DataHistOptions(trigger,EVL) == 1} then {
     NEO "            if (MTEvNumLSW) T_t\[keyDet\].inc(MTEvNumLSW); // increment raw data histogram"
     NEO ""
   }
   }
   if {$DataSourceOptions(trigger,EVM) == 1} then {
     NEO "            MTEvNumMSW = mastertrigger(0).EvNumMSW;"
   if {$DataHistOptions(trigger,EVM) == 1} then {
     NEO "            if (MTEvNumMSW) T_t\[keyDet\].inc(MTEvNumMSW); // increment raw data histogram"
     NEO ""
   }
   }
   if {$NeoCardOptions(trigger,tp) == 1} then {NEO "            mastertrigger(0).invalidate();"}
     NEO "   \}"

}


