# netvar.tcl
# Network variables controller window
# Revision History:
# Fri Nov 24 09:12:50 GMT 1995 SCL Daresbury Laboratory
# January 1996 A.Capaldi
#	display variable type
#

if {[info exists env(NetVarManager)] == 0} then {
   set resource eb02
} else {
   set resource $env(NetVarManager)
}
set manager $resource

set nvdebug 0
set started 0

set frame NV
set frametitle "Network Variables"
set icontitle "NetVars"

set username ""
set defaultuser ""

set anallist {}
set NVlist {}
set hostlist {}
set pidlist {}
set varlist {}
set typelist {}
set isarraylist {}

set curanal ""
set curhost ""
set curpid 0
set curvar ""
set curtype 0
set isarray ""
set curind 0
set curval 0.0
set newval 0.0


proc make_netvar_frame {} {
   global env debug defaultuser manager curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]" :resize F

      :panel PANEL1 :layout T :padx 5 :pady 5 :row-gap 8
         :layout H
         :button VIEW :label "View"
	 :text MANAGER :label "Manager Host:" :w 20 :valuex 14c :value $manager
	 :next-row T
         :button DEFUSER :label "Default User"
	 :text USER :label "User:" :w 20 :valuex 8c :value $defaultuser

      :panel PANEL2 :layout T :padx 5
         :layout V
         :message M1 :label "Running Analyses" :bold T
         :list ANALS :rows 5 :cols 35 :exclusive T
	 :text CURANAL :label "Current:" :w 20 :deaf T :inactive T :valuex 14c
         :message M2 :label "Analysis Hosts" :bold T
         :list HOSTS :rows 5 :cols 35 :exclusive T
	 :text CURHOST :label "Current:" :w 20 :deaf T :inactive T :valuex 14c
         :message M3 :label "Process Ids" :bold T
         :list PIDS :rows 5 :cols 35 :exclusive T
	 :text CURPID :label "Current:" :w 20 :deaf T :inactive T :valuex 14c
         :message M4  :label "Declared Variables" :bold T
         :list VARS :rows 5 :cols 35 :exclusive T
	 :text CURVAR :label "Current:" :w 20 :deaf T :inactive T :valuex 14c
	 :text TYPEVAR :label "Type:" :w 13 :deaf T :inactive T :valuex 14c
         :number INDEX :label "Array Index" :w 10 :min 0 :max 99999 :value $curind :inactive T :valuex 14c
         :text CURVAL :label "Current Value" :w 11 :value $curval :deaf T :inactive T :valuex 14c
         :text NEWVAL :label "New Value" :w 11 :value $newval :valuex 14c
         :row-gap 5
         :button SET :label "Set New Value"
	 :button REVIEW :label "ReView"

   :show T
}

proc display-orbix-error {call errno} {
   global manager
   set mess "Unknown error"
   if {$errno==100} then {set mess "Arguments wrong or variable not an array."}
   if {$errno==101} then {set mess "Problems binding to Orbix NetVarManager on host $manager."}
   if {$errno==102} then {set mess "Problems calling NetVarManager's methods."}
   display-error-report self "Attempting to $call." "Error $errno.\n" $mess
   return
   }

proc clear-anals {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   set anallist {}
   set NVlist {}
   :modify ANALS :append F :strings $anallist
}

proc clear-hosts {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   set hostlist {}
   :modify HOSTS :append F :strings $hostlist
}

proc clear-pids {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   set pidlist {}
   :modify PIDS :append F :strings $pidlist
}

proc clear-curs {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   set curanal ""
   set curhost ""
   set curpid 0
   set curvar ""
   set curtype ""
#   set curind 0
#   set isarray 0
}

proc clear-vars {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   set varlist {}
   set typelist {}
   set isarraylist {}
   :modify VARS :append F :strings $varlist
}

proc clear-index {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   set curind 0
   :modify INDEX :inactive F :value $curind :inactive T
}

proc clear-curval {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   set curval 0
   :modify CURVAL :inactive F :value $curval :inactive T
}

proc clear-newval {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   set newval 0
   :modify NEWVAL :value $newval
}

proc read-anals {manager user} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   set anallist {}
   set NVlist {}
   set pidslist {}
   set hostlist {}
   set pidlist {}

   set z [catch {orb "(netvar :manager '$manager' :user '$user' ?anals)"} NVlist]
   if {$z!=0} then {
      display-orbix-error "query running analyses" $NVlist
      return
   }

   set lll {}
   foreach a $NVlist {
      lappend lll [second $a]
   }
   set ll [lsort -ascii $lll]
   set p {}
   foreach a $ll {
      if {$a != $p} then {
         lappend anallist $a
      }
      set p $a
   }
}

proc get-hosts {user anal} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   set hostlist {}
   set pidslist {}
   foreach l $NVlist {
      if {[first $l] == $user && [second $l] == $anal} then {
         lappend pidslist [third $l]
         lappend hostlist [fourth $l]
      }
   }
}

proc get-pids {host} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   set i 0
   set pidlist {}
   foreach l $hostlist {
      if {$l == $host} then {
         lappend pidlist [lindex $pidslist $i]
      }
      incr i
   }
}

proc get-vars {manager user anal host pid} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   set varlist {}
   set typelist {}
   set isarraylist {}

   set z [catch {orb "(netvar :manager '$manager' :manager '$manager' :user '$user' :anal '$anal' :host '$host' :pid $pid ?variables)"} l]
   if {$z!=0} then {
      display-orbix-error "query declared variables" $l
      return
   }
   foreach a $l {
      lappend varlist [first $a]
      lappend typelist [second $a]
      lappend isarraylist [third $a]
   }
}

proc get-val {manager user anal host pid var isarray index} {
   set z [catch {orb "(netvar :manager '$manager' :user '$user' :anal '$anal' :host '$host' :pid $pid :name '$var' :isarray $isarray :index $index ?value)"} v]
   if {$z!=0} then {
      display-orbix-error "read variable value" $v
      return
   }
   return $v
}

proc set-val {manager user anal var val isarray index} {
   set z [catch {orb "(netvar :manager '$manager' :user '$user' :anal '$anal' :name '$var' :isarray $isarray :index $index :value '$val')"} v]
   if {$z!=0} then {
      display-orbix-error "write variable value" $v
      return
   }
}

proc bind-to-manager m {
   set z [catch {orb "(netvar :manager '$m' !bind)"} v]
   if {$z!=0} then {
      display-orbix-error "connect to manager" $v
   }
   return $z
}

proc display-anals {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   :modify ANALS :append F :strings $anallist
}

proc display-hosts {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   :modify HOSTS :append F :strings $hostlist
}

proc display-pids {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   :modify PIDS :append F :strings $pidlist
}

proc display-vars {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   :modify VARS :append F :strings $varlist
}

proc display-curs {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   :modify CURANAL :inactive F :value $curanal :inactive T
   :modify CURHOST :inactive F :value $curhost :inactive T
   :modify CURPID :inactive F :value $curpid :inactive T
   :modify CURVAR :inactive F :value $curvar :inactive T
   :modify TYPEVAR :inactive F :value $curtype :inactive T
}

proc display-curval {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   :modify CURVAL :inactive F :value $curval :inactive T
}

proc restart-display {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   global manager username

   clear-anals
   clear-hosts
   clear-pids
   clear-vars
   clear-curs
   clear-curval
   clear-index

   read-anals $manager $username
   display-anals
   display-curs

}

proc click-manager {v} {
   global manager
   set manager $v
   # force new binding
   if {![bind-to-manager $manager]} then {
      click-view
   }
}

proc click-user {v} {
   global username
   set username $v
   # force view
   click-view
}

proc click-defuser {} {
   global username defaultuser
   set username $defaultuser
   # force view
   :modify USER :value $username
   click-view
}

proc click-view args {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist

   if {$nvdebug} then {puts stderr "click-view $args"}

   set-busy
   if {[first $args]!=""} then {
      set tmpanal $curanal
      set tmphost $curhost
      set tmppid $curpid
      set tmpvar $curvar
      set tmpind $curind
   }
   restart-display
   if {[first $args]!=""} then {
      set curhost $tmphost
      set curpid $tmppid
      set curvar $tmpvar
      set curind $tmpind
   } else {
      set tmpanal [first $anallist]
   }
   click-anals $tmpanal

   clear-busy
}

proc click-anals {anal} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   global manager username

   if {$nvdebug} then {puts stderr "click-anals $anal"}

   set curanal $anal
   if {[lsearch $anallist $curanal]==-1} then {
      set curanal [first $anallist]
   }
   if {$curanal==""} then {
      return
   }
   get-hosts $username $curanal
   display-hosts

   display-curs
   click-hosts $curhost
}

proc click-hosts {host} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   global manager username

   if {$nvdebug} then {puts stderr "click-hosts $host"}

   set curhost $host
   if {[lsearch $hostlist $curhost]==-1} then {
      set curhost [first $hostlist]
   }
   if {$curhost==""} then {
      return
   }

   get-pids $curhost
   display-pids

   display-curs
   click-pids $curpid
}

proc click-pids {pid} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   global manager username

   if {$nvdebug} then {puts stderr "click-pids $pid"}

   set curpid $pid
   if {[lsearch $pidlist $curpid]==-1} then {
      set curpid [first $pidlist]
   }
   if {$curpid==0} then {
      return
   }

   get-vars $manager $username $curanal $curhost $curpid
   display-vars


   display-curs
   click-vars $curvar
}

proc click-vars {name} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   global manager username

   if {$nvdebug} then {puts stderr "click-vars $name"}

   clear-curval
   set curvar $name
   if {[lsearch $varlist $curvar]==-1} then {
      set curvar [first $varlist]
   }
   if {$curvar==""} then {
      set curtype ""
      set isarray 0
      :modify INDEX :inactive F :value 0 :inactive T
      return
   }

   set curtype [lindex $typelist [lsearch $varlist $curvar]]
   set isarray [lindex $isarraylist [lsearch $varlist $curvar]]

   if {$isarray==1} then {
      :modify INDEX :inactive F :value $curind
   } else {
      :modify INDEX :inactive F :value 0 :inactive T 
   }

   display-curs
   if {$curanal!="" && $curhost!="" && $curpid!=0 && $curvar!=""} then {
      set curval [get-val $manager $username $curanal $curhost $curpid $curvar $isarray $curind]
   }
   display-curval
}

proc click-index {index} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   if {$nvdebug} then {puts stderr "click-index $index"}

   set curind $index
   click-vars $curvar
}

proc click-newval {val} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist

   set newval $val 
}

proc click-set {v} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   global manager username

   if {$nvdebug} then {puts stderr "click-set $v"}

   set-val $manager $username $curanal $curvar $newval $isarray $curind
   click-vars $curvar
}

proc click-review args {
   click-view 1
}

proc click-default args {
   global nvdebug
   if {$nvdebug} then {puts stderr "click-default $args"}
}

proc start {} {
   global nvdebug curanal curhost curpid curvar curtype isarray curind curval newval
   global anallist NVlist pidslist hostlist pidlist varlist typelist isarraylist
   global manager defaultuser username started

   if {!$started} then {
      set defaultuser [first [exec /usr/bin/who am i]]
#      global env; set defaultuser $env(USER)
      set username $defaultuser
      make_netvar_frame
      set started 1
      click-review
   } else {open-frame}
}

proc click-quit {f} {close-frame $f}

proc finish args {destroy-frame}

