#  nwall.tcl--- Neutron Wall interface control frame
# 

set debug 0
set started 0

set testmode 0
set monitor 0

set frame nwall
set frametitle "Neutron Wall Interface Control Frame"
set icontitle "nwall"

set  modulename "nwall#1"

if {[info exists env(NWALLPCIserver)]} then {set resource $env(NWALLPCIserver)} \
    else {set resource pci1}

set PCIbus 0
set PCIslot 16

set gateselectchoices [list "FT pulse" "FT + user" "FT start" "Val3 pulse"]
set gatedelaychoices  [list "250 ns" "500 ns" "750 ns" "1000 ns" "1250 ns" "1500 ns" "1750 ns" "2000 ns"]
set gatewidthchoices  [list "250 ns" "500 ns" "750 ns" "1000 ns" "1250 ns" "1500 ns" "1750 ns" "2000 ns"]

set monitorachoices [list "Command Bus: Gate" "Command Bus: Busy" "Command Bus: DRDY" \
    "Master Trigger: FT" "Master Trigger Val3" "Error 1" "Error 2" "Veto"]
set monitorbchoices [list "Command Bus: Gate" "Command Bus: Busy" "Command Bus: DRDY" \
    "Master Trigger: FT" "Master Trigger Val3" "Error 1" "Error 2" "Veto"]

#    options
set gateselect 0
set gatedelay 0
set gatewidth 0

set monitora 0
set monitorb 0

#    conversion between widget name and register
set w2r(MONITORB) MonitorB_Select
set w2r(MONITORA) MonitorA_Select
set w2r(GATEWIDTH)    Gate_Width
set w2r(GATEDELAY)    Gate_Delay
set w2r(GATESELECT)   Gate_Select

set DataReadOutState -1


#    for driving the Euroball inspection line MPX

loadsource common.vxi.patch

set Inspection(MONITORA) DI1
set Inspection(MONITORB) DI2


#    list of menu items and associated actions for the ACTIONS menu button 

set actionlist {                                     \
   {"Redefine NWALL registers"     "define-module"}  \
   {"Stop NWALL"                   "click-halt"}     \
   {"Go NWALL"                     "click-go"}       \
   {"Reset NWALL"                  "click-reset"}    \
   {"Setup NWALL"                  "click-setup"}    \
   {"Flush NWALL"                  "click-flush"}    \
}


proc define-register {class name attributes} {
   global resource modulename
   set register /$resource/$modulename.$name
   set-footer "defining register $register"
   catch {EG "register :name '$register' :class '$class' !new"}
   if {$attributes != ""} then {EG "register :name '$register' :object '$attributes'"}
   clear-footer
}

proc write-register {name value} {
   global testmode resource modulename
   global monitor
   set register /$resource/$modulename.$name
   if {$monitor != 0} then {set-footer "writing $value to $register"}
   EG "register :name '$register' :value '$value'"
   if {$testmode == 1} then {insert-debug "writing $value to $register"}
   if {$monitor != 0} then {clear-footer}
}

proc read-register {name} {
   global testmode resource modulename
   global monitor
   set register /$resource/$modulename.$name
   if {$monitor != 0} then {set-footer "reading $register"}
   set value [EG "register :name '$register' ?value"]
   if {$testmode == 1} then {insert-debug "$register returned $value"}
   if {$monitor != 0} then {clear-footer}
   return $value
}

proc read-registers {name} {
   global testmode resource modulename
   global monitor
   set register /$resource/$modulename.$name
   if {$monitor != 0} then {set-footer "reading $register"}
   set value [EG "register :pattern '$register' ?names-and-values"]
   if {$testmode == 1} then {insert-debug "$register returned $value"}
   if {$monitor != 0} then {clear-footer}
   return $value
}

proc initialise-register {name} {
   global testmode resource modulename
   global monitor
   set register /$resource/$modulename.$name
   if {$monitor != 0} then {set-footer "initialising $register"}
   EG "register :name '$register' !initialise"
   if {$testmode == 1} then {insert-debug "initialising $register"}
   if {$monitor != 0} then {clear-footer}
}

proc rr {name} {read-register $name}
proc wr {name value} {write-register $name $value}

proc define-module {} {
  global resource modulename PCIbus PCIslot
  set-footer "deleting all NWALL registers"
  EG "register :pattern '/$resource/$modulename.*' !delete"

#    NWALL registers in area 2
  set PCIaddress [expr (($PCIbus << 8) + $PCIslot << 8) + 2]

  define-register xPCI ALTERA.CR "-a $PCIaddress -o 0x0 -w 32 -p rw"

  define-register xPCI Gate_Select "-a $PCIaddress -o 0x80 -w 32 -p rw -l2 -b0 -zd"
  define-register xPCI Gate_Width  "-a $PCIaddress -o 0x84 -w 32 -p rw -l3 -b0 -zd"
  define-register xPCI Gate_Delay  "-a $PCIaddress -o 0x88 -w 32 -p rw -l3 -b0 -zd"
  define-register xPCI MonitorA_Select  "-a $PCIaddress -o 0x8C -w 32 -p rw -l3 -b0 -zd"
  define-register xPCI MonitorB_Select  "-a $PCIaddress -o 0x90 -w 32 -p rw -l3 -b0 -zd"

  define-register xPCI CR       "-a $PCIaddress -o 0x94 -w 32 -p rw"
  define-register xPCI CR.bit0  "-a $PCIaddress -o 0x94 -w 32 -l1 -b0 -p rw"
  define-register xPCI CR.bit1  "-a $PCIaddress -o 0x94 -w 32 -l1 -b1 -p rw"

  define-register xPCI SR       "-a $PCIaddress -o 0x9c -w 32 -p ro"
  define-register xPCI SR.bit0  "-a $PCIaddress -o 0x9c -w 32 -l1 -b0 -p ro"
  define-register xPCI SR.bit1  "-a $PCIaddress -o 0x9c -w 32 -l1 -b1 -p ro"
  define-register xPCI SR.bit2  "-a $PCIaddress -o 0x9c -w 32 -l1 -b2 -p ro"

#    NWALL software registers

  define-register generic  VRS.Continue        "-n VRSContinue -t int"
  define-register generic  VRS.LUT             "-n VRSLookupTable -t string -p ro"
  define-register generic  VRS.LUTDefaultItem  "-n VRSDefaultItem -t int"
  define-register generic  VRS.LUTDefaultGroup "-n VRSDefaultGroup -t int"

  define-register generic  VRS.Active          "-n VRSActive -t int"
  define-register generic  VRS.StatusBlock     "-n CSBlock_Status -t int -a 1 -i 3 -l 10 -p ro"
  define-register generic  VRS.Alive           "-n CSBlock_Status -t int -a 1 -i 3 -p ro"
  define-register generic  VRS.State           "-n CSBlock_Status -t int -a 1 -i 4 -p ro"
  define-register generic  VRS.EventNumber     "-n CSBlock_Status -t int -a 1 -i 5 -p ro"
# Scalers are:
#   validationsSeen, gateNoValidation, validationNoGate, atticMissedEvent, ADCMissedEvent
  define-register generic  VRS.Scalers         "-n CSBlock_Status -t int -a 1 -i 7 -l 5 -p ro"

  define-register TECioctl VRS.Reset "-d tec -b $PCIbus -s $PCIslot -p wo -r TEC_INT_BREAK"
  define-register TECioctl VRS.Flush "-d tec -b $PCIbus -s $PCIslot -p wo -r TEC_QUEUE_FLUSH"

  click-redisplay
}

proc define-configuration  {server bus slot} {
   global resource PCIbus PCIslot

    set resource $server
    set PCIbus $bus
    set PCIslot $slot
}

proc update-widget {widget value} {
   :modify $widget :value $value
}

proc read_VRSState {} {return [expr [read-register VRS.State] & [read-register VRS.Active]]}

proc click-reset {} {
#insert-debug "[date] click-reset"
    write-register VRS.Active 0
# if VRS was halted after an error tell it to continue (implicitly does a reset)
# otherwise just do the reset
    if {[read-register VRS.Continue]==0} then {
        write-register VRS.Continue 1
    } else {
        write-register VRS.Reset 0
    }
}

proc click-setup {} {
#insert-debug "[date] click-setup"
}

proc click-flush {} {
#insert-debug "[date] click-flush"
}

proc click-halt {} {
#insert-debug "[date] click-halt"
    write-register VRS.Flush 0
#    write-register VRS.Active 0
   click-reset
}

proc click-go {} {
#insert-debug "[date] click-go"
    write-register VRS.Active 1
# if VRS was halted after an error tell it to continue (implicitly does a reset)
# otherwise just do the reset
    if {[read-register VRS.Continue]==0} then {
        write-register VRS.Continue 1
    } else {
        write-register VRS.Reset 0
    }
}

proc click-software {} {
   global  modulename
   inform [first [split $modulename #]]sftwre start
}

proc click-electronics {} {
#  display electronics sub frame
   global electronicsvisible electronicsframe
#   if {$electronicsvisible==0} then {make-electronics} else {open-frame $electronicsframe}
   click-redisplay
}

proc click-redisplay {} {
   global sr w2r
   global resource PCIbus PCIslot

    update-widget PCISERVER $resource
    update-widget PCIbus    $PCIbus
    update-widget PCISLOT   [lsearch [list 16 17 18 19] $PCIslot]

    foreach w {GATESELECT GATEWIDTH GATEDELAY MONITORA MONITORB} {
        update-widget $w [read-register $w2r($w)]
    }

    redisplaygateoptions

#    display  SR
     set sr [read-register SR]
     foreach bit {2 1 0} {update-widget NWALLSR$bit [expr $sr >> $bit & 1]}
}

proc redisplaygateoptions {} {
   global w2r

   set register $w2r(GATESELECT)
   switch [read-register $register] {
      0  -
      3   {
             :modify GATEWIDTH :inactive T :value -1
             :modify GATEDELAY :inactive T :value -1
          }
      1   {
             :modify GATEWIDTH :inactive F :value [read-register $w2r(GATEWIDTH)]
             :modify GATEDELAY :inactive T :value -1
          }
      2   {
             :modify GATEWIDTH :inactive F :value [read-register $w2r(GATEWIDTH)]
             :modify GATEDELAY :inactive F :value [read-register $w2r(GATEDELAY)]
          }
   }
}


proc click arg {
   global debug
   global PCIbus PCIslot resource Inspection
   global w2r actionlist

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]

   if {$c==":quit"} then {click-quit; return}
   switch $w {

      PCIBUS   {
          set PCIbus $v
          define-module
      }
      PCISLOT {
          set PCIslot [lindex [list 16 17 18 19] $v]
          define-module
      }
      PCISERVER {
          set resource $v
          define-module
      }

      GATESELECT -
      GATEDELAY -
      GATEWIDTH {
          set register $w2r($w)
          write-register $register $v
          update-widget  $w [read-register $register]
          redisplaygateoptions
      }

      MONITORA -
      MONITORB {
          set register $w2r($w)
          write-register $register $v
          connect-inspection $Inspection($w) nw
          update-widget  $w [read-register $register]
      }

      ACTIONS  {
          set action [lindex $actionlist $i]
          eval [second $action]
      }
      ENGINEERING       {click-$v}

      default   {insert-debug "$arg"}
   }
}


proc make-frame {} {
  global env PCIbus PCIslot resource
  global gateselectchoices gateselect
  global gatedelaychoices gatedelay
  global gatewidthchoices gatewidth
  global monitorachoices monitora
  global monitorbchoices monitorb
  global actionlist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/vme_module" \
      :icon-name "[self icon-title]" :resize F

   :panel MODULE :layout T :padx 10 :pady 10
      :layout H; :row-gap 10; :col-gap 20

      :text   PCISERVER :label "PCI server" :w 8 :value $resource
      :choice PCIBUS    :label "PCI bus"    :strings [list 0] :value $PCIbus
      :choice PCISLOT   :label "PCI slot"   :strings [list 16 17 18 19] :value [lsearch [list 16 17 18 19] $PCIslot]

   :panel SELECT :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 15
      :choice-stack GATESELECT :label "Gate select" :strings $gateselectchoices  :value $gateselect
      :choice-stack GATEDELAY  :label "Gate delay"  :strings $gatedelaychoices   :value $gatedelay
      :choice-stack GATEWIDTH  :label "Gate width"  :strings $gatewidthchoices   :value $gatewidth
     :next-row T
      :choice-stack MONITORA   :label "Monitor A source" :strings $monitorachoices :value $monitora
      :choice-stack MONITORB   :label "Monitor B source" :strings $monitorbchoices :value $monitorb

      :row-gap 10; :next-row T
      :message MA2 :label "NWALL Status Register"  :bold T
# :valuex 22c
      :checkbox NWALLSR0  :label "V785 Data Ready"     :deaf  T :valuex 15c
      :checkbox NWALLSR1  :label "V785 Busy"  :deaf  T :valuex 15c
      :checkbox NWALLSR2  :label "VXI Inhibit Action" :deaf  T :valuex 20c

   :panel CLICK :layout T :padx 10 :pady 10
      :layout H; :row-gap 15; :col-gap 15

      :button REDISPLAY :label "Redisplay"   :command "click-redisplay"

      :menubutton ENGINEERING :label "Engineering"  :menu [list electronics software]

      set menu {}
      foreach item $actionlist {lappend menu [first $item]}
      :menubutton ACTIONS     :label "More Actions" :strings $menu

      :button HELP

   :show T
}


proc start args {
   global started monitor
   if {$started == 0} then {
      make-frame
      set started 1
      set monitor 0
      click-redisplay
   } else {open-frame}
}

proc autostart {} {start; close-frame}

proc click-quit {} {close-frame}

proc finish args {destroy-frame}

