
set dratedef(50) "dead events"

set dratedef(60) "total events"
set dratedef(61) "good events"
set dratedef(62) "suspected events"
set dratedef(63) "events to tape"

set dratedef(71) "event contains cluster data"
set dratedef(72) "event contains clover data"
set dratedef(73) "event contains tapered data"
set dratedef(74) "event contains bgo ball data"
set dratedef(75) "event contains Trigger data"
set dratedef(76) "event contains Euclides data"
set dratedef(77) "event contains Neutron Wall data"


set frame STATS
set title "System Statistics"
set icon SysStats

set dstatf  ""

#    NEO++  system level counters

set dratedef(1) "address error"
set dratedef(2) "bad vsn"
set dratedef(3) "bad converter"
set dratedef(4) "bad fera vdc"
set dratedef(5) "merging error"
set dratedef(6) "item mismatch"
set dratedef(7) "family error"
set dratedef(8) "counter error"
set dratedef(9) "bound error"
set dratedef(10) "item missing"
set dratedef(11) "duplicated val"
set dratedef(12) "fera SW not < 0"
set dratedef(13) "Silena subAddress error"
set dratedef(14) "no events in block"
set dratedef(15) "bad event format"
set dratedef(16) "bad fragment map"
set dratedef(17) "wrong adc value"
set dratedef(18) "wrong container"
set dratedef(19) "bad group"
set dratedef(20) "too many items"
set dratedef(21) "bad subdetector"
set dratedef(22) "transmission problems"


proc make-frame {} {
   global dratedef dstatdata dstatf drateperiod
   
   set col1 200

   :frame [self frame-name] :label "Euroball Data Quality" :show-footer T :resize F
   
   :panel PANEL1 :layout T :padx 5 :pady 5
      :layout H; :row-gap 5; :col-gap 20
      
         :text DSTATF :label "Classification Spectrum"  :value $dstatf :valuex 24c :w 36
         :next-row T
         :text DSTATP :label "Update period (secs)"  :value $drateperiod :valuex 24c :w 8

   :panel PANEL2 :layout L :padx 5 :pady 5
      :layout H; :row-gap 5; :col-gap 20      
         for {set i 0} {$i <= 59} {incr i} {
            if {![info exists dratedef($i)]} then {continue}
            set dstatdata($i) 0
            :text L$i  :label $dratedef($i) :valuex $col1 :w 12 :deaf T
            :text R$i  :label ""  :w 8  :deaf T
            :next-row T
         }
         
   :panel PANEL3 :layout L :padx 5 :pady 5
      :layout H; :row-gap 5; :col-gap 20  
         for {set i 60} {$i <= 255} {incr i} {
            if {![info exists dratedef($i)]} then {continue}
            set dstatdata($i) 0
            :text L$i  :label $dratedef($i) :valuex $col1 :w 12 :deaf T
            :text R$i  :label ""  :w 8  :deaf T
            :next-row T
         }  
           
   :show T
   
   
         
}

proc update-dstats {} {
  global dstatf dratedef dstatdata 

  if {$dstatf == "[file join /softhist Astat]"} then {
     set dstatf [file join /softhist [send evFarm inquire job_name] Astat]
     :modify DSTATF :value $dstatf
  }
  if {$dstatf == "[file join /softhist Astat]"} then {
      set-footer "no event processor farm job known"
      return 1
  }

  set z [catch {exec eg-to-ascii -spectrum $dstatf -format y} c]
  if {$z != 0} then {
      set-footer "unable to access $dstatf"
      return 1
  }
  set l [lrange [split $c \n] 6 end]
  
  for {set i 0} {$i <= 255} {incr i} {
     if {![info exists dratedef($i)]} then {continue}
     set vl [lindex $l $i]
     :modify L$i :value $vl
     set dstatdata($i) $vl
  }
  return 0
}

set drateperiod 30
set dratesactive 0

proc update-drates {} {
  global  dratedef dstatdata drateperiod dratesactive
  
  if {$dratesactive == 0} then {return}
  if {$drateperiod <= 0} then {return}
  
  if {[winfo ismapped .]} then {

     for {set i 0} {$i <= 255} {incr i} {
        if {![info exists dratedef($i)]} then {continue}
        set old($i) $dstatdata($i)
     }
     if {[update-dstats] == 0 } then {
        for {set i 0} {$i <= 255} {incr i} {
           if {![info exists dratedef($i)]} then {continue}
           set vl [expr ($dstatdata($i) - $old($i)) / $drateperiod]
           :modify R$i :value $vl
        }
        set-footer "Updated [date]"
     }
  }
  after [expr $drateperiod * 1000] update-drates
}

proc start {} {
  global dratesactive drateperiod dstatf

  make-frame
  
  set dstatf [file join /softhist [send evFarm inquire job_name] Astat]
  :modify DSTATF :value $dstatf
  
  update-dstats
  after cancel update-drates
  after [expr $drateperiod * 1000] update-drates
  set dratesactive 1
}

proc stop-drates {} {
  global dratesactive
  set dratesactive 0
}


#    procedures to handle user option input

proc click-dstatf {v} {
   global dstatf
   set dstatf $v
}

proc click-dstatp {v} {
   global drateperiod
   set drateperiod $v
}

