#    sys.tcl     Euroball Version   /euroball/tcl/sys.tcl
#    sys.tcl	Euroball sbg version /MIDAS/sbg/tcl
#####################################################################
#               Modifications 23/05/1997
#
# procFarm           --> evFarm
# start-neo_spy      --> start-spyProcessors
# start-neo_analysis --> start-evProcessors
#
#####################################################################
set debug 0
set started 0

set frame SYS
set frametitle "Euroball Experiment Control"
set icontitle  "ExptCntl"

set autoperiod 5000
set autocount 10

set tapestreams 1
set tapedrives 4

set showcontrols 0


if {[exec hostname] == "eb63"} then {
   set vxireq 1
   set camacreq 0
   set vmereq 0
   set ecreq 1
   set epreq 1
   set spreq 0
   set tapereq 0
} else {
   set vxireq 0
   set camacreq 0
   set vmereq 0
   set ecreq 0
   set epreq 0
   set spreq 0
   set tapereq 0
}

set vxistate 0
set camacstate 0
set vmestate 0
set ecstate 0
set epstate 0
set spstate 0
set tapestate 0

set EPstate {}
set SPstate {}


#    list of menu items and associated actions for the ACTIONS menu button 

set actionlist {                                                    \
   {"Show Main Menu"                    "show_main_menu"}           \
   {"Start new tape files"              "do_newrun 1"}              \
   {"Target Position"                   "inform target start"}      \
   {"NEO++ Code Generator"              "inform neogen start"}      \
   {"Toggle System Controls State"      "click-system"}             \
   {"Enable all EC FCS outputs"         "click-FCS 1"}              \
   {"Disable all EC FCS outputs"        "click-FCS 0"}              \
}

proc show_main_menu {} {
   global env
#    see if security option enabled and check permissions if it is
   if {$env(MIDASSECURITY) != "disabled"} then {
      if {![inform security validate "bf-menu,c"]} then {
#    user does not have permission to open bf menu
         set msg "Access Denied\n\nA password is required to access the main menu and you have not supplied it"
         set z [catch {midas_dialog [self frame-name].warningDialog "Attempted Security Violation" \
           "$msg" "warning" 0 "OK" "supply password"} m]
         if {$z != 0} then {
            puts stderr "midas-warning: [self frame-name].warningDialog: $m"
            catch {destroy [self frame-name].warningDialog}
         } else {
            if {$m == 1} then {inform security start}
         }
         return "Access Denied"
      }
   }
   send bf open-frame
}

proc starttask {task} {
#    Check if it is necessary to pre-start server interface task

    set started [inform $task inquire started]
    if {$started == 0} then {
      send $task start
      send $task close-frame
    }
}

proc click-tapes {action} {

   starttask tape

#    set tape server state

   send tape click-egstate $action
   redisplay-tapes
   update_button_states
}

proc redisplay-tapes {} {
   global tapestreams tapestate showcontrols

   set-busy
   starttask tape

#    find number of tape data streams

   set n [send tape read-numstreams]
   if {$n != $tapestreams} then {
      set tapestreams $n
      make-frame
   }

#    find tape server state

   catch {set tapestate [send tape read-egstate]}
   if {$showcontrols == 1} then {:modify TAPE :value [expr $tapestate-1]}
   :modify TAPESTATE :value [expr $tapestate-1]

#    find data stream associations

   for {set stream 1} {$stream <= $tapestreams} {incr stream} {
       set assoc [send tape find-assoc-strings $stream]
       :modify TAPESTRM$stream :value "$assoc"
   }

   clear-busy
}

proc click-camac {action} {

   starttask camaccontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-reset}
    }

   set-footer "sending $p to CAMAC control task"
   set-busy
   set z [catch {send camaccontrol $p} m]
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}
   update_button_states
}

proc redisplay-camac {} {
   global camacstate showcontrols

   set-busy
   starttask camaccontrol

   catch {set camacstate [send camaccontrol read-camacstate]}
   if {$showcontrols == 1} then {:modify CAMAC :value $camacstate}
   :modify CAMACSTATE :value $camacstate

#    at present CAMAC state is unknown

   if {$showcontrols == 1} then {:modify CAMAC :value -1}
   :modify CAMACSTATE :colours "white black" :value 1
   set camacstate 0

   clear-busy
}

proc click-vme {action} {

   starttask vmecontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-setup}
      3    {set p DAQ-flush}
      4    {set p DAQ-reset}
    }

   set-footer "sending $p to VME control task"
   set-busy
   set z [catch {send vmecontrol $p} m]
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}
   update_button_states
}

proc redisplay-vme {} {
   global vmestate showcontrols

   set-busy
   starttask vmecontrol

   catch {set vmestate [send vmecontrol read-vmestate]}
   if {$showcontrols == 1} then {:modify VME :value $vmestate}

   if {$vmestate == -1} then {

#    at present VME state is unknown

      :modify VMESTATE :colours "white black" :value 1
      set vmestate 0
   } else {
      :modify VMESTATE :value $vmestate
   }

   clear-busy
}


proc click-vxi {action} {

   starttask vxicontrol

    switch $action {
      0    {set p DAQ-halt}
      1    {set p DAQ-go}
      2    {set p DAQ-flush}
      3    {set p DAQ-reset}
    }

   set-footer "sending $p to VXI control task"
   set-busy
   set z [catch {send vxicontrol $p} m]
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}
   redisplay-vxi
   update_button_states
   inform rates zero-rates
}

proc redisplay-vxi {} {
   global vxistate showcontrols

   set-busy
   starttask vxicontrol

   catch {set vxistate [send vxicontrol read-vxistate]}
   if {$showcontrols == 1} then {:modify VXI :value $vxistate}
   :modify VXISTATE :value $vxistate

   if {$vxistate == 0} then {
       :modify VXIPAUSE :label "CONTINUE"
   } else {
       :modify VXIPAUSE :label "PAUSE"
   }

   clear-busy
}

proc click-eventcollectors {action} {

   set-busy
   starttask ec

    switch $action {
      0    {set p stop-all}
      1    {set p start-all}
      2    {set p setup-all}
      3    {set p reset-all}
    }

   set-footer "sending $p to EC control task"
   set z [catch {send ec $p} m]
   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

   redisplay-eventcollectors
   update_button_states
}

proc reset-eventcollectors {} {
    click-eventcollectors 3
    click-eventcollectors 2
}

proc redisplay-eventcollectors {} {
   global ecstate showcontrols

   set-busy
   starttask ec

   catch {set ecstate [send ec EC-state Running]}
   if {$showcontrols == 1} then {:modify EC :value $ecstate}
   :modify ECSTATE :deaf F :value $ecstate :deaf T

   clear-busy
}

proc click-FCS {v} {

   set-busy
   starttask ec

   send ec set-DS $v
   send ec set-DD $v

   clear-busy
}

proc click-eventprocessors {action} {

   set-busy
   starttask evFarm

    switch $action {
      0    {set p ""}
      1    {set p "start-Processors"}
      2    {set p "load-Processors"}
      3    {set p "unload-Processors"}
    }
    
   set EPstate [send evFarm read-Farm_neo]
   if {$action == 3 && [lindex $EPstate 3] == "unload"} then {
       click-redisplay
       return
   }

   send evFarm open-frame

   if {$p != ""} then {
     set-footer "sending $p to Event Processor Farm control task"
     set z [catch {send evFarm $p} m]
   } else {set m ""}

   clear-busy

   if {$p == "load-Processors" && $m == "-1"} then {
      send evFarm click-load
      set m ""
      after 2000 click-redisplay
   }

   if {$m == ""} then {
       clear-footer
   } else {
       if {$m == "-1"} then {set m "An unknown error has occurred"}
       set-footer $m
   }

   send evFarm close-frame

   redisplay-eventprocessors
   update_button_states
}

proc redisplay-eventprocessors {} {
   global epstate EPstate showcontrols

   set-busy
   starttask evFarm

   set EPstate [send evFarm read-Farm_neo]
   if {[lindex $EPstate 3] == "start"} then {
       set epstate 1
   } else {
       set epstate 0
   }
   if {$showcontrols == 1} then {:modify EP :value $epstate}
   :modify EPSTATE :value $epstate

#    obtain current neo program name

   switch [lindex $EPstate 3] {
       start -
       stop -
       load      {set epprog [file join [lindex $EPstate 0] [lindex $EPstate 1]]}
       unload    {set epprog "No program loaded"}
       default   {set epprog "State unknown"}
   }
   :modify EPPROG :value $epprog

   clear-busy
}

proc click-spyprocessors {action} {

   set-busy
   starttask spyFarm

    switch $action {
      0    {set p ""}
      1    {set p "start-Processors"}
      2    {set p ""}
      3    {set p "unload-Processors"}
    }

   if {$p != ""} then {
     set-footer "sending $p to Spy Processor Farm control task"
     set z [catch {send spyFarm $p} m]
   } else {set m ""}

   clear-busy
   if {$m == ""} then {clear-footer} else {set-footer $m}

   redisplay-spyprocessors
   update_button_states
}

proc redisplay-spyprocessors {} {
   global spstate SPstate showcontrols

   set-busy
   starttask spyFarm

   set SPstate [send spyFarm read-Farm_neo]
   if {[lindex $SPstate 3] == "start"} then {
       set spstate 1
   } else {
       set spstate 0
   }
   if {$showcontrols == 1} then {:modify SP :value $spstate}
   :modify SPSTATE :value $spstate

#    obtain current neo program name

   switch [lindex $SPstate 3] {
       start -
       stop -
       load      {set spprog [file join [lindex $SPstate 0] [lindex $SPstate 1]]}
       unload    {set spprog "No program loaded"}
       default   {set spprog "State unknown"}
   }
   :modify SPPROG :value $spprog

   clear-busy
}

proc click-redisplay {} {
  global vxireq camacreq  vmereq ecreq epreq spreq tapereq

  foreach subsystem {VXI CAMAC VME EC EP SP TAPE} {
    set v [eval set [string tolower $subsystem]req]
       if {$v == 0} then {
         :modify [set subsystem]STATE :colours "white black" :value 0
       } else {
         :modify [set subsystem]STATE :colours "red green" :value 0
       }
  }

  update-rates

  :modify EPPROG    :value "unknown"
  :modify TAPESTRM1 :value "unknown"

  if {$vxireq == 1} then {redisplay-vxi}
  if {$camacreq == 1} then {redisplay-camac}
  if {$vmereq == 1} then {redisplay-vme}
  if {$ecreq == 1} then {redisplay-eventcollectors}
  if {$epreq == 1} then {redisplay-eventprocessors}
  if {$spreq == 1} then {redisplay-spyprocessors}
  if {$tapereq == 1} then {redisplay-tapes}

  update_button_states
}

proc update_button_states {} {
  global vxireq camacreq vmereq ecreq epreq spreq tapereq
  global vxistate camacstate vmestate ecstate epstate spstate tapestate
  global EPstate SPstate

#    if any one of the monitored subsystems is in the GO state then the
#    overall GO button should be disabled
  if {$vxireq == 1 && $vxistate == 1}     then {disable_go; return}
  if {$camacreq == 1 && $camacstate == 1} then {disable_go; return}
  if {$vmereq == 1 && $vmestate == 1}     then {disable_go; return}
  if {$ecreq == 1 && $ecstate == 1}       then {disable_go; return}
#  if {$epreq == 1 && $epstate == 1}       then {disable_go; return}
#  if {$spreq == 1 && $spstate == 1}       then {disable_go; return}
  if {$tapereq == 1 && $tapestate == 2}   then {disable_go; return}
  
#    So possibly OK to GO
#    Check some other required conditions

   if {$epreq == 1} then {
      if {[lindex $EPstate 3] == "unload"} then {disable_go; return}
   }
   
   if {$spreq == 1} then {
      if {[lindex $SPstate 3] == "unload"} then {disable_go; return}
   }

  :modify EBGO :colour green :inactive F
}


proc disable_go {} {
    :modify EBGO :colour white :inactive T
}

proc update-rates {} {
  global tapestreams tapedrives

    set rates [list 0 0 0 0 0 0 0 0]

    set-footer "Updating Rates"
    set names [inform rates inquire ScalerNames]
    set i 1
    foreach name $names {
       :modify RATE$i :label $name
       incr i
    }
    set rates [inform rates inquire triggerrates]
    foreach i {1 2 3 4 5 6 7 8} {
      :modify RATE$i :value [lindex $rates [expr $i - 1]]
    }

#    calculate deadtime -  warning this calculation is dependent on the trigger lca program

    set ftr1 [lindex $rates 0]
    set ftr2 [lindex $rates 1]
    set ftr3 [lindex $rates 2]
    set ftr4 [lindex $rates 3]
    set ft   [lindex $rates 4]

# DS modification

    set ftrtotal [expr $ftr4]
    set ftrlost  [expr $ftrtotal - $ft]

#    if {$ftrtotal > 0} then {
#       set deadtime [expr [expr $ftrlost * 100] / $ftrtotal]
#    } else {
#       set deadtime 0
#    }
#    :modify DEADTIME :value $deadtime


############## Modif CW 4/11/1999
#  if {$ftrtotal != 0  && $ft == 0} then {midas-warning "WARNING!!!! No Fast #Trigger!!!! \n"
#  break
#   }
#############################   



    set rates [inform rates inquire taperates]
    for {set i 1} {$i <= $tapestreams} {incr i} {
        :modify TAPERATE$i :value [lindex $rates [expr $i - 1]]
    }
    set used [inform rates inquire tapeusedpercent]
    for {set i 1} {$i <= $tapedrives} {incr i} {
        :modify TAPEUSED$i :value [lindex $used [expr $i - 1]]
    }
    clear-footer
}

proc auto-rates {} {
   global autoperiod autocount
   after $autoperiod auto-rates
   incr autocount
   if {![winfo ismapped .]} then {return}
   if {$autocount < 10} then {update-rates} else {set autocount 0; click-redisplay}
}

proc click-euroball {action} {
  global vxireq camacreq vmereq ecreq epreq spreq tapereq

   switch $action {
      0   {
          insert-log "Euroball Data Acquisition Stopped [date]"
          set-footer "Stopping Euroball Data Acquisition [date]"
          if {$vxireq == 1} then {click-vxi 0}
          if {$camacreq == 1} then {click-camac 0}
          if {$vmereq == 1} then {click-vme 0}
          if {$ecreq == 1} then {click-eventcollectors 0}
          if {$epreq == 1} then {click-eventprocessors 0}
          if {$spreq == 1} then {click-spyprocessors 0}
          if {$tapereq == 1} then {click-tapes 0}
          set-footer "Euroball Data Acquisition Stopped [date]"
         }
      1  {
          set-footer "Starting Euroball Data Acquisition [date]"
#    for GO perform checks on subsystem states
        if {$vxireq == 1} then {
#    check state of VXI system
          set state [inform vxicontrol VXI-ResetState]
          if {$state == 0} then {
             if {[midas-confirmation "One or more VXI crates have not been software reset"] == 0} \
                 then {clear-footer; return}
          }
          set state [inform vxicontrol VXI-RestoreState]
          if {$state == 0} then {
             if {[midas-confirmation "One or more VXI crates have not been software restored"] == 0} \
                 then {clear-footer; return}
          }
        }
        if {$ecreq == 1} then {
#    check state of EC system
          set state [send ec EC-state Configured]
          if {$state == 0} then {
             if {[midas-confirmation "One or more Event Collectors is not in the Configured state"] == 0} \
                 then {clear-footer; return}
          }
        }

          if {$vxireq == 1} then {click-vxi 2}
          if {$vmereq == 1} then {click-vme 3}
          if {$ecreq == 1} then {click-eventcollectors 1}
          if {$tapereq == 1} then {click-tapes 1}
          if {$spreq == 1} then {click-spyprocessors 1}
          if {$epreq == 1} then {click-eventprocessors 1}
          if {$vmereq == 1} then {click-vme 1}
          if {$camacreq == 1} then {click-camac 1}
          if {$vxireq == 1} then {click-vxi 1}
          set-footer "Euroball Data Acquisition Started [date]"
          insert-log "Euroball Data Acquisition Started [date]"
         }
   }
   click-redisplay
}

proc do_stop&go {} {
    insert-log "[date] Performing auto Stop and Go"
    click-euroball 0
    click-euroball 1
    insert-log "[date] Completed auto Stop and Go"
}

proc click-EBreset {} {
    global vxireq camacreq vmereq ecreq epreq spreq tapereq

#    allow the user to chicken out!

    if {![midas-confirmation "\n\
Are you sure you wish to RESET the data acquisition system?\n\n\
This will STOP data acquisition\n\
UNLOAD your neo++ program from the processor farm\n\
RESET all Event Collector hardware\n\
SETUP all Event Collectors\n\
RESET all VXI modules and VME ADCs etc\n\
and finally RELOAD the same neo++ program back into the processor farm\n\n\
This may take several minutes\n\
It is highly recommended if GOing the Data Acquisition System does not start to collect data.\n\n\
If you decide to continue please wait for the RESET sequence to complete and DO NOT touch the system while it is in progress.
"]} then {return}

    insert-log "Resetting Euroball Data Acquisition at [date]"

#    first stop everything
    insert-log "Euroball Data Acquisition Stopped [date]"
    set-footer "Stopping Euroball Data Acquisition [date]"
    if {$vxireq == 1} then {click-vxi 0}
    if {$camacreq == 1} then {click-camac 0}
    if {$vmereq == 1} then {click-vme 0}
    if {$ecreq == 1} then {click-eventcollectors 0}
    if {$epreq == 1} then {click-eventprocessors 0}
    if {$spreq == 1} then {click-spyprocessors 0}
    if {$tapereq == 1} then {click-tapes 0}
    
#      unload processor farm
    if {$epreq == 1} then {
       set-footer "Unloading Processor Farm"
       click-eventprocessors 3
    }
    
#      reset event collectors
    if {$ecreq == 1} then {
       set-footer "Reset Event Collectors"
       click-eventcollectors 3
    }
    
#      setup event collectors
    if {$ecreq == 1} then {
       set-footer "Setup Event Collectors"
       click-eventcollectors 2
    }

#       reset VME
    if {$vmereq == 1} then {
       set-footer "Resetting VME"
       click-vme 4
    }
    
#       reset VXI
    if {$vxireq == 1} then {
       set-footer "Resetting VXI"
       click-vxi 3
    }
    
#       Load processor farm
    if {$epreq == 1} then {
       set-footer "Loading Processor Farm"
       click-eventprocessors 2
    }
    
    insert-log "Euroball Data Acquisition System RESET and ready to GO at [date]"
    midas_dialog [self frame-name].resetDialog  \
      "System Reset Completion" "Euroball Data Acquisition System RESET and ready to GO at [date]" "" 0 "Ok"
}

proc click-vxipause {} {
   global vxistate
        switch $vxistate {
          1  {inform vxicontrol "Stop-VXI VXI"}
          0  {inform vxicontrol "Go-VXI  VXI"}
        }
    redisplay-vxi
    update_button_states
}

proc click-system {} {
    global showcontrols
    set showcontrols [expr $showcontrols + 1 & 1]
    make-frame
    click-redisplay
}


proc click arg {
        global debug
        global vxireq camacreq vmereq ecreq epreq spreq tapereq
        global actionlist

        if {$debug} then {insert-debug "click with argument \{$arg\}"}
        set f [lindex $arg 0]
        set c [lindex $arg 1]
        set w [lindex $arg 2]
        set v [lindex $arg 3]
        set i [lindex $arg 4]
        if {$c==":quit"} then {click-quit; return}
        switch -glob $w {
                VXI   {click-vxi $v}
                CAMAC {click-camac $v}
                VME   {click-vme $v}
                EC    {click-eventcollectors $v}
                EP    {click-eventprocessors $v}
                SP    {click-spyprocessors $v}

                TAPE  {click-tapes $v}
                SPECTRA {inform sb start; inform sd start}

                *REQ  {
                   eval set [string tolower $w] $v
                   click-redisplay
                }

                EBGO    {click-euroball 1}
                EBSTOP  {click-euroball 0}
                EBRESET {click-EBreset}

                SAVERESTORE {
                    inform [string tolower $v]saverestore start
                }

                ACTIONS  {
                    set action [lindex $actionlist $i]
                    eval [second $action]
                }

                default {insert-debug "sys:  click \{$arg\}"}
        }
}

proc start args {
   global started
   if {$started == 0} then {make-frame} else {open-frame}
   set started 1
   inform vxicontrol autostart
   inform vmecontrol autostart
   inform camaccontrol autostart
   if {[exec hostname] == "eb63"} then {
       inform rates start
       inform rates start-trig-rates
       inform rates start-tape-rates
       inform rates start-tapemonitor
       inform rates close-frame
   }
   click-redisplay
   auto-rates
}

proc click-quit args {close-frame; send bf click-quit}

proc finish args {destroy-frame}

proc frame-quit win  {
   catch {wm iconify [winfo toplevel $win]}
   wm withdraw [winfo toplevel $win]
   send bf click-quit
}

proc make-frame {} {
  global env tapestreams tapedrives showcontrols
  global vxireq camacreq vmereq ecreq epreq spreq tapereq
  global vxistate camacstate vmestate ecstate epstate spstate tapestate
  global actionlist

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/egarray" \
      :icon-name "[self icon-title]" :resize F

  if {$showcontrols == 1} then {
    :panel PANEL1 :layout R  :padx 15 :pady 25
     :layout V; :col-gap 10; :row-gap 10
      :choice VXI     :label "VXI Electronics:"   :strings [list Stop Go Flush Reset] :valuex 20c
      :choice CAMAC   :label "CAMAC Electronics:" :strings [list Stop Go Reset]       :valuex 20c
      :choice VME     :label "VME Electronics:"   :strings [list Stop Go Setup Flush Reset] :valuex 20c
      :choice EC      :label "Event Collectors:"  :strings [list Stop Go Setup Reset] :valuex 20c
      :choice EP      :label "Event Processors:"  :strings [list Stop Go Load Unload] :valuex 20c
      :choice SP      :label "Spy Processors:"    :strings [list Stop Go Load Unload] :valuex 20c
      :choice TAPE    :label "Tapes:"             :strings [list Stop Go Test]        :valuex 20c
  }

   :panel PANEL2A :layout L :padx 5 :pady 5
     :layout V; :col-gap 10; :row-gap 10
       :message MSG1A :label "Subsystems required:" :bold T
         :checkbox VXIREQ    :label "VXI Electronics"   :valuex 20c  :value $vxireq
         :checkbox CAMACREQ  :label "CAMAC Electronics" :valuex 20c  :value $camacreq
         :checkbox VMEREQ    :label "VME Electronics"   :valuex 20c  :value $vmereq
         :checkbox ECREQ     :label "Event Collectors"  :valuex 20c  :value $ecreq
         :checkbox EPREQ     :label "Event Processors"  :valuex 20c  :value $epreq
         :checkbox SPREQ     :label "Spy Processors"    :valuex 20c  :value $spreq
         :checkbox TAPEREQ   :label "Tapes"             :valuex 20c  :value $tapereq

   :panel PANEL2B :layout L :padx 5 :pady 5
     :layout V; :col-gap 10; :row-gap 6
       :message MSG1B :label "Open" :bold T
         :button VXICTRL      :label "  " :command "inform vxicontrol start"
         :button CAMACCTRL    :label "  " :command "inform camaccontrol start"
         :button VMECTRL      :label "  " :command "inform vmecontrol start"
        :vskip 2
         :button ECCTRL       :label "  " :command "inform ec start"
    #    :vskip -1
         :button EPCTRL       :label "  " :command "inform evFarm start"
    #    :vskip -1
         :button SPCTRL       :label "  " :command "inform spyFarm start"
    #    :vskip -1
         :button TAPECTRL     :label "  " :command "inform tape start"

   :panel PANEL2C :layout L :padx 5 :pady 5
     :layout V; :col-gap 10; :row-gap 10
#       :vskip 20
       :message MSGEB  :label "EUROBALL" :bold T :colour "-background black -foreground white"
       :button EBGO      :label "GO"     :w 10  :h 3 :colour green
       :button EBSTOP    :label "STOP"   :w 10  :h 3 :colour red
       :button EBRESET   :label "RESET"  :w 10  :h 2 :colour yellow
       :button VXIPAUSE  :label "PAUSE"  :w 10   :command "click-vxipause"


   :panel PANEL2D :layout L :padx 5 :pady 5
     :layout V; :col-gap 10; :row-gap 10
       :message MSG2 :label "Subsystem States:" :bold T
         :checkbox VXISTATE    :label "VXI Electronics"   :valuex 17c  :value $vxistate   :colours "white black" :deaf T
         :checkbox CAMACSTATE  :label "CAMAC Electronics" :valuex 17c  :value $camacstate :colours "white black" :deaf T
         :checkbox VMESTATE    :label "VME Electronics"   :valuex 17c  :value $vmestate   :colours "white black" :deaf T
         :checkbox ECSTATE     :label "Event Collectors"  :valuex 17c  :value $ecstate    :colours "white black" :deaf T
         :checkbox EPSTATE     :label "Event Processors"  :valuex 17c  :value $epstate    :colours "white black" :deaf T
         :checkbox SPSTATE     :label "Spy Processors"    :valuex 17c  :value $spstate    :colours "white black" :deaf T
         :checkbox TAPESTATE   :label "Tapes"             :valuex 17c  :value $tapestate  :colours "white black" :deaf T

   :panel PANEL3 :layout B :padx 5 :pady 5 :after FOOTER
     :layout H; :col-gap 10; :row-gap 10
      :text EPPROG  :label "Processor Farm Program"  :w 56 :deaf T :shifters T
#      :next-row T
#      :message MSGTAPEA :label "Tape System" :bold T
#      :row-gap 5
#      if {$tapestreams == 1} then {set state "F"} else {set state "T"}
#      :hskip 10
#      :button ENDEXPT :label "End Experiment" :command "do_endrun 1" :inactive $state
#      :button STARTEXPT  :label "Start Experiment"  :command "inform tape start"
#      :button NEXTRUN :label "Next Run" :command "do_newrun 1" :inactive $state
#      :next-row T
#      :message MSGTAPEB :label "Current Data Stream Associations"
      :next-row T
      :layout V
      for {set i 1} {$i <= $tapestreams} {incr i} {
        :text TAPESTRM$i :label "Stream $i:" :valuex 15c :w 50 :deaf T
      }

   :panel PANEL4 :layout B :padx 5 :pady 5 :after FOOTER
     :layout H; :col-gap 10; :row-gap 5
      :message MSG5 :label "Tape Stream Rates (Kbytes/sec)" :bold T
      for {set i 1} {$i <= $tapestreams} {incr i} {
        :text TAPERATE$i :label "Stream $i:" :w 6 :deaf T
      }
       :next-row T
      :message MSG6 :label "Tape Used (%)" :bold T
      for {set i 1} {$i <= $tapedrives} {incr i} {
        :text TAPEUSED$i :label "Drive $i:" :w 4 :deaf T
      }
       :next-row T
      :message MSG4 :label "Trigger Scaler Rates" :bold T
       :next-row T
       :text RATE1  :label "" :w 8 :valuex 8c :deaf T
       :text RATE2  :label "" :w 8 :valuex 7c :deaf T
       :text RATE3  :label "" :w 8 :valuex 7c :deaf T
       :text RATE4  :label "" :w 8 :valuex 7c :deaf T
       :next-row T
       :text RATE5  :label "" :w 8 :valuex 8c :deaf T
       :text RATE6  :label "" :w 8 :valuex 7c :deaf T
       :text RATE7  :label "" :w 8 :valuex 7c :deaf T
       :text RATE8  :label "" :w 8 :valuex 7c :deaf T
       :next-row T
#      :message MSG7 :label "Detector Dead Time (%)" :bold T
#       :next-row T
#      :hskip 50
#      :gauge DEADTIME :label "" :min 0 :max 100 :gauge-width 400 :gauge-height 10 :inactive T 
#      [:path DEADTIME].scale configure -troughcolor yellow -background blue -sliderlength 20

   :panel BUTTONS :layout B :padx 5 :pady 5 :after FOOTER
     :layout H; :col-gap 10; :row-gap 0
      :message MSG3 :label "Controls" :bold T
      :next-row T
      :row-gap 10
      :button VXISETUP  :label "Standard VXI Module Setup" :command "inform vxisetup start"
      :button HISTOGRAM :label "Histogrammer"  :command "inform hist start"
      :button SPECTRA   :label "Spectrum Viewer"
      :button PRINT     :label "Print Window"  :command "midas-capture window"
      :next-row T
      :button VXIRESTORE    :label "VXI Parameter Restore"  :command "inform vxirestore start"
      :button VXISAVE       :label "VXI Parameter Save"     :command "inform vxisave start"
      :menubutton SAVERESTORE  :label "Parameter Save/Restore" :strings [list CAMAC VME]
     :next-row T
      :col-gap 10
      :button REDISPLAY   :label "Redisplay"       :command click-redisplay
      :button STATISTICS  :label "Statistics"      :command "inform stats start"
      :button RATES       :label "Rates and Monitoring"  :command "inform rates start"

      set menu {}
      foreach item $actionlist {lappend menu [first $item]}
      :menubutton ACTIONS :label "More Actions"  :strings $menu

      :button HELP        :command click-help

   :show T
}

loadsource endrun+newrun
